/**
 * $Id$
 *
 *<center>
 *               Avocent Corporation. Proprietary Information.
 * \n<em>
 *      This software is supplied under the terms of a license agreement or
 *      nondisclosure agreement with Avocent Corporation, or one of its 
 *      affiliates, andaffiliates, and may not be used, disseminated, or 
 *      distributed except in accordance with the terms of that agreement.
 *
 *      3541 Gateway Boulevard Fremont, San Jose, CA 94538 U.S.A.
 *\n
 *                  US phone: 510-771-6100
 *
 *        Copyright &copy; 2001-2009 Avocent Corporation. All rights reserved.
 *</em> </center>
 *
 *  For aes_set_key(), please use following Copyright
 *
 *  FIPS-197 compliant AES implementation
 *
 *  Copyright (C) 2001-2004  Christophe Devine
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *----------------------------------------------------------------------------\n
 *  MODULES     Crypto Driver\n
 *----------------------------------------------------------------------------\n
 *  @file   aess_cryptodrv.c
 *  @brief  Crypt/Decrypt data
 */

/******************************************************************************
* Content
* ----------------
*   aes_set_key()                 - Calculate AES expansion key
*   rc4_set_key()                 - Calculate rc4 expansion key
*   aess_cryptodrv_reset()        - aess_cryptodrv_reset
*   aess_cryptodrv_cryptoeng()    - Crypto data
*   aess_cryptodrv_ioctl()        - Handl IOCTL commands
*   aess_cryptodrv_open()         - Open crypto device node
*   aess_cryptodrv_release()      - This function will be called when device node is closed. Set init flag to INIT_FAIL
*   aess_cryptodrv_init()         - Module init function. Initialize everything for crypto driver.
*   aess_cryptodrv_exit()         - Module exit function. Release everything for crypto driver.
******************************************************************************/

/******************************************************************************
 * Header File
 *****************************************************************************/
#define AESSCRYPTODRV_C
#include <linux/errno.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/cdev.h>
#include <linux/interrupt.h>
#include <asm/uaccess.h>
#include <asm/arch/hardware.h>
#include <asm/io.h>
#include <asm/cacheflush.h>
#include <linux/spinlock.h>
#include <linux/interrupt.h>
#include <asm/uaccess.h>
#include <asm/semaphore.h>
#include <linux/random.h>
#include <linux/init.h>
#include <linux/list.h>
#include <linux/slab.h>
#include <linux/sched.h>
#include <linux/wait.h>
#include <linux/crypto.h>
#include <linux/mm.h>
#include <linux/skbuff.h>
#include <linux/random.h>
#include <asm/scatterlist.h>
#include <linux/dma-mapping.h>
#include <asm/dma.h>
#include <linux/aess_debugfs.h>
#include <asm/arch/hw_regs.h>
#include "aess_cryptodrv.h"




/* Debugging Message */
#if 0
#define ASPEED_SCU_VIRTUAL  	IO_ADDRESS(ASPEED_SCU_BASE)

#define SCU_WR_PROT_KEY        	(ASPEED_SCU_VIRTUAL+0x00)
#define SCU_RESET_CTRL         	(ASPEED_SCU_VIRTUAL+0x04)
#define SCU_CLOCK_STOP_CTRL    	(ASPEED_SCU_VIRTUAL+0x0c)

#endif

/******************************************************************************
 * Constant value definitions
 *****************************************************************************/
 
/* Values for SCU_WR_PROT_KEY */
#define SCU_WR_KEY_EN          	 0x1688A8A8
#define SCU_WR_KEY_DIS         	 0x00000000

/* Bits of SCU_RESET_CTRL */
#define SCU_RESET_CRYPTO        	(1<<4)
#define SCU_EN_HAC_CLK        	(~(1<<13))

#define ENCRYPTION_MODE     0x00
#define DECRYPTION_MODE     0x01
#define CRYPTO_ENG_FLG      0x02
#define CRYPTO_ENG_IDLE     0x00

#define CRYP_DIS_INT              0x0000
#define CRYP_EN_INT               0x1000
#define CRYP_EN_RW                0x0000
#define CRYP_DIS_RW               0x0800
#define CRYP_EN_LOAD_CONTEXT      0x0000
#define CRYP_DIS_LOAD_CONTEXT     0x0400
#define CRYP_EN_SAVE_CONTEXT      0x0000
#define CRYP_DIS_SAVE_CONTEXT     0x0200
#define CRYP_AES_ALG              0x0000
#define CRYP_RC4_ALG              0x0100
#define CRYP_DECRYP               0x0000
#define CRYP_ENCRYP               0x0080
#define CRYP_AES_ECB_MODE         0x0000
#define CRYP_AES_CBC_MODE         0x0010
#define CRYP_AES_CFB_MODE         0x0020
#define CRYP_AES_OFB_MODE         0x0030
#define CRYP_AES_CTR_MODE         0x0040
#define CRYP_AES_128_KEY          0x0000
#define CRYP_AES_192_KEY          0x0004
#define CRYP_AES_256_KEY          0x0008
#define CRYP_INDEPENDENT_MODE     0x0000
#define CRYP_CASCADE_MODE_CFHS    0x0002
#define CRYP_CASCADE_MODE_HFCS    0x0003

#define RC4_EXPANSION_KEYSIZE        256
#define AES_128_EXPANSION_KEYSIZE    176
#define AES_192_EXPANSION_KEYSIZE    208
#define AES_256_EXPANSION_KEYSIZE    240
#define RC4_EXPANSION_KEYSIZE        256
#define AES_128_KEYSIZE              128
#define AES_192_KEYSIZE              192
#define AES_256_KEYSIZE              256

#define MAX_SOURCE_SIZE              0x800  
#define MAX_DESTINATION_SIZE         0x800 
#define MAX_CTXT_SIZE                0x110 //context buffer size
#define MAX_EXAPNSION_KEYSIZE        0x100 
#define KEY_OFFSET                    0x10
#define AES_CRYPTO_ALGORITHM           0x0
#define RC4_CRYPTO_ALGORITHM           0x1
    
/* semaphore */
struct semaphore aes_sem;

/*0:kmalloc, 1:dma_alloc_writecombine*/
#define KMALLOCATE_METHOD 0

#define ENABLE_DEBUG 1

/******************************************************************************
*   STRUCT       :  sCryptoStructType
******************************************************************************/
/**
 *  @brief   Contains all information of crypto data
 *
 *****************************************************************************/
typedef struct
{
  /**Crypto command. Please refer SOC datasheet to see the format of this command*/
	UINT32        u32Crypto_Cmd;

  /**Data length*/
	UINT32        u32Crypto_DLen;

  /**Expansion key size: 176|208|240*/
	UINT32        u32Crypto_EKeySize;

  /**Key size: 128, 192, 256*/
	UINT32        u32Crypto_KeySize;

	#if KMALLOCATE_METHOD
  /**Source data buffer, it can use virt_to_bus() calculate physical address*/
  volatile  	  UINT8 au8SData[MAX_SOURCE_SIZE]        __attribute__ ((aligned (8)));

  /**Destination data buffer, it can use virt_to_bus() calculate physical address*/
	volatile  	  UINT8 au8DData[MAX_DESTINATION_SIZE]  __attribute__ ((aligned (8)));

  /**Context buffer, it can use virt_to_bus() calculate physical address*/
	volatile  	  UINT8 au8Context[MAX_CTXT_SIZE] __attribute__ ((aligned (8)));
	#else
	/**Physical address source data buffer*/
	UINT32 u32SData_phyaddr;

  /**Physical address destination data buffer*/
	UINT32 u32DData_phyaddr;
	
	/**Physical address context buffer*/
	UINT32 u32Context_phyaddr;

  /**Source data poinnter*/
	volatile  	  UINT8 *au8SData;
	
	/**Destination data poinnter*/
	volatile  	  UINT8 *au8DData;
	
	/**Context poinnter*/
	volatile  	  UINT8 *au8Context;
	
	/**This value is used for address alignment if allocated address of *au8SData is not 8-bytes aligned*/
	UINT8 u8SData_addr_offset;
	
	/**This value is used for address alignment if allocated address of *au8DData is not 8-bytes aligned*/
	UINT8 u8DData_addr_offset;
	
	/**This value is used for address alignment if allocated address of *au8Context is not 8-bytes aligned*/
	UINT8 u8Context_addr_offset;
  #endif
  
  /**Crypto algorithm: AES | RC4*/
  UINT8         u8CryptoAlg;  
  
  /**Reserved for feature used*/
  UINT16         u16Reserved1;	
  
  /**Reserved for feature used*/
  UINT16        u16Reserved2;	
	
	/**Reserved for feature used*/
	void *        pReserved;
}sCryptoStructType;


/******************************************************************************
*   STRUCT       :  sRC4KeyType
******************************************************************************/
/**
 *  @brief   Contains data of RC4 key
 *
 *****************************************************************************/
typedef struct
{
    /** state array of RC4 Cipher*/
    UINT8 au8State[RC4_EXPANSION_KEYSIZE];

    /** x data of RC4 Cipher*/
    UINT8 u8X;

    /** y data of RC4 Cipher*/
    UINT8 u8Y;
} sRC4KeyType;

/******************************************************************************
*   STRUCT       :  sAESKeyType
******************************************************************************/
/**
 *  @brief   Contains data of AES key
 *
 *****************************************************************************/
typedef struct
{
    /** encryption round keys */
    UINT8 erk[MAX_EXAPNSION_KEYSIZE];     
    
    /** decryption round keys */
    UINT8 drk[MAX_EXAPNSION_KEYSIZE];     
    
    /** number of rounds */
    int nr;                               
}sAESKeyType;

sRC4KeyType G_sRC4Key;
sAESKeyType G_sAESKey;


#if KMALLOCATE_METHOD
sCryptoStructType *G_pHace = NULL;
#else
sCryptoStructType G_sHace;
sCryptoStructType *G_pHace=&G_sHace ;
#endif


/*Following is MACRO, Array, function to set AES expanstion key.*/
#define GET_UINT32_aes(n,b,i)                       \
{                                               \
    (n) = ( (UINT32) (b)[(i)    ] << 24 )       \
        | ( (UINT32) (b)[(i) + 1] << 16 )       \
        | ( (UINT32) (b)[(i) + 2] <<  8 )       \
        | ( (UINT32) (b)[(i) + 3]       );      \
}

/* round constants */

static const UINT32 RCON[10] =
{
    0x01000000, 0x02000000, 0x04000000, 0x08000000,
    0x10000000, 0x20000000, 0x40000000, 0x80000000,
    0x1B000000, 0x36000000
};
static const UINT32 FSb[256] =
{
    0x63, 0x7C, 0x77, 0x7B, 0xF2, 0x6B, 0x6F, 0xC5,
    0x30, 0x01, 0x67, 0x2B, 0xFE, 0xD7, 0xAB, 0x76,
    0xCA, 0x82, 0xC9, 0x7D, 0xFA, 0x59, 0x47, 0xF0,
    0xAD, 0xD4, 0xA2, 0xAF, 0x9C, 0xA4, 0x72, 0xC0,
    0xB7, 0xFD, 0x93, 0x26, 0x36, 0x3F, 0xF7, 0xCC,
    0x34, 0xA5, 0xE5, 0xF1, 0x71, 0xD8, 0x31, 0x15,
    0x04, 0xC7, 0x23, 0xC3, 0x18, 0x96, 0x05, 0x9A,
    0x07, 0x12, 0x80, 0xE2, 0xEB, 0x27, 0xB2, 0x75,
    0x09, 0x83, 0x2C, 0x1A, 0x1B, 0x6E, 0x5A, 0xA0,
    0x52, 0x3B, 0xD6, 0xB3, 0x29, 0xE3, 0x2F, 0x84,
    0x53, 0xD1, 0x00, 0xED, 0x20, 0xFC, 0xB1, 0x5B,
    0x6A, 0xCB, 0xBE, 0x39, 0x4A, 0x4C, 0x58, 0xCF,
    0xD0, 0xEF, 0xAA, 0xFB, 0x43, 0x4D, 0x33, 0x85,
    0x45, 0xF9, 0x02, 0x7F, 0x50, 0x3C, 0x9F, 0xA8,
    0x51, 0xA3, 0x40, 0x8F, 0x92, 0x9D, 0x38, 0xF5,
    0xBC, 0xB6, 0xDA, 0x21, 0x10, 0xFF, 0xF3, 0xD2,
    0xCD, 0x0C, 0x13, 0xEC, 0x5F, 0x97, 0x44, 0x17,
    0xC4, 0xA7, 0x7E, 0x3D, 0x64, 0x5D, 0x19, 0x73,
    0x60, 0x81, 0x4F, 0xDC, 0x22, 0x2A, 0x90, 0x88,
    0x46, 0xEE, 0xB8, 0x14, 0xDE, 0x5E, 0x0B, 0xDB,
    0xE0, 0x32, 0x3A, 0x0A, 0x49, 0x06, 0x24, 0x5C,
    0xC2, 0xD3, 0xAC, 0x62, 0x91, 0x95, 0xE4, 0x79,
    0xE7, 0xC8, 0x37, 0x6D, 0x8D, 0xD5, 0x4E, 0xA9,
    0x6C, 0x56, 0xF4, 0xEA, 0x65, 0x7A, 0xAE, 0x08,
    0xBA, 0x78, 0x25, 0x2E, 0x1C, 0xA6, 0xB4, 0xC6,
    0xE8, 0xDD, 0x74, 0x1F, 0x4B, 0xBD, 0x8B, 0x8A,
    0x70, 0x3E, 0xB5, 0x66, 0x48, 0x03, 0xF6, 0x0E,
    0x61, 0x35, 0x57, 0xB9, 0x86, 0xC1, 0x1D, 0x9E,
    0xE1, 0xF8, 0x98, 0x11, 0x69, 0xD9, 0x8E, 0x94,
    0x9B, 0x1E, 0x87, 0xE9, 0xCE, 0x55, 0x28, 0xDF,
    0x8C, 0xA1, 0x89, 0x0D, 0xBF, 0xE6, 0x42, 0x68,
    0x41, 0x99, 0x2D, 0x0F, 0xB0, 0x54, 0xBB, 0x16
};

/* reverse tables */
int KT_init = 1;

UINT32 KT0[256];
UINT32 KT1[256];
UINT32 KT2[256];
UINT32 KT3[256];
#define RT \
\
    V(51,F4,A7,50), V(7E,41,65,53), V(1A,17,A4,C3), V(3A,27,5E,96), \
    V(3B,AB,6B,CB), V(1F,9D,45,F1), V(AC,FA,58,AB), V(4B,E3,03,93), \
    V(20,30,FA,55), V(AD,76,6D,F6), V(88,CC,76,91), V(F5,02,4C,25), \
    V(4F,E5,D7,FC), V(C5,2A,CB,D7), V(26,35,44,80), V(B5,62,A3,8F), \
    V(DE,B1,5A,49), V(25,BA,1B,67), V(45,EA,0E,98), V(5D,FE,C0,E1), \
    V(C3,2F,75,02), V(81,4C,F0,12), V(8D,46,97,A3), V(6B,D3,F9,C6), \
    V(03,8F,5F,E7), V(15,92,9C,95), V(BF,6D,7A,EB), V(95,52,59,DA), \
    V(D4,BE,83,2D), V(58,74,21,D3), V(49,E0,69,29), V(8E,C9,C8,44), \
    V(75,C2,89,6A), V(F4,8E,79,78), V(99,58,3E,6B), V(27,B9,71,DD), \
    V(BE,E1,4F,B6), V(F0,88,AD,17), V(C9,20,AC,66), V(7D,CE,3A,B4), \
    V(63,DF,4A,18), V(E5,1A,31,82), V(97,51,33,60), V(62,53,7F,45), \
    V(B1,64,77,E0), V(BB,6B,AE,84), V(FE,81,A0,1C), V(F9,08,2B,94), \
    V(70,48,68,58), V(8F,45,FD,19), V(94,DE,6C,87), V(52,7B,F8,B7), \
    V(AB,73,D3,23), V(72,4B,02,E2), V(E3,1F,8F,57), V(66,55,AB,2A), \
    V(B2,EB,28,07), V(2F,B5,C2,03), V(86,C5,7B,9A), V(D3,37,08,A5), \
    V(30,28,87,F2), V(23,BF,A5,B2), V(02,03,6A,BA), V(ED,16,82,5C), \
    V(8A,CF,1C,2B), V(A7,79,B4,92), V(F3,07,F2,F0), V(4E,69,E2,A1), \
    V(65,DA,F4,CD), V(06,05,BE,D5), V(D1,34,62,1F), V(C4,A6,FE,8A), \
    V(34,2E,53,9D), V(A2,F3,55,A0), V(05,8A,E1,32), V(A4,F6,EB,75), \
    V(0B,83,EC,39), V(40,60,EF,AA), V(5E,71,9F,06), V(BD,6E,10,51), \
    V(3E,21,8A,F9), V(96,DD,06,3D), V(DD,3E,05,AE), V(4D,E6,BD,46), \
    V(91,54,8D,B5), V(71,C4,5D,05), V(04,06,D4,6F), V(60,50,15,FF), \
    V(19,98,FB,24), V(D6,BD,E9,97), V(89,40,43,CC), V(67,D9,9E,77), \
    V(B0,E8,42,BD), V(07,89,8B,88), V(E7,19,5B,38), V(79,C8,EE,DB), \
    V(A1,7C,0A,47), V(7C,42,0F,E9), V(F8,84,1E,C9), V(00,00,00,00), \
    V(09,80,86,83), V(32,2B,ED,48), V(1E,11,70,AC), V(6C,5A,72,4E), \
    V(FD,0E,FF,FB), V(0F,85,38,56), V(3D,AE,D5,1E), V(36,2D,39,27), \
    V(0A,0F,D9,64), V(68,5C,A6,21), V(9B,5B,54,D1), V(24,36,2E,3A), \
    V(0C,0A,67,B1), V(93,57,E7,0F), V(B4,EE,96,D2), V(1B,9B,91,9E), \
    V(80,C0,C5,4F), V(61,DC,20,A2), V(5A,77,4B,69), V(1C,12,1A,16), \
    V(E2,93,BA,0A), V(C0,A0,2A,E5), V(3C,22,E0,43), V(12,1B,17,1D), \
    V(0E,09,0D,0B), V(F2,8B,C7,AD), V(2D,B6,A8,B9), V(14,1E,A9,C8), \
    V(57,F1,19,85), V(AF,75,07,4C), V(EE,99,DD,BB), V(A3,7F,60,FD), \
    V(F7,01,26,9F), V(5C,72,F5,BC), V(44,66,3B,C5), V(5B,FB,7E,34), \
    V(8B,43,29,76), V(CB,23,C6,DC), V(B6,ED,FC,68), V(B8,E4,F1,63), \
    V(D7,31,DC,CA), V(42,63,85,10), V(13,97,22,40), V(84,C6,11,20), \
    V(85,4A,24,7D), V(D2,BB,3D,F8), V(AE,F9,32,11), V(C7,29,A1,6D), \
    V(1D,9E,2F,4B), V(DC,B2,30,F3), V(0D,86,52,EC), V(77,C1,E3,D0), \
    V(2B,B3,16,6C), V(A9,70,B9,99), V(11,94,48,FA), V(47,E9,64,22), \
    V(A8,FC,8C,C4), V(A0,F0,3F,1A), V(56,7D,2C,D8), V(22,33,90,EF), \
    V(87,49,4E,C7), V(D9,38,D1,C1), V(8C,CA,A2,FE), V(98,D4,0B,36), \
    V(A6,F5,81,CF), V(A5,7A,DE,28), V(DA,B7,8E,26), V(3F,AD,BF,A4), \
    V(2C,3A,9D,E4), V(50,78,92,0D), V(6A,5F,CC,9B), V(54,7E,46,62), \
    V(F6,8D,13,C2), V(90,D8,B8,E8), V(2E,39,F7,5E), V(82,C3,AF,F5), \
    V(9F,5D,80,BE), V(69,D0,93,7C), V(6F,D5,2D,A9), V(CF,25,12,B3), \
    V(C8,AC,99,3B), V(10,18,7D,A7), V(E8,9C,63,6E), V(DB,3B,BB,7B), \
    V(CD,26,78,09), V(6E,59,18,F4), V(EC,9A,B7,01), V(83,4F,9A,A8), \
    V(E6,95,6E,65), V(AA,FF,E6,7E), V(21,BC,CF,08), V(EF,15,E8,E6), \
    V(BA,E7,9B,D9), V(4A,6F,36,CE), V(EA,9F,09,D4), V(29,B0,7C,D6), \
    V(31,A4,B2,AF), V(2A,3F,23,31), V(C6,A5,94,30), V(35,A2,66,C0), \
    V(74,4E,BC,37), V(FC,82,CA,A6), V(E0,90,D0,B0), V(33,A7,D8,15), \
    V(F1,04,98,4A), V(41,EC,DA,F7), V(7F,CD,50,0E), V(17,91,F6,2F), \
    V(76,4D,D6,8D), V(43,EF,B0,4D), V(CC,AA,4D,54), V(E4,96,04,DF), \
    V(9E,D1,B5,E3), V(4C,6A,88,1B), V(C1,2C,1F,B8), V(46,65,51,7F), \
    V(9D,5E,EA,04), V(01,8C,35,5D), V(FA,87,74,73), V(FB,0B,41,2E), \
    V(B3,67,1D,5A), V(92,DB,D2,52), V(E9,10,56,33), V(6D,D6,47,13), \
    V(9A,D7,61,8C), V(37,A1,0C,7A), V(59,F8,14,8E), V(EB,13,3C,89), \
    V(CE,A9,27,EE), V(B7,61,C9,35), V(E1,1C,E5,ED), V(7A,47,B1,3C), \
    V(9C,D2,DF,59), V(55,F2,73,3F), V(18,14,CE,79), V(73,C7,37,BF), \
    V(53,F7,CD,EA), V(5F,FD,AA,5B), V(DF,3D,6F,14), V(78,44,DB,86), \
    V(CA,AF,F3,81), V(B9,68,C4,3E), V(38,24,34,2C), V(C2,A3,40,5F), \
    V(16,1D,C3,72), V(BC,E2,25,0C), V(28,3C,49,8B), V(FF,0D,95,41), \
    V(39,A8,01,71), V(08,0C,B3,DE), V(D8,B4,E4,9C), V(64,56,C1,90), \
    V(7B,CB,84,61), V(D5,32,B6,70), V(48,6C,5C,74), V(D0,B8,57,42)

#define V(a,b,c,d) 0x##a##b##c##d
static const UINT32 RT0[256] = { RT };
#undef V

#define V(a,b,c,d) 0x##d##a##b##c
static const UINT32 RT1[256] = { RT };
#undef V

#define V(a,b,c,d) 0x##c##d##a##b
static const UINT32 RT2[256] = { RT };
#undef V

#define V(a,b,c,d) 0x##b##c##d##a
static const UINT32 RT3[256] = { RT };
#undef V

#undef RT


/******************************************************************************
*   FUNCTION        :   aes_set_key
******************************************************************************/
/**
 *  @brief      Calculate AES expansion key
 *
 *  @return     0 for success
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function\n
 *
 *****************************************************************************/
int aes_set_key( 
                 /**AES expansion key buffer*/
                 sAESKeyType *ctx, 

                 /**Initial key*/
                 UINT8 *key, 
                 
                 /**Key size*/
                 int nbits,
                 
                 /**Crypto mode*/
                 UINT32 crypto_mode 
                 )
{
    int i;
    UINT32 *RK;
    #if 0
    UINT32 *SK;
    #endif

    switch( nbits )
    {
        case AES_128_KEYSIZE: ctx->nr = 10; break;
        case AES_192_KEYSIZE: ctx->nr = 12; break;
        case AES_256_KEYSIZE: ctx->nr = 14; break;
        default : return( 1 );
    }

    RK = (UINT32 *)ctx->erk;

    for( i = 0; i < (nbits >> 5); i++ )
    {
        GET_UINT32_aes( RK[i], key, i * 4 );
    }

    /* setup encryption round keys */

    switch( nbits )
    {
    case AES_128_KEYSIZE:

        for( i = 0; i < 10; i++, RK += 4 )
        {
            RK[4]  = RK[0] ^ RCON[i] ^
                        ( FSb[ (UINT8) ( RK[3] >> 16 ) ] << 24 ) ^
                        ( FSb[ (UINT8) ( RK[3] >>  8 ) ] << 16 ) ^
                        ( FSb[ (UINT8) ( RK[3]       ) ] <<  8 ) ^
                        ( FSb[ (UINT8) ( RK[3] >> 24 ) ]       );

            RK[5]  = RK[1] ^ RK[4];
            RK[6]  = RK[2] ^ RK[5];
            RK[7]  = RK[3] ^ RK[6];
        }
        break;

    case AES_192_KEYSIZE:

        for( i = 0; i < 8; i++, RK += 6 )
        {
            RK[6]  = RK[0] ^ RCON[i] ^
                        ( FSb[ (UINT8) ( RK[5] >> 16 ) ] << 24 ) ^
                        ( FSb[ (UINT8) ( RK[5] >>  8 ) ] << 16 ) ^
                        ( FSb[ (UINT8) ( RK[5]       ) ] <<  8 ) ^
                        ( FSb[ (UINT8) ( RK[5] >> 24 ) ]       );

            RK[7]  = RK[1] ^ RK[6];
            RK[8]  = RK[2] ^ RK[7];
            RK[9]  = RK[3] ^ RK[8];
            RK[10] = RK[4] ^ RK[9];
            RK[11] = RK[5] ^ RK[10];
        }
        break;

    case AES_256_KEYSIZE:

        for( i = 0; i < 7; i++, RK += 8 )
        {
            RK[8]  = RK[0] ^ RCON[i] ^
                        ( FSb[ (UINT8) ( RK[7] >> 16 ) ] << 24 ) ^
                        ( FSb[ (UINT8) ( RK[7] >>  8 ) ] << 16 ) ^
                        ( FSb[ (UINT8) ( RK[7]       ) ] <<  8 ) ^
                        ( FSb[ (UINT8) ( RK[7] >> 24 ) ]       );

            RK[9]  = RK[1] ^ RK[8];
            RK[10] = RK[2] ^ RK[9];
            RK[11] = RK[3] ^ RK[10];

            RK[12] = RK[4] ^
                        ( FSb[ (UINT8) ( RK[11] >> 24 ) ] << 24 ) ^
                        ( FSb[ (UINT8) ( RK[11] >> 16 ) ] << 16 ) ^
                        ( FSb[ (UINT8) ( RK[11] >>  8 ) ] <<  8 ) ^
                        ( FSb[ (UINT8) ( RK[11]       ) ]       );

            RK[13] = RK[5] ^ RK[12];
            RK[14] = RK[6] ^ RK[13];
            RK[15] = RK[7] ^ RK[14];
        }
        break;
    }
  #if 0//In aspeed, expansion keys for decryption and encryption are the same. 
       //So there is no need to run following codes.
   if (DECRYPTION_MODE==crypto_mode)
   {
      /* setup decryption round keys */

      if( KT_init )
      {
          for( i = 0; i < 256; i++ )
          {
              KT0[i] = RT0[ FSb[i] ];
              KT1[i] = RT1[ FSb[i] ];
              KT2[i] = RT2[ FSb[i] ];
              KT3[i] = RT3[ FSb[i] ];
          }

          KT_init = 0;
      }

      SK = ctx->drk;

      *SK++ = *RK++;
      *SK++ = *RK++;
      *SK++ = *RK++;
      *SK++ = *RK++;

      for( i = 1; i < ctx->nr; i++ )
      {
        RK -= 8;

        *SK++ = KT0[ (UINT8) ( *RK >> 24 ) ] ^
                KT1[ (UINT8) ( *RK >> 16 ) ] ^
                KT2[ (UINT8) ( *RK >>  8 ) ] ^
                KT3[ (UINT8) ( *RK       ) ]; RK++;

        *SK++ = KT0[ (UINT8) ( *RK >> 24 ) ] ^
                KT1[ (UINT8) ( *RK >> 16 ) ] ^
                KT2[ (UINT8) ( *RK >>  8 ) ] ^
                KT3[ (UINT8) ( *RK       ) ]; RK++;

        *SK++ = KT0[ (UINT8) ( *RK >> 24 ) ] ^
                KT1[ (UINT8) ( *RK >> 16 ) ] ^
                KT2[ (UINT8) ( *RK >>  8 ) ] ^
                KT3[ (UINT8) ( *RK       ) ]; RK++;

        *SK++ = KT0[ (UINT8) ( *RK >> 24 ) ] ^
                KT1[ (UINT8) ( *RK >> 16 ) ] ^
                KT2[ (UINT8) ( *RK >>  8 ) ] ^
                KT3[ (UINT8) ( *RK       ) ]; RK++;
      }

      RK -= 8;

      *SK++ = *RK++;
      *SK++ = *RK++;
      *SK++ = *RK++;
      *SK++ = *RK++;
    }
  #endif  
    return( 0 );
}
/******************************************************************************
*   FUNCTION        :   rc4_set_key
******************************************************************************/
/**
 *  @brief      Calculate rc4 expansion key
 *
 *  @return     None
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function\n
 *
 *****************************************************************************/
void rc4_set_key(
                        /** xRC4-structure key */
                        sRC4KeyType  *psKey,
                        
                        /** pointer to the key */
                        UINT8     *pu8Key,

                        /** length of the key */
                        UINT16    u16KeyLen
                   )
{
    UINT16   u16KeyIdx = 0;
    UINT16   u16StateIdx1 = 0;
    UINT16   u16StateIdx2 = 0;
    UINT8    *pu8State;
    UINT8    u8TempVal;    

    pu8State = &(psKey->au8State[0]);
    psKey->u8X = 0;
    psKey->u8Y = 0;

    while(u16StateIdx1 < RC4_EXPANSION_KEYSIZE)
    {
        pu8State[u16StateIdx1] = u16StateIdx1;
        u16StateIdx1++;
    }
    
    u16StateIdx1 =0;
    while(u16StateIdx1 < RC4_EXPANSION_KEYSIZE)
    {
      u8TempVal = pu8State[u16StateIdx1];
      u16StateIdx2 = (UINT8) (u16StateIdx2 + pu8State[u16StateIdx1] + pu8Key[u16KeyIdx]);
      pu8State[u16StateIdx1] = pu8State[u16StateIdx2];
      pu8State[u16StateIdx2] = u8TempVal;
      u16StateIdx1++;
      u16KeyIdx++;
      if( u16KeyLen <= u16KeyIdx )
           u16KeyIdx = 0;
    }
    return;
}




/* Linux device data structure */ 
dev_t aes_dev;
struct cdev *cryptodrv_cdev; 



/* driver name will passed by insmod command */
static char *driver_name="aess_cryptodrv";

/* Init flag */
static UINT8 S_u8CryptoInitFlag = INIT_FAIL;


/******************************************************************************
*   FUNCTION        :   aess_cryptodrv_reset
******************************************************************************/
/**
 *  @brief      Reset Crypto engine
 *
 *  @return     None
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Static function\n
 *
 *****************************************************************************/
static void aess_cryptodrv_reset(void)
{
    unsigned int u32Rst=0;
    
    /** reset CRYPTO module */
    iowrite32(SCU_WR_KEY_EN, SCU_WR_PROT_KEY);  /* Write enable key */
    
    u32Rst = ioread32(SCU_RESET_CTRL) | SCU_RESET_CRYPTO;/*Reset CRYPTO */
    iowrite32(u32Rst, SCU_RESET_CTRL);
    
    u32Rst = ioread32(SCU_RESET_CTRL) & (~SCU_RESET_CRYPTO);/* Stop reset CRYPTO */
    iowrite32(u32Rst, SCU_RESET_CTRL);
    
    u32Rst = ioread32(SCU_CLOCK_STOP_CTRL) & SCU_EN_HAC_CLK;/* Enable CRYPTO Clock*/
    iowrite32(u32Rst, SCU_CLOCK_STOP_CTRL);
    iowrite32(SCU_WR_KEY_DIS, SCU_WR_PROT_KEY);  /* Write Disable key*/

    #if KMALLOCATE_METHOD
    
    /** Set crypto base address*/
    iowrite32((virt_to_bus(&G_pHace->au8SData[0]))&0x0ffffff8,ASPEED_CRYPTO_CRY_SDATA_BA);//source data base address
    iowrite32((virt_to_bus(&G_pHace->au8DData[0]))&0x0ffffff8,ASPEED_CRYPTO_CRY_DDATA_BA);//destination data base address
    iowrite32((virt_to_bus(&G_pHace->au8Context[0]))&0x0ffffff8,ASPEED_CRYPTO_CRY_CBUF_BA);//context data base address 
    #else
    
    /** Set crypto base address*/
    iowrite32(G_pHace->u32SData_phyaddr,ASPEED_CRYPTO_CRY_SDATA_BA); /* source data physical base address */
    iowrite32(G_pHace->u32DData_phyaddr,ASPEED_CRYPTO_CRY_DDATA_BA); /* destination data physical base address */
    iowrite32(G_pHace->u32Context_phyaddr,ASPEED_CRYPTO_CRY_CBUF_BA);/* context data physical base address */
    #endif
}

/******************************************************************************
*   FUNCTION        :   aess_cryptodrv_cryptoeng
******************************************************************************/
/**
 *  @brief      Crypto data
 *
 *  @return     0 for success, -EINVAL for failure
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Static function\n
 *
 *****************************************************************************/
static int aess_cryptodrv_cryptoeng(

                                  /**Crypto information from user space*/
                                  sCryptoInfoType *psCryptoInfo,
                                  
                                  /**Crypto Mode*/
                                  UINT32 u32CryptoMode
                                  )
{
    int i;
    UINT8 *pu8InputText;
    UINT8 *pu8OutputText;
    UINT8 *pu8InputBlock;
    UINT8 *pu8OutputBlock;
    UINT8 *pu8InitVector;
    UINT8 *pu8InitKey;
    UINT32 u32Counter = 0;
    UINT32 u32DataLen = psCryptoInfo->u32DataLen;
 
    pu8InitKey = &psCryptoInfo->pu8InitKey[0];
    pu8InitVector = &psCryptoInfo->pu8InitVector[0];
    pu8InputText = &psCryptoInfo->pu8Input[0];
    pu8OutputText = &psCryptoInfo->pu8Output[0];
    
    #if ENABLE_DEBUG
    DRV_MSG("aess_cryptodrv_cryptoeng()\n");
    #endif

    /* get semaphore */
    if(down_interruptible(&aes_sem))
    {
        return -ERESTARTSYS;
    }


    
    /********** Prepare data before entering Crypto engine ******************************/
    if (MAX_SOURCE_SIZE < u32DataLen)
    {
        DRV_ERROR("Block size(0x%x) is larger than 0x%x\n",u32DataLen,MAX_SOURCE_SIZE);

        /* release semaphore */
        up(&aes_sem);
        return -EINVAL;
    }
    
    
    /* reset crypto commad */
    G_pHace->u32Crypto_Cmd = 0;
    
    if(AES_CRYPTO_ALGORITHM==psCryptoInfo->u8CryptoAlg)
    {
        /* check the key size setting */
        switch (psCryptoInfo->u16KeySize)
        {
            case AES_DRV_128_BITS:
                G_pHace->u32Crypto_Cmd |= CRYP_AES_128_KEY;
                G_pHace->u32Crypto_EKeySize = AES_128_EXPANSION_KEYSIZE;
                G_pHace->u32Crypto_KeySize = AES_128_KEYSIZE;
                #if ENABLE_DEBUG
                DRV_MSG("KEY SIZE is 128 bits\n");
                #endif
                break;
            case AES_DRV_192_BITS:
                G_pHace->u32Crypto_Cmd |= CRYP_AES_192_KEY ;
                G_pHace->u32Crypto_EKeySize = AES_192_EXPANSION_KEYSIZE;
                G_pHace->u32Crypto_KeySize = AES_192_KEYSIZE;
                #if ENABLE_DEBUG
                DRV_MSG("KEY SIZE is 192 bits\n");
                #endif
                break;
            case AES_DRV_256_BITS:
                G_pHace->u32Crypto_Cmd |= CRYP_AES_256_KEY ;
                G_pHace->u32Crypto_EKeySize = AES_256_EXPANSION_KEYSIZE;
                G_pHace->u32Crypto_KeySize = AES_256_KEYSIZE;
                #if ENABLE_DEBUG
                DRV_MSG("KEY SIZE is 256 bits\n");
                #endif
                break;
            default:
                DRV_ERROR("AES key size(0x%x) is not support\n",psCryptoInfo->u16KeySize);
                /* release semaphore */
                up(&aes_sem);
                
                /* Invalid argument */
                
                return -EINVAL;
        }
        
        switch(psCryptoInfo->u8Mode)
        {
            case AES_DRV_MODE_ECB:
                G_pHace->u32Crypto_Cmd |= CRYP_AES_ECB_MODE;
                #if ENABLE_DEBUG
                DRV_MSG("AES_DRV_MODE_ECB\n\n");
                #endif
                break;
        
            case AES_DRV_MODE_CBC:
                G_pHace->u32Crypto_Cmd |= CRYP_AES_CBC_MODE;

                DRV_MSG("Initial vector is \n");
                for (i = 0; i < 16; i += 4)
                {
                     //copy initial vector to context buffer
                     *((UINT32 *)(G_pHace->au8Context+i)) = *((UINT32 *)(pu8InitVector+i));  
                     DRV_MSG("%x",*((UINT32 *)(pu8InitVector+i)));
                }
                DRV_MSG("\n");

                #if ENABLE_DEBUG
                DRV_MSG("AES_DRV_MODE_CBC\n\n"); 
                #endif 
                break;

            case AES_DRV_MODE_CFB:
                #if ENABLE_DEBUG
                DRV_MSG("AES_DRV_MODE_CFB\n\n");
                #endif
                break;

            case AES_DRV_MODE_OFB:
                #if ENABLE_DEBUG
                DRV_MSG("AES_DRV_MODE_OFB\n\n");
                #endif
                break;

            case AES_DRV_MODE_CTR:
                #if ENABLE_DEBUG
                DRV_MSG("AES_DRV_MODE_CTR\n\n");
                #endif
                break;            

            default:
                DRV_ERROR("mode is not support\n");
                /* Invalid argument */
                
                /* release semaphore */
                up(&aes_sem);
                return -EINVAL;
        }
        
        /* Calculate expansion key by initkey */
        aes_set_key(&G_sAESKey, &pu8InitKey[0],G_pHace->u32Crypto_KeySize,u32CryptoMode);
        for (i=0;i<G_pHace->u32Crypto_EKeySize;i+=4)
        {
            /**For aspeed, expansion keys for decryption and encryption are the same.*/
            if(ENCRYPTION_MODE == u32CryptoMode || DECRYPTION_MODE == u32CryptoMode)
            {
                G_pHace->au8Context[KEY_OFFSET+i] = G_sAESKey.erk[i+3];
                G_pHace->au8Context[KEY_OFFSET+i+1] = G_sAESKey.erk[i+2];
                G_pHace->au8Context[KEY_OFFSET+i+2] = G_sAESKey.erk[i+1];
                G_pHace->au8Context[KEY_OFFSET+i+3] = G_sAESKey.erk[i];
            }else
            {
                DRV_ERROR("Crypto mode is not correct\n");
                
                /* release semaphore */
                up(&aes_sem);
                return -EINVAL;
            }
        }
        G_pHace->u32Crypto_Cmd |= CRYP_AES_ALG;
    }else if(RC4_CRYPTO_ALGORITHM==psCryptoInfo->u8CryptoAlg)
    {
         G_pHace->u32Crypto_EKeySize = RC4_EXPANSION_KEYSIZE;
         G_pHace->u32Crypto_KeySize = psCryptoInfo->u16KeySize;
         
         /* initialize index in context buffer */
         G_pHace->au8Context[8] =1;//index I
         G_pHace->au8Context[9] =0;//index J
         
         /*Get rc4 expansion key*/
         rc4_set_key(&G_sRC4Key, &pu8InitKey[0],G_pHace->u32Crypto_KeySize);
         for (i=0;i<G_pHace->u32Crypto_EKeySize;i++)
         {
             G_pHace->au8Context[KEY_OFFSET+i] =  G_sRC4Key.au8State[i];
         }
         G_pHace->u32Crypto_Cmd |= CRYP_RC4_ALG;
    }else
    {
       DRV_ERROR("1. Crypto algorithm[%d] is not correct\n",psCryptoInfo->u8CryptoAlg);
       
       /* release semaphore */
       up(&aes_sem);
       return -EINVAL;
    }/*end of if(AES_CRYPTO_ALGORITHM==psCryptoInfo->u8CryptoAlg)*/

    
    pu8InputBlock = pu8InputText;
    pu8OutputBlock = pu8OutputText;
            
    /*Copy Inputblock text*/
    for (i=0;i<u32DataLen;i+=4 )
    {
        *((UINT32 *)(G_pHace->au8SData+i)) = *((UINT32 *)(pu8InputBlock+i));
    }
    
    /* set data length to crytpo engine register */
    iowrite32(u32DataLen,ASPEED_CRYPTO_CRY_DATALEN);

    
    #if 0
    DRV_MSG("\nInput text is  %x%x%x%x%x%x%x%x%x%x%x%x%x%x%x%x\n\n",*((UINT32 *)&G_pHace->au8SData[0]),*((UINT32 *)&G_pHace->au8SData[4]),*((UINT32 *)&G_pHace->au8SData[8]),*((UINT32 *)&G_pHace->au8SData[12]),
    *((UINT32 *)&G_pHace->au8SData[16]),*((UINT32 *)&G_pHace->au8SData[20]),*((UINT32 *)&G_pHace->au8SData[24]),*((UINT32 *)&G_pHace->au8SData[28]),
    *((UINT32 *)&G_pHace->au8SData[32]),*((UINT32 *)&G_pHace->au8SData[36]),*((UINT32 *)&G_pHace->au8SData[40]),*((UINT32 *)&G_pHace->au8SData[44]),
    *((UINT32 *)&G_pHace->au8SData[48]),*((UINT32 *)&G_pHace->au8SData[52]),*((UINT32 *)&G_pHace->au8SData[56]),*((UINT32 *)&G_pHace->au8SData[60]));
    #endif
    
    #if ENABLE_DEBUG
    DRV_MSG("Source address is %x\n",ioread32(ASPEED_CRYPTO_CRY_SDATA_BA));
    DRV_MSG("destination address is %x\n",ioread32(ASPEED_CRYPTO_CRY_DDATA_BA));
    DRV_MSG("context buffer address is %x\n",ioread32(ASPEED_CRYPTO_CRY_CBUF_BA));
    DRV_MSG("Crypto data length is 0x%x\n",ioread32(ASPEED_CRYPTO_CRY_DATALEN));
    #endif
  
    #if ENABLE_DEBUG
    DRV_MSG("Init key is %x %x %x %x\n", *((UINT32 *)&pu8InitKey[0]), *((UINT32 *)&pu8InitKey[4]), *((UINT32 *)&pu8InitKey[8]), *((UINT32 *)&pu8InitKey[12]));
    DRV_MSG("pu32ContextData is  %x %x %x %x %x %x %x %x\n", *((UINT32 *)&G_pHace->au8Context[0]), *((UINT32 *)&G_pHace->au8Context[4]), *((UINT32 *)&G_pHace->au8Context[8]), *((UINT32 *)&G_pHace->au8Context[12]),
             *((UINT32 *)&G_pHace->au8Context[16]), *((UINT32 *)&G_pHace->au8Context[20]), *((UINT32 *)&G_pHace->au8Context[24]), *((UINT32 *)&G_pHace->au8Context[28]));
    #endif


    #if KMALLOCATE_METHOD
    flush_cache_all();
    #endif
    
    /********** Enter Crypto engine ******************************/
    if (CRYPTO_ENG_IDLE != (ioread32(ASPEED_CRYPTO_ST) & CRYPTO_ENG_FLG))
    {
    	  DRV_INFO("ECB engine is not ready, status is: %x\n",ioread32(ASPEED_CRYPTO_ST));
    	  DRV_INFO("0x0 %x\n",ioread32(ASPEED_CRYPTO_CRY_SDATA_BA));
        DRV_INFO("0x04 %x\n",ioread32(ASPEED_CRYPTO_CRY_DDATA_BA));
        DRV_INFO("0x8 %x\n",ioread32(ASPEED_CRYPTO_CRY_CBUF_BA));
        DRV_INFO("0xc %x\n",ioread32(ASPEED_CRYPTO_CRY_DATALEN));
        DRV_INFO("0x10 %x\n",ioread32(ASPEED_CRYPTO_CRY_CMD));
        DRV_INFO("0x1c %x\n",ioread32(ASPEED_CRYPTO_ST));

        /* reset H/W Crypto module */
        aess_cryptodrv_reset();
    }
    
    
    /*Disable crypto INT, Enable cryp read-in/write-out, En loading context data, En saving context data
      AES crypto ALG, Cryp engine works in independent mode.*/
    G_pHace->u32Crypto_Cmd |=(CRYP_DIS_INT | CRYP_EN_RW | CRYP_EN_LOAD_CONTEXT | CRYP_EN_SAVE_CONTEXT | 
                      CRYP_INDEPENDENT_MODE) ;
       
       
    if(ENCRYPTION_MODE==u32CryptoMode)
    {
        G_pHace->u32Crypto_Cmd |= CRYP_ENCRYP;
    }else if(DECRYPTION_MODE==u32CryptoMode)
    {
        G_pHace->u32Crypto_Cmd |= CRYP_DECRYP;
    }else
    {
        DRV_ERROR("Cryption mode(%x) is not correct\n",u32CryptoMode);
           
        /* release semaphore */
        up(&aes_sem);
        return -EINVAL;
    }
       
    /*Start crypto engine*/
    iowrite32(G_pHace->u32Crypto_Cmd,ASPEED_CRYPTO_CRY_CMD);
    
    #if ENABLE_DEBUG
    DRV_MSG("ASPEED_CRYPTO_CRY_CMD is %x\n",ioread32(ASPEED_CRYPTO_CRY_CMD));
    #endif
    
    while (1)
    {
        /* wait until process is finished */
            
        if(u32Counter == 100000)
        {
            DRV_ERROR("Decrypt/Encrypt data takes too much time\n");

            /* release semaphore */
            up(&aes_sem);
            return -EAGAIN;
        }

        if(CRYPTO_ENG_IDLE == (ioread32(ASPEED_CRYPTO_ST) & CRYPTO_ENG_FLG))
            break;

        u32Counter++;
    }

 
    #if 0   
    DRV_MSG("\ncipher text is  %x%x%x%x%x%x%x%x%x%x%x%x%x%x%x%x\n\n",*((UINT32 *)&G_pHace->au8DData[0]),*((UINT32 *)&G_pHace->au8DData[4]),*((UINT32 *)&G_pHace->au8DData[8]),*((UINT32 *)&G_pHace->au8DData[12]),
       *((UINT32 *)&G_pHace->au8DData[16]),*((UINT32 *)&G_pHace->au8DData[20]),*((UINT32 *)&G_pHace->au8DData[24]),*((UINT32 *)&G_pHace->au8DData[28]),
       *((UINT32 *)&G_pHace->au8DData[32]),*((UINT32 *)&G_pHace->au8DData[36]),*((UINT32 *)&G_pHace->au8DData[40]),*((UINT32 *)&G_pHace->au8DData[44]),
       *((UINT32 *)&G_pHace->au8DData[48]),*((UINT32 *)&G_pHace->au8DData[52]),*((UINT32 *)&G_pHace->au8DData[56]),*((UINT32 *)&G_pHace->au8DData[60]));
    
    
    DRV_MSG("Encryption is finished, Counter is %d\n",u32Counter);
    #endif

    
    /********** Exit Crypto Engine  ******************************/
    if(AES_CRYPTO_ALGORITHM==psCryptoInfo->u8CryptoAlg)
    {
        switch(psCryptoInfo->u8Mode)
        {
            case AES_DRV_MODE_ECB:

                for (i = 0; i < u32DataLen; i++)
                {
                    pu8OutputBlock[i]  = G_pHace->au8DData[i];
                }
                break;

            case AES_DRV_MODE_CBC:
            
                // copy output data to init vector 

                for (i = 0; i < u32DataLen; i++)
                {
                    pu8OutputBlock[i] = G_pHace->au8DData[i];
                }

                for (i =0 ; i < 16; i += 4)
                {
                    if(ENCRYPTION_MODE == u32CryptoMode)
                    {
                        *((UINT32 *) (pu8InitVector + i)) 
                        = *((UINT32 *) (&G_pHace->au8DData[(u32DataLen-16+i)]));
                    }else if(DECRYPTION_MODE == u32CryptoMode)
                    {
                      *((UINT32 *) (pu8InitVector + i)) 
                        = *((UINT32 *) (&G_pHace->au8SData[(u32DataLen-16+i)]));
                    }else
                    {
                       DRV_ERROR("CryptoMode is not correct after crypto engine");
                       
                       /* release semaphore */
                       up(&aes_sem);
                       return -EINVAL;
                    }    
                }
                break;
            case AES_DRV_MODE_CFB: 
                break;

            case AES_DRV_MODE_OFB: 
                break;

            case AES_DRV_MODE_CTR:  
                break;
            default:
                DRV_ERROR("AES operation mode is not support\n");
                
                /* release semaphore */
                up(&aes_sem);

                /* Invalid argument */
                return -EINVAL;
        }
    }else if(RC4_CRYPTO_ALGORITHM==psCryptoInfo->u8CryptoAlg)
    {

        for (i = 0; i < u32DataLen; i++ )
        {
            pu8OutputBlock[i] = G_pHace->au8DData[i];
        }
    }else
    {
    	  DRV_ERROR("4. Crypto algorithm[%d] is not correct\n",psCryptoInfo->u8CryptoAlg);

        /* release semaphore */
        up(&aes_sem);    	  
        return -EINVAL;
    }
    DRV_MSG("Exit aess_cryptodrv_cryptoeng()\n");
    
    /* release semaphore */
    up(&aes_sem);
    return 0;
}   


/******************************************************************************
*   FUNCTION        :   aess_cryptodrv_ioctl
******************************************************************************/
/**
 *  @brief      Handl IOCTL commands
 *
 *  @return     0 for success, -EPERM/-EFAULT for failure
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Static function\n
 *
 *****************************************************************************/
static int aess_cryptodrv_ioctl(
                               /** Inode is corresponding to the file descriptor fd
                                   passed on by the application. It is not used in this function */
                                 struct inode * inode, 
                                 
                               /** Inode is corresponding to the file descriptor fd
                                   passed on by the application. It is not used in this function */
                                 struct file *flip, 
                                 
                                 /**IOCTL command passed on by the application */
                                 unsigned int cmd, 
                                 
                                 /**Input parameter passed on by the application. The data type is struct sCryptoInfoType*/
                                 unsigned long arg
                                 )
{   
    int err_check;
    unsigned int u32CryptoMode = 0;
    sCryptoInfoType *psCryptoInfo = (sCryptoInfoType *) arg;
    
    if (S_u8CryptoInitFlag != INIT_OK)
    {
        /* Return Operation not permitted, need init first */
        return -EPERM;
    }
   
    /* Check the structure address is OK */
    if (!access_ok(VERIFY_WRITE, (void *) psCryptoInfo, sizeof(sCryptoInfoType)))
    {
        /* error occur */
        return -EFAULT;
    }
	  
    switch(cmd)
    {
        case AESS_CRYPTODRV_ENCRYPT:
     
            DRV_MSG("aess_cryptodrv_ioctl: AESS_CRYPTODRV_DECRYPT\n");
            u32CryptoMode = ENCRYPTION_MODE;
            
            break;
			
        case AESS_CRYPTODRV_DECRYPT:

            
            DRV_MSG("aess_cryptodrv_ioctl: AESS_CRYPTODRV_DECRYPT\n");
            u32CryptoMode = DECRYPTION_MODE;

            break;
        case AESS_CRYPTODRV_CRYPT:

            /* For RC4 algorithm, there is no difference between encryption and decryption.
               So we don't need to set value to u32CryptoMode*/
            DRV_MSG("aess_cryptodrv_ioctl: AESS_CRYPTODRV_CRYPT\n");
            
            break;
        default:        
            return (-EINVAL);
    }
    
    /* Pass the command to command handler */
    err_check = aess_cryptodrv_cryptoeng(psCryptoInfo,u32CryptoMode);

    /* 0->ok, minus->fail */
    return err_check;
}

/******************************************************************************
*   FUNCTION        :   aess_cryptodrv_open
******************************************************************************/
/**
 *  @brief      Open crypto device node
 *
 *  @return     0 for success, -EPERM for failure
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Static function\n
 *
 *****************************************************************************/
static int aess_cryptodrv_open(
                               /** Inode is corresponding to the file descriptor fd
                                   passed on by the application. It is not used in this function */
                               struct inode *inode, 
                                
                               /** Inode is corresponding to the file descriptor fd
                                   passed on by the application. It is not used in this function */
                               struct file *filp
                               )
{
    DRV_MSG("aess_cryptodrv_open\n");

    if (S_u8CryptoInitFlag != INIT_OK)
    {
        DRV_MSG("first time to open\n");
        
        // initiate semaphores 
        init_MUTEX(&aes_sem);

	      // reset Crypto module 
	      aess_cryptodrv_reset();
        S_u8CryptoInitFlag = INIT_OK;
    }else
    {
        return -EPERM;
    }    
    return 0;
}


/******************************************************************************
*   FUNCTION        :   aess_cryptodrv_release
******************************************************************************/
/**
 *  @brief      This function will be called when device node is closed. Set init flag to INIT_FAIL
 *
 *  @return     0
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Static function\n
 *
 *****************************************************************************/
static int aess_cryptodrv_release(
                               /** Inode is corresponding to the file descriptor fd
                                   passed on by the application. It is not used in this function */
                                 struct inode* inode, 
                                 
                               /** Inode is corresponding to the file descriptor fd
                                   passed on by the application. It is not used in this function */
                                 struct file *flip
                                 
                                 )
{
    DRV_MSG("aess_cryptodrv_release\n");
    S_u8CryptoInitFlag = INIT_FAIL;
    return 0;
}

/******************************************************************************
*   FUNCTION        :   aess_cryptodrv_release
******************************************************************************/
/**
 *  @brief      This function will be called when device node is closed. Set init flag to INIT_FAIL
 *
 *  @return     0
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Static function\n
 *
 *****************************************************************************/
struct file_operations aess_cryptodrv_fops = 
{
    .open = aess_cryptodrv_open,
    .ioctl = aess_cryptodrv_ioctl,
    .release = aess_cryptodrv_release,
};

/******************************************************************************
*   FUNCTION        :   aess_cryptodrv_init
******************************************************************************/
/**
 *  @brief      Module init function. Initialize everything for crypto driver.
 *
 *  @return     0
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Static function\n
 *
 *****************************************************************************/
int __init aess_cryptodrv_init(void)
{
    int result;
    DRV_MSG("aess_cryptodrv_init\n");
    cryptodrv_cdev = cdev_alloc();
    cryptodrv_cdev->ops = &aess_cryptodrv_fops;
    result = alloc_chrdev_region(&aes_dev, 0, 1, driver_name);
    
    if (result < 0) 
    {
        return result;
    }
    
    cdev_add(cryptodrv_cdev, aes_dev, 1);

    /* show this message for aes driver install shell used */
    printk("mknod /dev/aess_cryptodrv c %d 0\n", MAJOR(aes_dev));

    //allocaing memory, The virtual address of G_pHace is linear address.
    //We can use virt_to_bus() macro to get the physical address
    #if KMALLOCATE_METHOD
	  G_pHace =(struct sCryptoStructType *)(((unsigned long) kmalloc(sizeof(struct sCryptoStructType),GFP_KERNEL)));
	  if(G_pHace == NULL)
	  {
        DRV_URGENT("Out of memory\n");
	      return -ENOMEM;
	  }
	  
	  //Initialize G_pHace structure pointer
	  memset(G_pHace, 0, sizeof(struct sCryptoStructType));
	  #if ENABLE_DEBUG
	  DRV_MSG("the virtual address of source data is %x\n",&G_pHace->au8SData[0]);
	  DRV_MSG("the virtual address of destination data is %x\n",&G_pHace->au8DData[0]);
	  DRV_MSG("the virtual address of context data is %x\n",&G_pHace->au8Context[0]);
    DRV_MSG("the physical address of source data is %x\n",virt_to_bus(&G_pHace->au8SData[0]));
	  DRV_MSG("the physical address of destination data is %x\n",virt_to_bus(&G_pHace->au8DData[0]));
	  DRV_MSG("the physical address of context data is %x\n",virt_to_bus(&G_pHace->au8Context[0]));
	  #endif
    
	      
	  #else

	  G_pHace->au8SData  = dma_alloc_writecombine(NULL, MAX_SOURCE_SIZE+8, &G_pHace->u32SData_phyaddr, GFP_DMA|GFP_KERNEL);
    G_pHace->au8DData   = dma_alloc_writecombine(NULL, MAX_DESTINATION_SIZE+8, &G_pHace->u32DData_phyaddr, GFP_DMA|GFP_KERNEL);
    G_pHace->au8Context  = dma_alloc_writecombine(NULL, MAX_CTXT_SIZE+8, &G_pHace->u32Context_phyaddr, GFP_DMA|GFP_KERNEL);
	  if(G_pHace->au8SData == NULL||G_pHace->au8DData ==NULL||G_pHace->au8Context==NULL )
	  {
        DRV_URGENT("Out of memory\n");
	      return -ENOMEM;
	  }
	  G_pHace->u8SData_addr_offset = 0;
	  G_pHace->u8DData_addr_offset = 0;
	  G_pHace->u8Context_addr_offset = 0;
	  if(G_pHace->u32SData_phyaddr&0x7)
	  {
         //It is not 8bytes aligned
          G_pHace->u8SData_addr_offset = (8-(G_pHace->u32SData_phyaddr&0x7));
	      G_pHace->au8SData+= G_pHace->u8SData_addr_offset;
	      G_pHace->u32SData_phyaddr += G_pHace->u8SData_addr_offset;
	  }
	  if(G_pHace->u32DData_phyaddr&0x7)
	  {
        //It is not 8bytes aligned
	      G_pHace->u8DData_addr_offset = (8-(G_pHace->u32DData_phyaddr&0x7));
	      G_pHace->au8DData += G_pHace->u8DData_addr_offset;
	      G_pHace->u32DData_phyaddr += G_pHace->u8DData_addr_offset;
	  }
	      
	  if(G_pHace->u32Context_phyaddr&0x7)
	  {
        //It is not 8bytes aligned
        G_pHace->u8Context_addr_offset += (8-(G_pHace->u32Context_phyaddr&0x7));
	      G_pHace->au8Context += G_pHace->u8Context_addr_offset;
	      G_pHace->u32Context_phyaddr += G_pHace->u8Context_addr_offset;
	  }
	  
	  #if ENABLE_DEBUG
	  DRV_MSG("the virtual address of source data is %x\n",(unsigned int)&G_pHace->au8SData[0]);
	  DRV_MSG("the virtual address of destination data is %x\n",(unsigned int)&G_pHace->au8DData[0]);
	  DRV_MSG("the virtual address of context data is %x\n",(unsigned int)&G_pHace->au8Context[0]);

    DRV_MSG("the physical address of source data is %x\n",G_pHace->u32SData_phyaddr);
	  DRV_MSG("the physical address of destination data is %x\n",G_pHace->u32DData_phyaddr);
	  DRV_MSG("the physical address of context data is %x\n",G_pHace->u32Context_phyaddr);
	  #endif

	  #endif

    /*Create files in /sys/kernel/debug/aess_cryptodrv folder for driver debug*/
    aess_debugfs_default_create(driver_name,NULL,0,0);
    return (STATUS_OK);
}

/******************************************************************************
*   FUNCTION        :   aess_cryptodrv_exit
******************************************************************************/
/**
 *  @brief      Module exit function. Release everything for crypto driver.
 *
 *  @return     0
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Static function\n
 *
 *****************************************************************************/
static void __exit aess_cryptodrv_exit(void)
{
    DRV_MSG("aess_cryptodrv_exit\n");
    
    /**Release allocated memory*/
	  #if KMALLOCATE_METHOD
	  if (NULL != G_pHace)
        kfree(G_pHace);
    #else
    if(NULL != G_pHace->au8SData)
        dma_free_coherent(NULL, MAX_SOURCE_SIZE+8,(void *)(G_pHace->au8SData - G_pHace->u8SData_addr_offset), G_pHace->u32SData_phyaddr);
    
    if(NULL != G_pHace->au8DData)
        dma_free_coherent(NULL,MAX_DESTINATION_SIZE+8,(void *)(G_pHace->au8DData - G_pHace->u8DData_addr_offset), G_pHace->u32DData_phyaddr);
    
    if(NULL != G_pHace->au8Context)
        dma_free_coherent(NULL,MAX_CTXT_SIZE+8,(void *)(G_pHace->au8Context - G_pHace->u32Context_phyaddr), G_pHace->u32Context_phyaddr);
    #endif
	
	  /* Release the character device related resource */
    cdev_del(cryptodrv_cdev);
 	  unregister_chrdev_region(aes_dev, 1);
 	  
 	  /*Remove files in /sys/kernel/debug/aess_cryptodrv folder*/
    aess_debugfs_remove(driver_name,NULL,0,0);
}


MODULE_DESCRIPTION("AESS Crypto Driver");
MODULE_AUTHOR("Jason Hung <jason.hung@avocent.com>");
MODULE_LICENSE("GPL");
MODULE_VERSION("0.1.0.0");
module_param(driver_name, charp, S_IRUGO);
module_init(aess_cryptodrv_init);
module_exit(aess_cryptodrv_exit);
/* End of code */
