/**
 * $Id$
 *
 *<center>
 *               Avocent Corporation. Proprietary Information.
 * \n<em>
 *      This software is supplied under the terms of a license agreement or
 *      nondisclosure agreement with Avocent Corporation, or one of its 
 *      affiliates, andaffiliates, and may not be used, disseminated, or 
 *      distributed except in accordance with the terms of that agreement.
 *
 *      3541 Gateway Boulevard Fremont, San Jose, CA 94538 U.S.A.
 *\n
 *                  US phone: 510-771-6100
 *
 *        Copyright &copy; 2001-2009 Avocent Corporation. All rights reserved.
 *</em> </center>
 *----------------------------------------------------------------------------\n
 *  MODULES     Crypto driver head file\n
 *----------------------------------------------------------------------------\n
 *  @file   aess_cryptodrv.h
 *  @brief  Crypt/Decrypt data
 */
 
#ifndef AESSCRYPTODRV_H
#define AESSCRYPTODRV_H

/* IOCTL command */
#define AESS_ENCRPDRV_IOC_MAGIC 0xC2
#define AESS_CRYPTODRV_ENCRYPT     _IOWR(AESS_ENCRPDRV_IOC_MAGIC, 0, sCryptoInfoType)
#define AESS_CRYPTODRV_DECRYPT     _IOWR(AESS_ENCRPDRV_IOC_MAGIC, 1, sCryptoInfoType)
#define AESS_CRYPTODRV_CRYPT       _IOWR(AESS_ENCRPDRV_IOC_MAGIC, 2, sCryptoInfoType)
#define AESS_CRYPTODRV_TEST        _IOWR(AESS_ENCRPDRV_IOC_MAGIC, 3, int)

#define AES_CRYPTO_ALGORITHM           0x0
#define RC4_CRYPTO_ALGORITHM           0x1

/******************************************************************************
*   Enum      :   eCryptoDrvKeySizeType
******************************************************************************/
/**
 *  @brief   Crypto driver key size enumeration type
 *
 *****************************************************************************/
typedef enum
{
	/** 128 bits */
	AES_DRV_128_BITS = 0,
	
	/** Cipher block chaining, CBC mode */
	AES_DRV_192_BITS,
	
	/** Cipher feedback, CFB mode */
	AES_DRV_256_BITS
	
} eCryptoDrvKeySizeType;


/******************************************************************************
*   Enum      :   eCryptoDrvModeType
******************************************************************************/
/**
 *  @brief   CRYPTO driver mode enumeration type
 *
 *****************************************************************************/
typedef enum
{
	/** Electronic code book, ECB mode */
	AES_DRV_MODE_ECB = 0,
	
	/** Cipher block chaining, CBC mode */
	AES_DRV_MODE_CBC,
	
	/** Cipher feedback, CFB mode */
	AES_DRV_MODE_CFB,
	
	/** Output feedback, OFB mode */
	AES_DRV_MODE_OFB,
	
	/** Counter, CTR mode */
	AES_DRV_MODE_CTR,
	
	/** ECB_DECRYPTO mode */
	AES_DRV_MODE_ECB_DECRYPT,
	
	/** CBC DECRYPT */
	AES_DRV_MODE_CBC_DECRYPT,
	
	
} eCryptoDrvModeType;


/******************************************************************************
*   STRUCT      :    sCryptoInfoType
******************************************************************************/
/**
 *  @brief   Structure to Crypto driver related data parameter.
 *
 *****************************************************************************/
typedef struct 
{
    /** Initial key */
    UINT8 *pu8InitKey;
    
    /** Initial vector*/
    UINT8 *pu8InitVector;
    
    /** Pointer to the source buffer */
    UINT8 *pu8Input;
    
    /** Pointer to the target buffer */
    UINT8 *pu8Output;
    
    /** Reserved */
    void *pReserved;

    /** Key size option */
    UINT32 u32DataLen;
    
    /** Key size option */
    UINT16 u16KeySize;

    /** Mode option */
    UINT8 u8Mode;
    
    /** Crypto algorithm */
    UINT8 u8CryptoAlg;
    
} sCryptoInfoType;


#ifdef AESSCRYPTODRV_C 

/* For temporary use, because no type.h now */
#define STATUS_OK     0
#define STATUS_FAIL   1


/* init flag */
#define INIT_OK                 0x0
#define INIT_FAIL               0x1

#else

			 
#endif   /* AESSCRYPTODRV_C */

#endif   /* AESSCRYPTODRV_H */

/* End of code */
