/**
 * $Id: aess_eventhandlerdrv.c,v 1.0 2009/03/16 11:00:07 wchen Exp $
 *
 *<center>
 *               Avocent Corporation. Proprietary Information.
 * \n<em>
 *      This software is supplied under the terms of a license agreement or
 *      nondisclosure agreement with Avocent Corporation, or its subsidiary, and
 *      may not be copied, disseminated, or distributed except in accordance
 *      with the terms of that agreement.
 *
 *      2001 Gateway Place, Suite 520W, San Jose, California, 95110 U.S.A.
 *\n
 *                  US phone: 408.436.6333
 *
 *        Copyright &copy; 2008 Avocent Corporation.
 *</em> </center>
 *----------------------------------------------------------------------------\n
 *  MODULES     linux driver
 *----------------------------------------------------------------------------\n
 *  @file   aess_eventhandlerdrv.c
 *  @brief  This is the kernel-layer event handler
 *
 *  @internal
 *  $RCSfile: aess_eventhandlerdrv.c,v $
 *  $Revision: 1.0 $
 *  $Date: 2009/03/16 10:16:00 $
 *  $Author: waynechen $
 *  $Source: http://tpe-subversion/AESSTPE/puti/puti_fw/ProjectBase/trunk/platform/wpcm450/evb/src/drivers/eventhandler/eventhandler_driver $
 *  $Name:  $
 *----------------------------------------------------------------------------*/
/******************************************************************************
* Content
* ----------------
*   aess_eventhandler_open()     - Open event handler node.
*   aess_eventhandler_close()    - Close event handler node.
*   aess_generate_driver_event()    - Generate an event.
*   aess_eventhandler_read()     - Read events from buffer.
*   aess_eventhandler_init()     - The module init function.
*   aess_eventhandler_exit()     - The module exit function.
******************************************************************************/
/*
 * WPCM450 event handler driver.
 *
 * Copyright (C) 2009 Avocent Corp.
 *
 * This file is subject to the terms and conditions of the GNU
 * General Public License. This program is distributed in the hope
 * that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 */
#define AESSEVENTHANDLER_C
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/cdev.h>
#include <linux/spinlock.h>
#include <linux/aess_debugfs.h>
#include <linux/stddef.h>
#include <linux/kfifo.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include "aess_eventhandlerdrv.h"
 
/** Linux device data structure */ 
dev_t evt_dev;
struct cdev *eventhandler_cdev; 

/** driver name will passed by insmod command */
static char *driver_name = EVENTHANDLER_DRIVER_NAME;
 
/** Init flag */
static UINT8 S_u8InitFlag = false;

static DECLARE_WAIT_QUEUE_HEAD(wq);

/** for debugfs */
static u32 u32tmpcounter = 0;

static struct kfifo *event_kfifo;
static spinlock_t event_kfifo_lock = SPIN_LOCK_UNLOCKED;

#define STATUS_OK        0
#define STATUS_FAIL      1 

/******************************************************************************
*   FUNCTION        :   aess_eventhandler_open
******************************************************************************/
/**
 *  @brief      Open event handler node.
 *
 *  @return     STATUS_OK
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None.
 *
 *  @internal   Function Type: Public function\n
 *
 *****************************************************************************/
static int aess_eventhandler_open(

                /** The inode pointer from the kernel. */
                struct inode *inode,

                /** The file pointer points to this file. */
                struct file *filp

                                 )
{
    DRV_MSG("aess_eventhandler_open!\n");

	if (S_u8InitFlag == false)
	{
		S_u8InitFlag = true;
	}

    kfifo_reset(event_kfifo);
	return (STATUS_OK);      
}


/******************************************************************************
*   FUNCTION        :   aess_eventhandler_close
******************************************************************************/
/**
 *  @brief      Close event handler node.
 *
 *  @return     0
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None.
 *
 *  @internal   Function Type: Public function\n
 *
 *****************************************************************************/
static int aess_eventhandler_close(

                /** The inode pointer from the kernel. */
                struct inode *inode,

                /** The file pointer points to this file. */
                struct file *filp

                                  )
{
    DRV_MSG("aess_eventhandler_close exit!\n");
	return 0;
}



/******************************************************************************
*   FUNCTION        :   aess_generate_driver_event
******************************************************************************/
/**
 *  @brief      Generate an event.
 *
 *  @return     STATUS_OK
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None.
 *
 *  @internal   Function Type: Public function\n
 *
 *****************************************************************************/
int aess_generate_driver_event(

                /** The driver ID. */
                UINT16 u16DriverID, 
                
                /** The event ID */
                UINT32 u32EventID
                
                              )
{
    sEventData buffer;
    buffer.u16DriverID = u16DriverID;
    buffer.u32EventID = u32EventID;

    DRV_MSG("aess_generate_driver_event: u16DriverID: 0x%x, u32EventID: 0x%x\n",
            (unsigned int) u16DriverID, (unsigned int) u32EventID);

	/** If nobody open node */
	if (S_u8InitFlag == false)
	{
		return -STATUS_FAIL;
	}

    if (kfifo_len(event_kfifo) >= (MAX_EVENT_NUMBER * sizeof(sEventData)))
    {
        return -STATUS_FAIL;
    }
	

    kfifo_put(event_kfifo, (unsigned char *) &buffer, sizeof(sEventData));

	/** Change the flag to wake up the read data process */
	wake_up_interruptible(&wq);
	return (STATUS_OK);
}


/******************************************************************************
*   FUNCTION        :   aess_eventhandler_read
******************************************************************************/
/**
 *  @brief      Read events from the event buffer.
 *
 *  @return     STATUS_OK
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None.
 *
 *  @internal   Function Type: Public function\n
 *
 *****************************************************************************/
static int aess_eventhandler_read(
        
                /** The file pointer points to this file. */
                struct file *filp, 
                    
                /** The user-space buffer */
                char *buf, 

                /** The amount of transfering data */
				size_t count, 
                
                /** The position of the current file */
                loff_t *ppos

                                )
{
    sEventData buffer;

	/** If no data (S_u8TotalEvent = 0), go to sleep */
	wait_event_interruptible(wq, kfifo_len(event_kfifo) > 0);

    kfifo_get(event_kfifo, (unsigned char *) &buffer, sizeof(sEventData));

    DRV_MSG("aess_eventhandler_read: u16DriverID: 0x%x, u32EventID: 0x:%x\n",
            (unsigned int) buffer.u16DriverID, (unsigned int) buffer.u32EventID);
	
	/** Copy data to user mode */
	if (copy_to_user(buf ,(char *) &buffer, sizeof(sEventData)))
	{ 
		return -EIO;
	}

	return (STATUS_OK);
}




struct file_operations aess_eventhandler_fops = {
	.open = aess_eventhandler_open, 
	.read = aess_eventhandler_read,
	.release = aess_eventhandler_close,        
};




/******************************************************************************
*   FUNCTION        :   aess_eventhandler_init
******************************************************************************/
/**
 *  @brief      The module initialization function.
 *
 *  @return     STATUS_OK
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None.
 *
 *  @internal   Function Type: Public function\n
 *
 *****************************************************************************/
int __init aess_eventhandler_init(void)
{
	int result;
	
	printk(KERN_DEBUG "KN:init_aess_mem_module\n");
	
	eventhandler_cdev = cdev_alloc();

	eventhandler_cdev->ops = &aess_eventhandler_fops;
	result = alloc_chrdev_region(&evt_dev, 0, 1, driver_name);

	printk("mknod /dev/aess_eventhandlerdrv c %d 0\n", MAJOR(evt_dev));

	if (result < 0) {
        DRV_ERROR("KN:Registering the event handler device failed with %d\n", MAJOR(evt_dev));
		return result;
	}

	cdev_add(eventhandler_cdev, evt_dev, 1);

    /** Allocate kfifo */
    event_kfifo = kfifo_alloc(MAX_EVENT_NUMBER * sizeof(sEventData), GFP_KERNEL, (void *) &event_kfifo_lock);

    /** Create Debugfs */
    aess_debugfs_default_create(driver_name, NULL, 0, 0);
    aess_debugfs_create_file(driver_name, "u32", (u32 *) &u32tmpcounter, DBG_TYPE32, 0);

	return (STATUS_OK);
}


/******************************************************************************
*   FUNCTION        :   aess_eventhandler_exit
******************************************************************************/
/**
 *  @brief      The module exit function.
 *
 *  @return     None.
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None.
 *
 *  @internal   Function Type: Public function\n
 *
 *****************************************************************************/
static void __exit aess_eventhandler_exit(void)
{
    DRV_MSG("aess_eventhandler_exit\n");
	cdev_del(eventhandler_cdev);
	unregister_chrdev_region(evt_dev, 1);

    /** Free kfifo */
    kfifo_free(event_kfifo);

    /** Remove Debugfs */
    aess_debugfs_remove(driver_name, NULL, 0, 0);
}



MODULE_DESCRIPTION("AESS Event Handler Driver");
MODULE_AUTHOR("Wayne Chen <wayne.chen@avocent.com>");
MODULE_LICENSE("GPL");
MODULE_VERSION("0.1.0.0");
module_param(driver_name, charp, S_IRUGO);
module_init(aess_eventhandler_init);
module_exit(aess_eventhandler_exit);
EXPORT_SYMBOL(aess_generate_driver_event);

/* End of code */
