/**
 * \n<em>
 *               Avocent Corporation. Proprietary Information.
 *      This software is supplied under the terms of a license agreement or
 *      nondisclosure agreement with Avocent Corporation, or one of its affiliates, and
 *      may not be used, copied, disseminated, or distributed except in accordance
 *      with the terms of that agreement.
 *
 *      3541 Gateway Boulevard, Fremont, California, 94538 U.S.A.
 *                  US Phone: 510-771-6100
 *      Copyright 2001-2009 Avocent Corporation. All rights reserved.
 *</em> </center>
 *----------------------------------------------------------------------------\n
 *  MODULES     aess fantach sensor driver\n
 *----------------------------------------------------------------------------\n
 * @file    aess_fansensordrv.c 
 * @brief   fantach sensor driver
 *----------------------------------------------------------------------------
 */
/******************************************************************************
* Content
* ----------------
*   aess_fansensor_open() - linux fantach sensor open function
*   aess_fansensor_read() - linux fantach sensor read function
*   aess_fansensor_ioctl() - linux fantach sensor i/o control function
*   aess_fansensor_release() - linux fantach sensor release function 
*   aess_fansensor_init() - linux fantach sensor init function
*   aess_fansensor_initialize() - initialization function for i/o control
*   aess_fansensor_exit() - linux fantach sensor exit function
******************************************************************************/


#define AESSFANSENSORDRV_C
#include <linux/module.h>
#include <linux/delay.h>
#include <linux/types.h>
#include <linux/fs.h>
#include <asm/uaccess.h>
#include <linux/slab.h>
#include <linux/signal.h>
#include <linux/spinlock.h>
#include <linux/cdev.h>
#include <linux/kernel.h>
#include <asm/io.h>
#include "aess_fansensordrv.h"
#include <asm/arch/hardware.h>
#include <linux/aess_debugfs.h>



//Debug Message
static u8   u8tmpcounter = 0;
static u16 u16tmpcounter = 0;
static u32 u32tmpcounter = 0;


/* driver name will passed by insmod command */
static char *driver_name = "aess_fansensordrv"; 

/* Init flag */
static UINT8 S_u8init_flag = AESSFANSENSOR_NOT_INIT;

static dev_t dev;      
struct cdev *fansensor_cdev;


/******************************************************************************
*   FUNCTION        :   aess_fansensor_open
******************************************************************************/
/**
 * @brief  aess_fansensor_open
 * @return STATUS_OK
 * @dependency     none
 * @limitation     none
 * @warning        none
 * @note           none
 * @internal       Function type: local function \n 
 */
static int aess_fansensor_open(
                               /** device node */
                               struct inode *inode, 

			       /** file pointer */
			       struct file *filp
			       )
{
    volatile UINT32 u32temp = 0; 

    if (S_u8init_flag == AESSFANSENSOR_NOT_INIT)
    {   
        DRV_MSG("KN: Open the Fan Sensor Device!!!\n");

        //SCU disable protection
        iowrite32(0x1688A8A8, (void *) SCU_PKR);

        // disable PWM reset function
        u32temp = ioread32((void *) SCU_RESET_CTRL);
        iowrite32((UINT32) (u32temp & ~(1<<9)), (void *) SCU_RESET_CTRL);

	// enable FANTACH0-FANTACH11
	u32temp = ioread32((void *) SCU_MUTI_FUN_PIN_CTRL1);
        iowrite32((UINT32) (u32temp | (1<<22)), (void *) SCU_MUTI_FUN_PIN_CTRL1);

        //SCU enable protection
        iowrite32(0x0, (void *) SCU_PKR);

        iowrite32((UINT32) 0xFF03FF03, (void *) (AST2050_FAN_BASE_REG + 0x4));        

        iowrite32((UINT32) 0x0, (void *) (AST2050_FAN_BASE_REG + 0x8));        
        iowrite32((UINT32) 0x0, (void *) (AST2050_FAN_BASE_REG + 0xC));  

        iowrite32((UINT32) FAN_TACH_CTRL0_DEFAULT_VALUE, (void *) FAN_REG_TYPEM_CTRL0_ADDR);        
        iowrite32((UINT32) FAN_TACH_CTRL1_DEFAULT_VALUE, (void *) FAN_REG_TYPEM_CTRL1_ADDR);        
        iowrite32((UINT32) FAN_TACH_CTRL0_DEFAULT_VALUE, (void *) FAN_REG_TYPEN_CTRL0_ADDR);        
        iowrite32((UINT32) FAN_TACH_CTRL1_DEFAULT_VALUE, (void *) FAN_REG_TYPEN_CTRL1_ADDR);
        
        //set PWM source, default assign is fan0~15 : ABCD ABCD ABCD ABCD
        iowrite32((UINT32) FAN_TACH_SOURCE_DEFAULT_VALUE, (void *) FAN_REG_TACH_SOURCE_ADDR);
        
        //disable all fan, and enable fan tach clock
        u32temp = ioread32((void *) FAN_REG_GEN_CTRL_ADDR);
        iowrite32((UINT32) (u32temp | 0x00000001), (void *) FAN_REG_GEN_CTRL_ADDR);  


        //clear trigger info
        iowrite32(0, (void *) FAN_REG_TRIGGER_ADDR);                  

        iowrite32((UINT32) FAN_TACH_CTRL0_DEFAULT_VALUE, (void *) FAN_REG_TYPEM_CTRL0_ADDR);        

        S_u8init_flag = AESSFANSENSOR_INIT_OK;
    }
    
    return (STATUS_OK);
}

/******************************************************************************
*   FUNCTION        :   aess_fansensor_initialize
******************************************************************************/
/**
 * @brief  aess_fansensor_initialize
 * @return STATUS_OK 
 * @dependency     none
 * @limitation     none
 * @warning        none
 * @note           none
 * @internal       Function type: local function \n 
 */
static int aess_fansensor_initialize(
                               /** data structure to pass between upper layer and driver*/
                               sFanSensorData *pFanSensorStruct
			       )
{
	UINT32 u32temp;

        //enable specified fan, and enable fan tach clock
        u32temp = ioread32((void *) FAN_REG_GEN_CTRL_ADDR);
	u32temp |= (1 << (pFanSensorStruct->u16ChannelNum+16));
        iowrite32(u32temp, (void *) FAN_REG_GEN_CTRL_ADDR); 

	return (STATUS_OK);
}


/******************************************************************************
*   FUNCTION        :   aess_fansensor_read
******************************************************************************/
/**
 * @brief  aess_fansensor_read
 * @return STATUS_OK when success, -EFAULT when fail 
 * @dependency     none
 * @limitation     none
 * @warning        none
 * @note           none
 * @internal       Function type: local function \n 
 */
static int aess_fansensor_read(
                               /** data structure to pass between upper layer and driver*/
                               sFanSensorData *pFanSensorStruct
			       )
{
    volatile UINT32 u32temp = 0;
    UINT8  u8ChannelNum;
    UINT32 u32SpeedDivisor;

    int rpm;

    DRV_MSG("KN:Enter aess_fansensor_read !!\n");


    /* Check the buffer address exist in the structure is OK */
    if(!access_ok(VERIFY_READ, &(pFanSensorStruct->u16ChannelNum), sizeof(UINT16)))
    {
        DRV_ERROR("KN:buffer access error!!\n");
        /* error occur */
        return -EFAULT;
    }     

    /* Check the buffer address exist in the structure is OK */
    if(!access_ok(VERIFY_WRITE, &(pFanSensorStruct->u8FanSpeedReading), sizeof(UINT8)))
    {
        DRV_ERROR("KN:buffer access error!!\n");
        /* error occur */
        return -EFAULT;
    }

    u8ChannelNum = pFanSensorStruct->u16ChannelNum;   

    //trigger fan measure 0-->1
    iowrite32(0, (void *) FAN_REG_TRIGGER_ADDR);
    barrier();
    iowrite32((UINT32) (1 << u8ChannelNum), (void *) FAN_REG_TRIGGER_ADDR);    
    
    /* Wait 1 ms to get the correct fan tach reading */
    mdelay(1);

    //read fan tach value
    u32temp = 0;
    u32temp = ioread32((void *) FAN_REG_RESULT_ADDR);

    DRV_INFO("KN: FAN_REG_RESULT_ADDR Register is %08x !!\n",(unsigned int)u32temp);

    DRV_INFO("AST2050_FAN_BASE_REG + 0x0 = 0x%lx\n", (unsigned long)ioread32((void *) (AST2050_FAN_BASE_REG + 0x0)));
    DRV_INFO("AST2050_FAN_BASE_REG + 0x4 = 0x%lx\n", (unsigned long)ioread32((void *) (AST2050_FAN_BASE_REG + 0x4)));
    DRV_INFO("AST2050_FAN_BASE_REG + 0x8 = 0x%lx\n", (unsigned long)ioread32((void *) (AST2050_FAN_BASE_REG + 0x8)));
    DRV_INFO("AST2050_FAN_BASE_REG + 0xC = 0x%lx\n", (unsigned long)ioread32((void *) (AST2050_FAN_BASE_REG + 0xC)));
    DRV_INFO("AST2050_FAN_BASE_REG + 0x10 = 0x%lx\n", (unsigned long)ioread32((void *) (AST2050_FAN_BASE_REG + 0x10)));
    DRV_INFO("AST2050_FAN_BASE_REG + 0x14 = 0x%lx\n", (unsigned long)ioread32((void *) (AST2050_FAN_BASE_REG + 0x14)));
    DRV_INFO("AST2050_FAN_BASE_REG + 0x18 = 0x%lx\n", (unsigned long)ioread32((void *) (AST2050_FAN_BASE_REG + 0x18)));
    DRV_INFO("AST2050_FAN_BASE_REG + 0x1C = 0x%lx\n", (unsigned long)ioread32((void *) (AST2050_FAN_BASE_REG + 0x1C)));
    DRV_INFO("AST2050_FAN_BASE_REG + 0x20 = 0x%lx\n", (unsigned long)ioread32((void *) (AST2050_FAN_BASE_REG + 0x20)));
    DRV_INFO("AST2050_FAN_BASE_REG + 0x24 = 0x%lx\n", (unsigned long)ioread32((void *) (AST2050_FAN_BASE_REG + 0x24)));
    DRV_INFO("AST2050_FAN_BASE_REG + 0x28 = 0x%lx\n", (unsigned long)ioread32((void *) (AST2050_FAN_BASE_REG + 0x28)));
    DRV_INFO("AST2050_FAN_BASE_REG + 0x2C = 0x%lx\n", (unsigned long)ioread32((void *) (AST2050_FAN_BASE_REG + 0x2C)));
    DRV_INFO("AST2050_FAN_BASE_REG + 0x30 = 0x%lx\n", (unsigned long)ioread32((void *) (AST2050_FAN_BASE_REG + 0x30)));
    DRV_INFO("AST2050_FAN_BASE_REG + 0x34 = 0x%lx\n", (unsigned long)ioread32((void *) (AST2050_FAN_BASE_REG + 0x34)));
    DRV_INFO("AST2050_FAN_BASE_REG + 0x38 = 0x%lx\n", (unsigned long)ioread32((void *) (AST2050_FAN_BASE_REG + 0x38)));
    DRV_INFO("AST2050_FAN_BASE_REG + 0x3C = 0x%lx\n", (unsigned long)ioread32((void *) (AST2050_FAN_BASE_REG + 0x3C)));

    if ((u32temp & FAN_TACH_READ_OK_MASK) > 0)
    {

        switch (pFanSensorStruct->u8FanSpeedDivisor)
        {
            case ONE_PULSE_REVOLUTION:
                /** 1 pulse per revolution */
                u32SpeedDivisor = 1;
                break;

            case TWO_PULSE_REVOLUTION:
                /** 2 pulses per revolution */
                u32SpeedDivisor = 2;
                break;

            case FOUR_PULSE_REVOLUTION:
                /** 4 pulses per revolution */
                u32SpeedDivisor = 4;
                break;

            case EIGHT_PULSE_REVOLUTION:
                /** 8 pulses per revolution */
                u32SpeedDivisor = 8;
                break;

            case SIXTEEN_PULSE_REVOLUTION:
                /* 16 pulses per revolution */
                u32SpeedDivisor = 16;
                break;

            default:
                u32SpeedDivisor = 2;
                break;
        }

        u32temp = u32temp & FAN_TACH_DATA_OK_MASK;
        rpm = 24000000 * 60 / (u32SpeedDivisor * u32temp * 2);
        pFanSensorStruct->u8FanSpeedReading = (UINT8) ((rpm >> 6)&0xff);

        DRV_INFO("FAN: u8ChannelNum = %d, u8FanSpeedReading = %d\n", u8ChannelNum, pFanSensorStruct->u8FanSpeedReading);
    }
    else
    {   
        DRV_INFO("FAN: u8ChannelNum = %d, read fail, return 0\n", u8ChannelNum);
        pFanSensorStruct->u8FanSpeedReading = 0;
        
        return (STATUS_OK);	       
    }

    return (STATUS_OK);

}

/******************************************************************************
*   FUNCTION        :   aess_fansensor_ioctl
******************************************************************************/
/**
 * @brief   aess_fansensor_ioctl 
 * @return  return code from aess_read or -EFAULT when fail
 * @dependency     none
 * @limitation     none
 * @warning        none
 * @note           none
 * @internal       Function type: local function \n 
 */
static int aess_fansensor_ioctl(
                                /** device node */
                                struct inode * inode, 

				/** file pointer */
				struct file *filp,

                                /** i/o control command number */
				unsigned int cmd, 
			
			        /** data structure for passing to i/o control command */
				unsigned long arg
				)
{
    int err_check;
    sFanSensorData *pFanSensorStruct = (sFanSensorData *) arg;

    /* Check the structure address is OK */
    if(!access_ok(VERIFY_WRITE, (void *) pFanSensorStruct, sizeof(sFanSensorData)))
    {
        DRV_ERROR("KN:struct access error!!\n");
        /* error occur */
        return -EFAULT;
    }

    switch(cmd)
    {   
    	case AESS_FANSENSORDRV_CONFIG:
	    err_check = aess_fansensor_initialize(pFanSensorStruct);
	    break;
        case AESS_FANSENSORDRV_RW:       
            err_check = aess_fansensor_read(pFanSensorStruct);  
            break;

        default:
            DRV_ERROR("KN:aess_fansensor_ioctl, command error!!! \n");
            err_check = -EINVAL;
    }

    DRV_MSG("KN:IO control Error no is %d!!\n", err_check);

    /* 0->ok, minus->fail */
    return err_check;
}

/******************************************************************************
*   FUNCTION        :   aess_fansensor_release
******************************************************************************/
/**
 * @brief  aess_fanseosor_release
 * @return         STATUS_OK
 *
 * @dependency     none
 * @limitation     none
 * @warning        none
 * @note           none
 * @internal       Function type: Global function \n 
 *****************************************************************************/
static int aess_fansensor_release(
                                  /** device node */
                                  struct inode *inode, 

				  /** file pointer */
				  struct file *filp
				  )
{
    iowrite32((UINT32) 0x0, (void *) FAN_REG_TYPEM_CTRL0_ADDR);        
    iowrite32((UINT32) 0x0, (void *) FAN_REG_TYPEM_CTRL1_ADDR);        
    iowrite32((UINT32) 0x0, (void *) FAN_REG_TYPEN_CTRL0_ADDR);        
    iowrite32((UINT32) 0x0, (void *) FAN_REG_TYPEN_CTRL1_ADDR);
    
    S_u8init_flag = AESSFANSENSOR_NOT_INIT;

    return (STATUS_OK);
}

struct file_operations aess_fansensor_fops = {
    .open = aess_fansensor_open, 
    .ioctl = aess_fansensor_ioctl,
    .release = aess_fansensor_release,
};

/******************************************************************************
*   FUNCTION        :   aess_fansensor_init
******************************************************************************/
/**
 * @brief  aess_fansensor_init
 * @return STATUS_OK when success, other when fail
 * @dependency     none
 * @limitation     none
 * @warning        none
 * @note           none
 * @internal       Function type: Global function \n 
 */
int __init aess_fansensor_init(void)
{
    int result;

    DRV_MSG("Init_aess_fansensor_module Sucess!!! \n");
    fansensor_cdev = cdev_alloc();

    fansensor_cdev->ops = &aess_fansensor_fops;
    result = alloc_chrdev_region(&dev, 0, 1, driver_name);

    printk("mknod /dev/aess_fansensordrv c %d 0\n", MAJOR(dev));

    if (result < 0)
    {
        DRV_ERROR("KN:Registering the character device failed with %d\n", MAJOR(dev));
        return result;
    }
    	   
    cdev_add(fansensor_cdev, dev, 1);
    
    aess_debugfs_default_create(driver_name,NULL,0,0);
    aess_debugfs_create_file(driver_name,"u32",(u32 *)&u32tmpcounter,DBG_TYPE32,0);
    aess_debugfs_create_file(driver_name,"u16",(u16 *)&u16tmpcounter,DBG_TYPE16,0);
    aess_debugfs_create_file(driver_name,"u8",(u8 *)&u8tmpcounter,DBG_TYPE8,0);    

    return (STATUS_OK);
}

/******************************************************************************
*   FUNCTION        :   aess_fansensor_exit
******************************************************************************/
/**
 * @brief  aess_fansensor_exit
 * @return none
 * @dependency     none
 * @limitation     none
 * @warning        none
 * @note           none
 * @internal       Function type: Global function \n 
 */
void __exit aess_fansensor_exit(void)
{
    DRV_MSG ("KN:aess_fansensor_exit \n");	
    aess_debugfs_remove(driver_name,NULL, 0 , 0);
    cdev_del(fansensor_cdev);
    unregister_chrdev_region(dev,1);
    

    return;
}


MODULE_DESCRIPTION("AESS Fan Sensor Driver");
MODULE_AUTHOR("Steven Lai <steven.lai@avocent.com");
MODULE_LICENSE("GPL");
MODULE_VERSION("0.1.0.0");
module_param(driver_name, charp, S_IRUGO);
module_init(aess_fansensor_init);
module_exit(aess_fansensor_exit);

/* End of code */
