/**
 * \n<em>
 *               Avocent Corporation. Proprietary Information.
 *      This software is supplied under the terms of a license agreement or
 *      nondisclosure agreement with Avocent Corporation, or one of its affiliates, and
 *      may not be used, copied, disseminated, or distributed except in accordance
 *      with the terms of that agreement.
 *
 *      3541 Gateway Boulevard, Fremont, California, 94538 U.S.A.
 *                  US Phone: 510-771-6100
 *      Copyright 2001-2009 Avocent Corporation. All rights reserved.
 *</em> </center>
 *----------------------------------------------------------------------------\n
 *  MODULES     aess fantach sensor driver\n
 *----------------------------------------------------------------------------\n
 * @file    aess_fansensordrv.h 
 * @brief   fantach sensor driver header
 *----------------------------------------------------------------------------
 */

#if !defined (AESSFANSENSORDRV_H)
#define AESSFANSENSORDRV_H



/******************************************************************************
*   STRUCT      :   sFanSensorData
******************************************************************************/
/**
 *  @brief   Structure to FanSensor driver related data parameter.
 *
 *****************************************************************************/
typedef struct
{
	/** Set Channel no. */
	UINT16 u16ChannelNum;

	/** Set Fan SpeedReading */
	UINT8 u8FanSpeedReading;        
	
	/** Set FanSensor Fan Speed Clock */
	UINT8 u8FanSpeedDivisor;     
	
} sFanSensorData;

/* ioctl definitions */
#define AESS_FANSENSORDRV_IOC_MAGIC    0xBD
#define AESS_FANSENSORDRV_CONFIG           _IOWR(AESS_FANSENSORDRV_IOC_MAGIC, 0, sFanSensorData)
#define AESS_FANSENSORDRV_RW           _IOWR(AESS_FANSENSORDRV_IOC_MAGIC, 1, sFanSensorData)

#ifdef AESSFANSENSORDRV_C

#include <linux/spinlock.h>
#include <asm-arm/arch-ast2050/hw_regs.h>

#define STATUS_OK     0

/* init flag */
#define AESSFANSENSOR_NOT_INIT    0
#define AESSFANSENSOR_INIT_OK     1
#define TIMEOUT_COUNTER_NOT_READY 0
#define TIMEOUT_COUNTER_READY     1
#define TRUE                      1
#define FALSE                     0

/* Contrl bit mask definition */
/* FAN_SPEED_CONTROL_Register */
#define	FAN_TACH_READ_OK_MASK								0x80000000
#define	FAN_TACH_DATA_OK_MASK								0xfffff

#define FAN_TACH_CTRL0_DEFAULT_VALUE				0x10000001
#define FAN_TACH_CTRL1_DEFAULT_VALUE				0x04000000
#define FAN_TACH_SOURCE_DEFAULT_VALUE				0xe4e4e4e4

#define FAN_MEASURE_PERIOD									6 //300ms
#define TIMEOUT_5s                      		5

#define ONE_PULSE_REVOLUTION        1
#define TWO_PULSE_REVOLUTION        2
#define FOUR_PULSE_REVOLUTION       4
#define EIGHT_PULSE_REVOLUTION      8
#define SIXTEEN_PULSE_REVOLUTION    16




/******************************************************************************
*   STRUCT      :   Function Prototype 
******************************************************************************/
/**
 *  @brief   Prototype for each private function.
 *
 *****************************************************************************/
 
static int aess_fansensor_read(sFanSensorData *pFanSensorStruct);

#endif   /* AESSFANSENSORDRV_C */

#endif   /* AESSFANSENSORDRV_H */

/* End of code */
