/**
 * $Id$
 *
 *<center>
 *               Avocent Corporation. Proprietary Information.
 * \n<em>
 *      This software is supplied under the terms of a license agreement or
 *      nondisclosure agreement with Avocent Corporation, or  one of its 
 *      affiliates, andaffiliates, and may not be used, disseminated, or 
 *      distributed except in accordance with the terms of that agreement.
 *
 *      3541 Gateway Boulevard Fremont, San Jose, CA 94538 U.S.A.
 *\n
 *                  US phone: 510-771-6100
 *
 *        Copyright &copy; 2001-2009 Avocent Corporation. All rights reserved.
 *</em> </center>
 *----------------------------------------------------------------------------\n
 *  MODULES     Linux device Driver                                           \n
 *----------------------------------------------------------------------------\n
 *  @file   aess_gpiodrv.c
 *  @brief  gpio driver 
 *  @internal
 *  $RCSfile: aess_gpiodrv.c,v $
 *  $Revision: $
 *  $Date: 2009/03/10 13:44:48 $
 *  $Author: Taiyi Kuo $
 *  $Source: $
 *  $Name:  $
 *----------------------------------------------------------------------------*/
 /******************************************************************************
* Content
* ----------------
*   ShowRegValue()            - Dump Register value for debug.
*   MultiFunctionCfg()        - Multi-function ping configue to GPIO function.
*   Get_Debounce()            - Get the specific ping debouncing status.  
*   Set_Debounce()            - Set the specific ping debouncing status.  
*   Reg_Get32()               - Get the specific ping register status.  
*   Reg_Set32()               - Set the specific ping register status.   
*   aess_gpio_open()          - The function is invoked when application open driver note.
*   aess_gpio_close()         - The function is invoked when application close the device note.
*   aess_gpio_commander()     - The function is exported for to others to confige interrupt status. 
*   aess_gpio_config()        - The function will set the gpio ping interrupt type.
*   aess_gpio_read()          - Get gpio ping status.  
*   aess_gpio_write()         - Set gpio ping status.
*   aess_gpio_ioctl()         - The API is invoked when user space application issues an IOCTL command.
*   aess_gpio_init()          - The function is invoked when biospost module is loaded into the kernel.
*   aess_gpio_exit()          - The function is invoked when biospost driver be removed from kernal.
*   aess_gpio_GetEventPin()   - The function report the gpio ping interrput status .
******************************************************************************/


#define AESSGPIODRV_C

#include <linux/module.h>
#include <linux/fs.h>
#include <linux/cdev.h>
#include <linux/spinlock.h>
#include <asm/uaccess.h>
#include <asm/io.h>

//#include <asm-arm/arch-ast2050/hw_regs.h>

#include "aess_gpiodrv.h"

/* Debugging Message */
#define GPIO_DEBUG
// #define GPIO_DEBUG2
#ifdef GPIO_DEBUG
#define DEBUG_MSG(fmt, args...) printk( "\n GPIO: " fmt, ## args)
#else
#define DEBUG_MSG(fmt, args...)
#endif

/* implement debug message with debugfs */
#include <linux/aess_debugfs.h>
static u8   u8tmpcounter = 0;
static u16 u16tmpcounter = 0;
static u32 u32tmpcounter = 0;

/* driver name will passed by insmod command */
static char *driver_name="aess_gpiodrv"; 

/* The init flag for aess_gpio_open */
static UINT8 S_u8init_flag = AESSGPIO_NOT_INIT;

/* Linux device data structure */
dev_t gpio_dev;
struct cdev *gpio_cdev;

#define SCU_Lock                (ASPEED_SCU_VIRTUAL+0x00)

#define SCU_2C                  (ASPEED_SCU_VIRTUAL+0x2C)
#define REG_MiscCtrl            SCU_2C
#define SCU_70                  (ASPEED_SCU_VIRTUAL+0x70)
#define REG_HWTrap              SCU_70
#define SCU_74                  (ASPEED_SCU_VIRTUAL+0x74)
#define REG_MultiFuncPinCtrl1   SCU_74
#define SCU_78                  (ASPEED_SCU_VIRTUAL+0x78)
#define REG_MultiFuncPinCtrl2   SCU_78

// spinlock init
spinlock_t reg_lock=SPIN_LOCK_UNLOCKED;

#define GPIO_E0_Port 0
#define GPIO_E1_Port 1
UINT8  GPIO_EX_Port =0xff;

/******************************************************************************
*   FUNCTION        :   ShowRegValue
******************************************************************************/
/**
 *  @brief      Dump Register value for debug
 *
 *  @return     None                 
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function \n
 *
 *****************************************************************************/

static void ShowRegValue(
                         /** function name */
                         char* func_name
                         )
{
    DRV_URGENT( "\t %s()   \n", func_name );    
    DRV_URGENT( "\tDATA_VALUE=0x%08x\n", ioread32(REG_DATA_VALUE) );    
    DRV_URGENT( "\tDIRECTION =0x%08x\n", ioread32(REG_DIRECTION) );    
    DRV_URGENT( "\tINT_ENABLE=0x%08x\n", ioread32(REG_INT_ENABLE) );    
    DRV_URGENT( "\tINT_TYPE0 =0x%08x\n", ioread32(REG_INT_TYPE0) );    
    DRV_URGENT( "\tINT_TYPE1 =0x%08x\n", ioread32(REG_INT_TYPE1) );    
    DRV_URGENT( "\tINT_TYPE2  =0x%08x\n", ioread32(REG_INT_TYPE2) );    
    DRV_URGENT( "\tINT_STATUS=0x%08x\n", ioread32(REG_INT_STATUS) );    
    DRV_URGENT( "\tRESET_TOLERANT=0x%08x\n", ioread32(REG_RESET_TOLERANT) );    
    DRV_URGENT( "\tEXT_DATA_VALUE=0x%08x\n", ioread32(REG_EXT_DATA_VALUE) );    
    DRV_URGENT( "\tEXT_DIRECTION =0x%08x\n", ioread32(REG_EXT_DIRECTION) );    
    DRV_URGENT( "\tEXT_INT_ENABLE=0x%08x\n", ioread32(REG_EXT_INT_ENABLE) );    
    DRV_URGENT( "\tEXT_INT_TYPE0 =0x%08x\n", ioread32(REG_EXT_INT_TYPE0) );    
    DRV_URGENT( "\tEXT_INT_TYPE1 =0x%08x\n", ioread32(REG_EXT_INT_TYPE1) );    
    DRV_URGENT( "\tEXT_INT_TYPE2 =0x%08x\n", ioread32(REG_EXT_INT_TYPE2) );    
    DRV_URGENT( "\tEXT_INT_STATUS=0x%08x\n", ioread32(REG_EXT_INT_STATUS) );    
    DRV_URGENT( "\tEXT_RESET_TOLERANT=0x%08x\n", ioread32(REG_EXT_RESET_TOLERANT) );    
    DRV_URGENT( "\tDEBOUNCE1     =0x%08x\n", ioread32(REG_DEBOUNCE1) );    
    DRV_URGENT( "\tDEBOUNCE2     =0x%08x\n", ioread32(REG_DEBOUNCE2) );    
    DRV_URGENT( "\tEXT_DEBOUNCE1 =0x%08x\n", ioread32(REG_EXT_DEBOUNCE1) );    
    DRV_URGENT( "\tEXT_DEBOUNCE2 =0x%08x\n", ioread32(REG_EXT_DEBOUNCE2) );    
    DRV_URGENT( "\tDEBOUNCE_TIME1=0x%08x\n", ioread32(REG_DEBOUNCE_TIME1) );    
    DRV_URGENT( "\tDEBOUNCE_TIME2=0x%08x\n", ioread32(REG_DEBOUNCE_TIME2) );    
    DRV_URGENT( "\tDEBOUNCE_TIME3=0x%08x\n", ioread32(REG_DEBOUNCE_TIME3) );
    DRV_URGENT( "\tMiscCtrl(2C)=0x%08x\n", ioread32(REG_MiscCtrl) );
    DRV_URGENT( "\tHWTrap(Trap)=0x%08x\n", ioread32(REG_HWTrap) );
    DRV_URGENT( "\tMultiFuncCtrl(74)=0x%08x\n", ioread32(REG_MultiFuncPinCtrl1) );
    DRV_URGENT( "\tMultiFuncCtrl(78)=0x%08x\n", ioread32(REG_MultiFuncPinCtrl2) );
}


/******************************************************************************
*   FUNCTION        :   MultiFunctionCfg
******************************************************************************/
/**
 *  @brief      Multi-function ping configue to GPIO function....
 *
 *  @return     Return 0 if configure success.
 *              Return 1 if configure fail.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function \n
 *
 *****************************************************************************/
int MultiFunctionCfg(
                    /** gpio ping number */ 
                    unsigned int pin_num
                    )
{
    // follow the AST2050 data sheet 1.7 section
    UINT32 u32tmp =0 ; 

    /* To unlock SCU 0x1688a8a8-> unlock ; others lock  */
    iowrite32( 0x1688a8a8 ,SCU_Lock );

    DRV_URGENT( "\tMultiFunctionCfg::Pin=%d\n", pin_num );
    
    if( pin_num == 4 ||  pin_num == 5)
    {
        u32tmp = ioread32(REG_MultiFuncPinCtrl1) ;
        u32tmp &= (~ (0x00000001<<25) ) ;
        iowrite32( u32tmp ,REG_MultiFuncPinCtrl1 ); 
    }
    else if( pin_num == 8 )          // GPIOB0
    {
        u32tmp = ioread32(REG_MultiFuncPinCtrl2) ;
        u32tmp |=  (0x00000001 <<4) ;
        iowrite32( u32tmp ,REG_MultiFuncPinCtrl2 ); 
    }
    else if( pin_num == 9 || pin_num == 10 )  // GPIOB1 ;GPIOB2 
    {
        u32tmp = ioread32(REG_MultiFuncPinCtrl1) ;
        u32tmp &= (~ (0x00000001<<2) ) ;
        iowrite32( u32tmp ,REG_MultiFuncPinCtrl1); 
    }
    else if( pin_num == 12 ) // GPIOB4 
    {
        u32tmp = ioread32(REG_HWTrap) ;
        u32tmp &= ( ~(( 0x0000001<<5) || (0x00000001<<23) ) ) ;
        iowrite32( u32tmp , REG_HWTrap ); 
    }
    else if( pin_num == 13 || pin_num == 15) // GPIOB5 ; GPIOB7 
    {
        u32tmp = ioread32(REG_HWTrap) ;
        u32tmp &= ( ~( 0x0000001<<5) ) ;
        iowrite32( u32tmp , REG_HWTrap ); 
    }
    else if( pin_num == 14 ) // GPIOB6 
    {
        u32tmp = ioread32(REG_HWTrap) ;
        u32tmp &= ( ~( 0x00000001<<5) ) ;
        iowrite32( u32tmp , REG_HWTrap ); 
        
        u32tmp = ioread32(REG_MultiFuncPinCtrl2) ;
        u32tmp &= ( ~( 0x00000001<<3) ) ;
        iowrite32( u32tmp , REG_MultiFuncPinCtrl2 ); 
    }
    else if( pin_num == 16 || pin_num == 17 ) // GPIOC0 ; GPIOC1 
    {
        u32tmp = ioread32(REG_MultiFuncPinCtrl1) ;
        u32tmp &= ( ~( 0x00000001<<7) ) ;
        iowrite32( u32tmp , REG_MultiFuncPinCtrl1 ); 
    }
    else if( pin_num == 18 ) // GPIOC2 
    {
        u32tmp = ioread32(REG_MultiFuncPinCtrl1) ;
        u32tmp &= ( ~( 0x00000001<<8) ) ;
        iowrite32( u32tmp , REG_MultiFuncPinCtrl1 ); 
    }
    else if( pin_num == 19 ) // GPIOC3 
    {
        u32tmp = ioread32(REG_MultiFuncPinCtrl1) ;
        u32tmp &= ( ~( 0x00000001<<9) ) ;
        iowrite32( u32tmp , REG_MultiFuncPinCtrl1 ); 
    }
    else if( pin_num == 20 ) // GPIOC4 
    {
        u32tmp = ioread32(REG_MultiFuncPinCtrl1) ;
        u32tmp &= ( ~( 0x00000001<<10) ) ;
        iowrite32( u32tmp , REG_MultiFuncPinCtrl1 ); 
    }
    else if( pin_num == 21 ) // GPIOC5 
    {
        u32tmp = ioread32(REG_MultiFuncPinCtrl1) ;
        u32tmp &= ( ~( 0x00000001<<11) ) ;
        iowrite32( u32tmp , REG_MultiFuncPinCtrl1 ); 
    }
    else if( pin_num == 60 || pin_num == 61 ) // GPIOH4 ;GPIOH5 
    {
        u32tmp = ioread32(REG_MultiFuncPinCtrl1) ;
        u32tmp &= ( ~( 0x00000003<<15) ) ;
        iowrite32( u32tmp , REG_MultiFuncPinCtrl1 ); 
    }
    else if( pin_num == 62 || pin_num == 63 ) // GPIOH6 ;GPIOH7 
    {
        u32tmp = ioread32(REG_MultiFuncPinCtrl1) ;
        u32tmp &= ( ~( 0x00000001<<16) ) ;
        iowrite32( u32tmp , REG_MultiFuncPinCtrl1 ); 
    }
    else if( pin_num == 30 ) // GPIOD6
    {
        u32tmp = ioread32(REG_MultiFuncPinCtrl1) ;
        u32tmp &= ( ~( 0x00000001<<18) ) ;
        iowrite32( u32tmp , REG_MultiFuncPinCtrl1 ); 
    }
    else if( pin_num == 31 ) // GPIOD7
    {
        u32tmp = ioread32(REG_MiscCtrl) ;
        u32tmp &= ( ~( 0x00000001<<1) ) ;
        iowrite32( u32tmp , REG_MiscCtrl ); 

        u32tmp = ioread32(REG_MultiFuncPinCtrl1) ;
        u32tmp &= ( ~( 0x00000001<<18) ) ;
        iowrite32( u32tmp , REG_MultiFuncPinCtrl1 ); 
    }
    else if( pin_num == 22 || pin_num == 23) // GPIOC6 ; GPIOC7
    {
        u32tmp = ioread32(REG_MultiFuncPinCtrl1) ;
        u32tmp &= ( ~( 0x00000001<<20 || 0x00000001<<12) ) ;
        iowrite32( u32tmp , REG_MultiFuncPinCtrl1 ); 
    }
    else if( pin_num == 56 || pin_num == 57) // GPIOH0 ; GPIOH1 
    {
        u32tmp = ioread32(REG_MultiFuncPinCtrl1) ;
        u32tmp &= ( ~( 0x00000001<<13) ) ;
        iowrite32( u32tmp , REG_MultiFuncPinCtrl1 ); 
    }
    else if( pin_num == 58 || pin_num == 59) // GPIOH2 ;GPIOH3 
    {
        u32tmp = ioread32(REG_MultiFuncPinCtrl1) ;
        u32tmp &= ( ~( 0x00000001<<14) ) ;
        iowrite32( u32tmp , REG_MultiFuncPinCtrl1 ); 
    }

    else if( pin_num >= 44 && pin_num <= 49 ) // GPIOF40 ;GPIOG1 
    {
        u32tmp = ioread32(REG_MultiFuncPinCtrl1) ;
        u32tmp &= ( ~( 0x00000001<<23) ) ;
        iowrite32( u32tmp , REG_MultiFuncPinCtrl1 ); 
    }
    else if( pin_num >= 32 && pin_num <= 39 ) // GPIOE0 ;GPIOE7 (GPIO E?_1 ) Group 1
    {
        if( GPIO_EX_Port == GPIO_E1_Port )
        {
            DEBUG_MSG( "\t===== MultiFunctionCfg Error =====\n" );
            DEBUG_MSG( "\tGPIO E port Group 2 is used, can not initial Gropp 1\n" );			
            return 1;
        }

        if (0xff == GPIO_EX_Port)
        {			
            DEBUG_MSG( "\tGPIO E port Group 1 init\n" );			
        }

        GPIO_EX_Port = GPIO_E0_Port ;

        u32tmp = ioread32(REG_MultiFuncPinCtrl1) ;
        u32tmp &= (~( 0x00000001<<22 )) ;
        iowrite32( u32tmp , REG_MultiFuncPinCtrl1 ); 
        
        u32tmp = ioread32(REG_MultiFuncPinCtrl1) ;
        u32tmp &= (~( 0x00000001 <<27 )) ;
        iowrite32( u32tmp , REG_MultiFuncPinCtrl1 ); 
        
    }
    else if( pin_num >= 40 && pin_num <= 43 ) // GPIOF0 -GPIOF3 
    {
        u32tmp = ioread32(REG_MultiFuncPinCtrl1) ;
        u32tmp &= (~( 0x00000001<<22 )) ;
        iowrite32( u32tmp , REG_MultiFuncPinCtrl1 ); 
    }
    else if( pin_num >= 64 && pin_num <= 71 ) // GPIOE0 ;GPIOE7 (GPIO E?_2 ) Group 2
    {
        if( GPIO_EX_Port == GPIO_E0_Port )
        {			
            DEBUG_MSG( "\t===== MultiFunctionCfg Error =====\n" );
            DEBUG_MSG( "\tGPIO E port Group 1 is used, can not initial Gropp 2\n" );			
            return 1;
        }

        if (0xff == GPIO_EX_Port)
        {			
            DEBUG_MSG( "\tGPIO E port Group 2 init\n" );			
        }

        GPIO_EX_Port = GPIO_E1_Port ;

        //u32tmp = ioread32(REG_HWTrap) ;
        //u32tmp &= ( ~( 0x00000007<<6) ) ;
        //iowrite32( u32tmp , REG_HWTrap ); 
        
        u32tmp = ioread32(REG_MultiFuncPinCtrl1) ;
        u32tmp |= ( 0x00000001 <<27 ) ;
        u32tmp &= ( ~(0x00000001 <<20) ) ;
        iowrite32( u32tmp , REG_MultiFuncPinCtrl1 ); 
    }

    //DEBUG_MSG( "\tcaton - %s, %s\n", __FILE__, __FUNCTION__ );
    DRV_URGENT( "\tMultiFunctionCfg::Value=0x%08x\n\n", u32tmp );

    iowrite32( 0 ,SCU_Lock );
    
    return 0;
}
/******************************************************************************
*   FUNCTION        :   Get_Debounce
******************************************************************************/
/**
 *  @brief      Get the specific ping debouncing status.
 *
 *  @return     Return 0 if debounce function disable.
 *              Return 1 if debounce function enable.
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function \n
 *
 *****************************************************************************/
UINT8 Get_Debounce( 
                    /** gpio ping number */ 
                    unsigned int pin_num
                  )
{
    int pin_in_reg = pin_num %32 ;
    unsigned int addr = REG_DEBOUNCE1 ;
    unsigned int tmp ;
    
    if( pin_num > 32 )
        addr = REG_EXT_DEBOUNCE1 ;
    
    tmp = ioread32( addr );

    if( (tmp & (0x1 << pin_in_reg))  != 0 )
        return 0x01 ;
    else        
        return 0 ;   
}

/******************************************************************************
*   FUNCTION        :   Set_Debounce
******************************************************************************/
/**
 *  @brief      Set the specific ping debouncing status.
 *
 *  @return     None
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function \n
 *
 *****************************************************************************/
void Set_Debounce( 
                    /** gpio ping number */ 
                  unsigned int pin_num , 

                    /** value */ 
                  unsigned int value 
                 )
{
    int pin_in_reg = pin_num %32 ;
    unsigned int addr = 0 ;
    unsigned int tmp = 0;
    
    if( pin_num < 32 ) // for pin[32~63]
        addr = REG_DEBOUNCE1 ;
    else        
        addr = REG_EXT_DEBOUNCE1 ;

    tmp = ioread32( addr );
    
    if( value !=0 )
        {  tmp |= (0x1 << pin_in_reg) ;  }
    else               
        {  tmp &= ( ~(0x1 << pin_in_reg) ) ; }
    
    iowrite32( tmp , addr ) ;
}


/******************************************************************************
*   FUNCTION        :   Reg_Get32
******************************************************************************/
/**
 *  @brief      Get the specific ping register status.
 *
 *  @return     Return 1 if bit is set .
 *              Return 0 if bit is not set .
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function \n
 *
 *****************************************************************************/
UINT8 Reg_Get32( 
                /** gpio ping number */
                unsigned int pin_num , 
                
                /** specific address */
                unsigned int addr 
                )
{
    unsigned int tmp32=0 ;
    int pin_in_reg = pin_num %32 ;
   
    if( pin_num < 32 )
        tmp32 = ioread32( addr );    
    else                 
        tmp32 = ioread32( addr+ 0x20 );   
        
    if( (tmp32 & (0x1 << pin_in_reg) ) != 0 ) 
        {  return 1 ;} 
    else                                      
        {  return 0 ;}
}


/******************************************************************************
*   FUNCTION        :   Reg_Set32
******************************************************************************/
/**
 *  @brief      Set the specific ping register status.
 *
 *  @return     Return 1 if bit is set .
 *              Return 0 if bit is not set .
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function \n
 *
 *****************************************************************************/
void Reg_Set32( 
                /** gpio ping number */
                unsigned int pin_num , 
                
                /** value */
                unsigned int value , 
                
                /** specific address */
                unsigned int addr 
                )
{
    unsigned int tmp32=0;
    
    if( pin_num < 32 )   
        tmp32 = ioread32( addr );    
    else                 
        tmp32 = ioread32( addr+ 0x20 ); 

    if(  value == 0 )    
        tmp32 &= (~(0x1 <<( pin_num %32))) ;        
    else                 
        tmp32 |= ( 0x1 << ( pin_num %32) )  ;        
 
    if( pin_num < 32 )   
        iowrite32( tmp32 ,addr );
    else                 
        iowrite32( tmp32 , addr+ 0x20 ); 
    
}

/******************************************************************************
*   FUNCTION        :   aess_gpio_open
******************************************************************************/
/**
 *  @brief      The function is invoked when application open driver note.
 *
 *  @return     STATUS_OK never fail.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function \n
 *
 *****************************************************************************/
static int aess_gpio_open(
                           /** inode structure point */
                           struct inode *inode, 

                           /** file structure point */
                           struct file *filp
                          )
{
    if (S_u8init_flag == AESSGPIO_NOT_INIT)
    {
        /* Set GPIO Sensitivity for Interrupt controller */
        IRQ_SET_HIGH_LEVEL(INT_GPIO); 
        IRQ_SET_LEVEL_TRIGGER(INT_GPIO); 	

        //for ast2100, we only provide one debounce time for user using
        //so clear debounce setting Register #2
        //so set debunce setting Register #1 to enable debounce, or clear to disable
        
        spin_lock( &reg_lock );

        /* default no-debouncing */        
        iowrite32(0, (void *) REG_DEBOUNCE1 ); 
        iowrite32(0, (void *) REG_DEBOUNCE2 ); 
        iowrite32(0, (void *) REG_EXT_DEBOUNCE1 ); 
        iowrite32(0, (void *) REG_EXT_DEBOUNCE2 ); 
        
        // following WCPM450 platform set debouncing time to 16ms //
        /* Debouncing_time = REG_DEBOUNCe_TIME1 * PCLK
           PCLK = H-PLL /8 ; H-PLL =200M ==> REG_DEBOUNCe_TIME1 = 400000      */
        iowrite32(400000, (void *) REG_DEBOUNCE_TIME1 ); 
        S_u8init_flag = AESSGPIO_INIT_OK;
        
        spin_unlock( &reg_lock );
        
        DRV_INFO("aess_gpio_open()::return (STATUS_OK)\n");

    }
	else
	{
	    DRV_INFO("aess_gpio_open()::return (STATUS_OK):: Open Already!\n");
	}

    ShowRegValue("aess_gpio_open()");    
    
    return (STATUS_OK);
}

/******************************************************************************
*   FUNCTION        :   aess_gpio_close
******************************************************************************/
/**
 *  @brief      The function is invoked when application close the device note.
 *
 *  @return     STATUS_OK never fail.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function \n
 *
 *****************************************************************************/
static int aess_gpio_close(
                           /** inode structure point */
                           struct inode *inode, 

                           /** file structure point */
                           struct file *filp
                          )
{
    DRV_INFO("aess_gpio_close()::return (STATUS_OK)\n");
    return (STATUS_OK);
}



/******************************************************************************
*   FUNCTION        :   aess_gpio_commander
******************************************************************************/
/**
 *  @brief      The function is exported for to others to use.
 *
 *  @return     Return STATUS_OK if fucntion execute success.
 *              Return others if fucntion execute fail.
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function \n
 *
 *****************************************************************************/
int aess_gpio_commander(
                        /** gpio ping number*/
                        UINT8 u8PortPin,  
                        
                        /** Reference to command list */
                        UINT8 u8CommandType,
                        
                        /** Reference to command list */
                        UINT8 u8CommandNum,
                        
                        /** Data buffer, right now is UINT8 */
                        void *pData
                        )
{
    sGPIOData sGPIOStruct;
    sGPIOStruct.u8PortNum = u8PortPin;
    sGPIOStruct.u8CommandType = u8CommandType;
    sGPIOStruct.u8CommandNum = u8CommandNum;
    sGPIOStruct.pData = pData;    

    // if no one open then initate it
    if (S_u8init_flag == AESSGPIO_NOT_INIT)
        aess_gpio_open(0,0);

    /* Ensure port and pin number does not exceed bounds */  
    if (sGPIOStruct.u8PortNum >= GPIO_PIN_MAX)
    {
        DRV_ERROR("aess_gpio_commander()::return -STATUS_FAIL:: (sGPIOStruct.u8PortNum >= GPIO_PIN_MAX)\n");
        return -STATUS_FAIL;
    }
    
    /* GPIO_CONFIG command doesn't need to use the pData parameter */
    if ((pData == NULL) && (u8CommandType != GPIO_CONFIG))
    {
        DRV_ERROR("aess_gpio_commander()::return -STATUS_FAIL::(pData == NULL) && (u8CommandType != GPIO_CONFIG)\n");
        return -STATUS_FAIL;
    }    

    /* Dispath the command */
    switch(u8CommandType)
    {
        case GPIO_CONFIG:    
            return (aess_gpio_config(&sGPIOStruct));
			
        case GPIO_WRITE:    
            return (aess_gpio_write(&sGPIOStruct));
            		
        case GPIO_READ:    
            return (aess_gpio_read(&sGPIOStruct));
			
        default:
            DRV_ERROR("aess_gpio_commander()::return -STATUS_FAIL::ucommand type error\n");
            return -STATUS_FAIL;     
    }                           
}

/******************************************************************************
*   FUNCTION        :   aess_gpio_config
******************************************************************************/
/**
 *  @brief      The function will set the gpio ping interrupt type.
 *
 *  @return     Return STATUS_OK if fucntion execute success.
 *              Return others if fucntion execute fail.
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function \n
 *
 *****************************************************************************/
static int aess_gpio_config(
                            /** sGPIOData structure point */
                            sGPIOData *pGPIOStruct
                            )
{           
    unsigned int value;
    UINT8 PinNum = pGPIOStruct->u8PortNum ;
    
    if( PinNum >= 64 ) // GPIO_Port_E2 
        PinNum -=32 ;
    
    if( *(UINT8*)pGPIOStruct->pData != 0 ) 
        value = 1;
    else
        value = 0;                             
    
    spin_lock( &reg_lock );

    if( pGPIOStruct->u8CommandNum != MULTI_FUNC_CFG )
    {
        // set the specific pin value to low
        Reg_Set32( pGPIOStruct->u8PortNum , 0 , REG_DATA_VALUE ); 
        // Set the specific pin in input direction
        Reg_Set32( pGPIOStruct->u8PortNum , 0 , REG_DIRECTION ); 
    }     
    /* Config the specific GPIO pin */
    switch(pGPIOStruct->u8CommandNum)
    {
        case INPUT_INT_OFF:   
            // disnable interrupt
            Reg_Set32( PinNum , 0 , REG_INT_ENABLE ); 
            // clean the specific pin interruprt status
            Reg_Set32( PinNum, 1 , REG_INT_STATUS ); 
            /*
            // set the specific pin value
            Reg_Set32( pGPIOStruct->u8PortNum , 0 , REG_DATA_VALUE ); 
            // set the specific pin direction in output
            Reg_Set32( pGPIOStruct->u8PortNum , 1 , REG_DIRECTION ); 
            */
            break;
            
        case INPUT_INT_RISING:    
            // disnable interrupt
            Reg_Set32( PinNum , 0 , REG_INT_ENABLE ); 
            // clean the specific pin interruprt status
            Reg_Set32( PinNum , 1 , REG_INT_STATUS ); 
            
            // Set rising-edge trigger     
            Reg_Set32( PinNum , 1 , REG_INT_TYPE0 ); 	
            Reg_Set32( PinNum , 0 , REG_INT_TYPE1 ); 	
            Reg_Set32( PinNum , 0 , REG_INT_TYPE2 ); 	
            
            // enable interrupt
            Reg_Set32( PinNum , 1 , REG_INT_ENABLE ); 
            break;
			
        case INPUT_INT_FAILING:    
            // disnable interrupt
            Reg_Set32( PinNum , 0 , REG_INT_ENABLE ); 
            // clean the specific pin interruprt status
            Reg_Set32( PinNum , 1 , REG_INT_STATUS ); 
            
            // Set failling-edge trigger     
            Reg_Set32( PinNum , 0 , REG_INT_TYPE0 ); 	
            Reg_Set32( PinNum , 0 , REG_INT_TYPE1 ); 	
            Reg_Set32( PinNum , 0 , REG_INT_TYPE2 ); 	
            
            // enable interrupt
            Reg_Set32( PinNum , 1 , REG_INT_ENABLE ); 
            break;
			
        case INPUT_INT_HIGH:    
            // disnable interrupt
            Reg_Set32( PinNum , 0 , REG_INT_ENABLE ); 
            // clean the specific pin interruprt status
            Reg_Set32( PinNum , 1 , REG_INT_STATUS ); 
            
            // Set high-level trigger     
            Reg_Set32( PinNum , 1 , REG_INT_TYPE0 ); 	
            Reg_Set32( PinNum , 1 , REG_INT_TYPE1 ); 	
            Reg_Set32( PinNum , 0 , REG_INT_TYPE2 ); 	
            
            // enable interrupt
            Reg_Set32( PinNum , 1 , REG_INT_ENABLE ); 
            break;                                    
            
        case INPUT_INT_LOW:    
            // disnable interrupt
            Reg_Set32( PinNum , 0 , REG_INT_ENABLE ); 
            // clean the specific pin interruprt status
            Reg_Set32( PinNum , 1 , REG_INT_STATUS ); 
            
            // Set low-level trigger     
            Reg_Set32( PinNum , 0 , REG_INT_TYPE0 ); 	
            Reg_Set32( PinNum , 1 , REG_INT_TYPE1 ); 	
            Reg_Set32( PinNum , 0 , REG_INT_TYPE2 ); 	
            
            // enable interrupt
            Reg_Set32( PinNum , 1 , REG_INT_ENABLE ); 
            break;                                    
			
        case INPUT_INT_BOTH:    
            // disnable interrupt
            Reg_Set32( PinNum , 0 , REG_INT_ENABLE ); 
            // clean the specific pin interruprt status
            Reg_Set32( PinNum , 1 , REG_INT_STATUS ); 
            
            // Set double-edge trigger     
            Reg_Set32( PinNum , 1 , REG_INT_TYPE2 ); 	
            
            // enable interrupt
            Reg_Set32( PinNum , 1 , REG_INT_ENABLE ); 
            break;                                    

        case MULTI_FUNC_CFG:
            // Multi-Functiion Pin Configuration 
            MultiFunctionCfg( pGPIOStruct->u8PortNum ) ;    
            break;
			
        default:
            DRV_ERROR("aess_gpio_config()::return -STATUS_FAIL::command error");
            spin_unlock( &reg_lock );
            return -STATUS_FAIL;
    }                
    

    ShowRegValue("aess_gpio_config()");    
    
    spin_unlock( &reg_lock );
    
    DRV_INFO("aess_gpio_config()::return STATUS_OK");
    
    return (STATUS_OK);
	
}


/******************************************************************************
*   FUNCTION        :   aess_gpio_read
******************************************************************************/
/**
 *  @brief      Get gpio ping status.
 *
 *  @return     Return STATUS_OK if fucntion execute success.
 *              Return others if fucntion execute fail.
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function \n
 *
 *****************************************************************************/
static int aess_gpio_read(
                           /** sGPIOData structure point */
                           sGPIOData *pGPIOStruct
                          )
{
//    DRV_INFO("aess_gpio_read()::port=%d ;command=%d", pGPIOStruct->u8PortNum, pGPIOStruct->u8CommandNum);

    UINT8 PinNum = 0 ;

    spin_lock( &reg_lock );
    
    PinNum = (UINT8)pGPIOStruct->u8PortNum ;
    
    if( PinNum >= 64 ) // GPIO_Port_E2 
        PinNum -=32 ;
         
    switch(pGPIOStruct->u8CommandNum)
    {
        case NORMAL_INPUT:
            //chnage the specific pin in input direction REG_DATA_VALUE
            Reg_Set32( pGPIOStruct->u8PortNum , 0 , REG_DIRECTION );        
            // get the specific pin value
            *((UINT8*)pGPIOStruct->pData)=Reg_Get32( PinNum , REG_DATA_VALUE );
            break;

        case INT_STATUS_INPUT:
            // get the specific pin interrupt status 
            *((UINT8*)pGPIOStruct->pData)=Reg_Get32( PinNum , REG_INT_STATUS );
            break;

        case INT_ENABLE_INPUT:
            // get the specific pin interrupt status 
            *((UINT8*)pGPIOStruct->pData)=Reg_Get32( PinNum , REG_INT_ENABLE );
            break;

        case OUTPUT_INPUT:
            // get the specific pin value 
            *((UINT8*)pGPIOStruct->pData)=Reg_Get32( PinNum , REG_DATA_VALUE );
            break;

        case DIRECT_INPUT:
            // get the specific pin direction 
            *((UINT8*)pGPIOStruct->pData)=Reg_Get32( PinNum , REG_DIRECTION );
            break;
            
        default:
            DRV_ERROR("aess_gpio_read()::return -EIO::Not support pGPIOStruct->u8CommandNum=0x%x \n",pGPIOStruct->u8CommandNum);
            spin_unlock( &reg_lock );
            return -EIO;
    }

    ShowRegValue("aess_gpio_read()");    
    spin_unlock( &reg_lock );
    
    return (STATUS_OK);
}

/******************************************************************************
*   FUNCTION        :   aess_gpio_write
******************************************************************************/
/**
 *  @brief      Set gpio ping status.
 *
 *  @return     Return STATUS_OK if fucntion execute success.
 *              Return others if fucntion execute fail.
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function \n
 *
 *****************************************************************************/
static int aess_gpio_write(
                           /** sGPIOData structure point */
                            sGPIOData *pGPIOStruct
                          )
{
    unsigned int value ;
    UINT8 PinNum = pGPIOStruct->u8PortNum ;
    

    if( PinNum >= 64 ) // GPIO_Port_E2 
        PinNum -=32 ;
    
    if ( *(UINT8*)pGPIOStruct->pData != 0 )     
        value = 1 ;
    else
        value = 0 ;
    
    DRV_INFO("aess_gpio_write()::port=ping=%d\n", pGPIOStruct->u8PortNum);
    
    spin_lock( &reg_lock );
    
    switch(pGPIOStruct->u8CommandNum)
    {
        case NORMAL_OUTPUT:
            // set the output value
            Reg_Set32( PinNum , value , REG_DATA_VALUE );
            // chnage the specific pin direction pin output
            Reg_Set32( PinNum ,   1   , REG_DIRECTION );
            break ;
		
        case DEBOUNCE_OUTPUT:   
            // enable /disable De-bounce
            Set_Debounce( PinNum , value ) ;
            break;

        case CLR_INT_OUTPUT:    
            // clean interrupt status
            Reg_Set32( PinNum , 1 , REG_INT_STATUS );
            break;

        case EVENT_TYPE_OUTPUT:    
            // debug only :: retrun interrupt sensivity type 0 register value  
            Reg_Set32( PinNum , value , REG_EXT_INT_TYPE0 );
            break;

        case EVENT_POLARITY_OUTPUT:    
            // debug only :: retrun interrupt sensivity type 1 register value  
            Reg_Set32( PinNum , value , REG_EXT_INT_TYPE1 );
            break;

        case EVENT_TYPE2_OUTPUT:    
            // debug only :: retrun interrupt sensivity type 2 register value  
            Reg_Set32( PinNum , value , REG_EXT_INT_TYPE2 );
            break;

        case EVENT_ENABLE_OUTPUT:    
            // enable specific interrupt function  
            Reg_Set32( PinNum , value , REG_INT_ENABLE );
            break;

        case RESET_TOL_OUTPUT:    
            // enable specific interrupt function  
            Reg_Set32( PinNum , value , REG_INT_ENABLE );
            break;
		
        case GPIO_SELECTED_OUTPUT:    
        case BLINK_OUTPUT:
        case POWER_OUTPUT:    
        case OPEN_DRAIN_OUTPUT:   
            DRV_ERROR("aess_gpio_write()::command error:: chip not support\n");
            break ;
        
        default:
            DRV_ERROR("aess_gpio_write()::command error\n");
            spin_unlock( &reg_lock );
            return -EINVAL;
	}         

    ShowRegValue("aess_gpio_write()");    
    
    spin_unlock( &reg_lock );
    
    return (STATUS_OK);                   
    
}

/******************************************************************************
*   FUNCTION        :   aess_gpio_ioctl
******************************************************************************/
/**
 *  @brief      The API is invoked when user space application issues an IOCTL command. 
 *              The following is the list of commands that supported currently.
 *              1.	AESS_GPIODRV_R
 *                  Get gpio ping status.
 *              2.	AESS_GPIODRV_W
 *                  Get gpio ping status.
 *              3.	AESS_GPIODRV_CFG
 *                  Set gpio interrupt type.
 *
 *  @return     STATUS_OK if driver execute the command success.
 *              Others if execute the command fail.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function \n
 *
 *****************************************************************************/
static int aess_gpio_ioctl(
                                 /** inode structure point */
                              struct inode *inode, 
           
                              /** file structure point */
                              struct file *filp,
           
                              /** command from user space */
                              unsigned int cmd, 
    
                              /** sBIOSPostInfo structure point */
                              unsigned long arg
                            )
{
    int err_check;
    sGPIOData *pGPIOStruct = (sGPIOData *) arg;

    if (S_u8init_flag == AESSGPIO_NOT_INIT)
    {
        aess_gpio_open(0,0);
    }

    /* Check the structure address is OK */
    if(!access_ok(VERIFY_WRITE, (void *) pGPIOStruct, sizeof(sGPIOData)))
    {
        DRV_ERROR("aess_gpio_ioctl()::return -EFAULT:: !access_ok(VERIFY_WRITE)\n");
        /* error occur */
        return -EFAULT;
    }

    DRV_INFO("pGPIOStruct:: information\n ");
    DRV_INFO("pGPIOStruct->u8CommandType= %d \n", pGPIOStruct->u8CommandType );
    DRV_INFO("pGPIOStruct->u8PortNum=%d \n", pGPIOStruct->u8PortNum );
    DRV_INFO("pGPIOStruct->u8PinNum=%d \n", pGPIOStruct->u8PinNum )

    MultiFunctionCfg( pGPIOStruct->u8PortNum ) ;    
	  
    switch(cmd)
    {
        case AESS_GPIODRV_R:
            pGPIOStruct->u8CommandType = GPIO_READ;
            {        
                /* Check the buffer address exist in the structure is OK */
                if(!access_ok(VERIFY_READ, pGPIOStruct->pData, sizeof(UINT8)))
                {
                    DRV_ERROR("aess_gpio_ioctl()::return -EFAULT:: !access_ok(VERIFY_READ)\n");
                    /* error occur */
                    err_check = -EFAULT;
                }        
                else
                    err_check = aess_gpio_read(pGPIOStruct);
            }
            break;
			
		case AESS_GPIODRV_W:
            pGPIOStruct->u8CommandType = GPIO_WRITE;
            {
                /* Check the buffer address exist in the structure is OK */
                if(!access_ok(VERIFY_WRITE, pGPIOStruct->pData, sizeof(UINT8)))
                {
                    err_check = -EFAULT;
                }
                else
                    err_check = aess_gpio_write(pGPIOStruct);         
            }
			break;
			
		case AESS_GPIODRV_CFG:
            pGPIOStruct->u8CommandType = GPIO_CONFIG;
            {
                err_check = aess_gpio_config(pGPIOStruct);         
            }
            break;
            
        default:
            DRV_ERROR("aess_gpio_ioctl()::command error\n");
            err_check = -EINVAL;
            break;
    }
	
    /* 0->ok, minus->fail */
    return err_check;
}

static struct file_operations aess_gpio_fops = 
{
    .open = aess_gpio_open, 
    .ioctl = aess_gpio_ioctl,
    .release = aess_gpio_close,     
};


/******************************************************************************
*   FUNCTION        :   aess_gpio_init
******************************************************************************/
/**
 *  @brief      The function is invoked when biospost module is loaded into the kernel.
 *
 *  @return     STATUS_OK if driver initial success.
 *              Others if initial fail.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function  \n
 *
 *****************************************************************************/
int __init aess_gpio_init(void)
{
    int result;

    gpio_cdev = cdev_alloc();
    gpio_cdev->ops = &aess_gpio_fops;
    result = alloc_chrdev_region(&gpio_dev, 0, 1, driver_name);
    
    printk("mknod /dev/aess_gpiodrv c %d 0\n", MAJOR(gpio_dev));
    
    if (result < 0) 
    {
        printk("KN:Registering the GPIO character device failed with %d\n", MAJOR(gpio_dev));
        return result;
    }
    
    cdev_add(gpio_cdev, gpio_dev, 1);
    
    /* initial debugfs related resource */
    aess_debugfs_default_create(driver_name,NULL,0,0);
    aess_debugfs_create_file(driver_name,"u32",(u32 *)&u32tmpcounter,DBG_TYPE32,0);
    aess_debugfs_create_file(driver_name,"u16",(u16 *)&u16tmpcounter,DBG_TYPE16,0);
    aess_debugfs_create_file(driver_name,"u8",(u8 *)&u8tmpcounter,DBG_TYPE8,0);	
    
    DRV_INFO("aess_gpio_init()::return (STATUS_OK)");
    return (STATUS_OK);
}

/******************************************************************************
*   FUNCTION        :   aess_gpio_exit
******************************************************************************/
/**
 *  @brief      The function is invoked when biospost driver be removed from kernal.
 *
 *  @return     None.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function \n
 *
 *****************************************************************************/
static void __exit aess_gpio_exit(void)
{
    DRV_INFO("aess_gpio_exit():: return \n");
    cdev_del(gpio_cdev);
    unregister_chrdev_region(gpio_dev, 1);
    
	/* remove debugfs related resource */
    aess_debugfs_remove(driver_name,NULL,0,0); 	    
}


/******************************************************************************
*   FUNCTION        :   aess_gpio_GetEventPin
******************************************************************************/
/**
 *  @brief      The function is report the gpio ping interrput status .
 *
 *  @return     None.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function \n
 *
 *****************************************************************************/
int aess_gpio_GetEventPin( 
                           /** gpio P0~P31 interrupt status */ 
                           UINT32* pValue ,

                           /** gpio P32~P63 interrupt status */ 
                           UINT32* pValueExt
                           )
{
    int i =0 ;

    *pValue = ioread32( REG_INT_STATUS ) ;
    if( (int)*pValue !=0 ) i++ ;       // GPIO interrupt found! ( pin 0~31 )
    *(pValueExt ) = ioread32( REG_EXT_INT_STATUS ) ;
    if( (int)*(pValueExt ) !=0 ) i++ ;   // GPIO interrupt found! ( pin 32~63 )

    return i ;
     
}

MODULE_DESCRIPTION("AESS GPIO Driver");
MODULE_AUTHOR("Taiyi Kuo <taiyi.kuo@avocent.com>");
MODULE_LICENSE("GPL");
MODULE_VERSION("0.1.0.0");
module_init(aess_gpio_init);
module_exit(aess_gpio_exit);
module_param(driver_name, charp, S_IRUGO);
EXPORT_SYMBOL(aess_gpio_commander);
EXPORT_SYMBOL(aess_gpio_GetEventPin);


/* End of code */
