/**
 * $Id$
 *
 *<center>
 *               Avocent Corporation. Proprietary Information.
 * \n<em>
 *      This software is supplied under the terms of a license agreement or
 *      nondisclosure agreement with Avocent Corporation, or  one of its 
 *      affiliates, andaffiliates, and may not be used, disseminated, or 
 *      distributed except in accordance with the terms of that agreement.
 *
 *      3541 Gateway Boulevard Fremont, San Jose, CA 94538 U.S.A.
 *\n
 *                  US phone: 510-771-6100
 *
 *        Copyright &copy; 2001-2009 Avocent Corporation. All rights reserved.
 *</em> </center>
 *----------------------------------------------------------------------------\n
 *  MODULES     Linux device Driver                                           \n
 *----------------------------------------------------------------------------\n
 *  @file   aess_gpiodrv.h
 *  @brief  gpio driver 
 *  @internal
 *  $RCSfile: aess_gpiodrv.h,v $
 *  $Revision: $
 *  $Date: 2009/03/10 13:44:48 $
 *  $Author: Taiyi Kuo $
 *  $Source: $
 *  $Name:  $
 *----------------------------------------------------------------------------*/

#if !defined (AESSGPIODRV_H)
#define AESSGPIODRV_H


/***********************************************************/
/* Global include area                                     */
/* GPIO command type                                       */
/***********************************************************/
#define GPIO_CONFIG    0 
#define GPIO_WRITE     1
#define GPIO_READ      2

/* GPIO-config command number */
#define INPUT_INT_OFF            0x0
#define INPUT_INT_RISING         0x1
#define INPUT_INT_FAILING        0x2
#define INPUT_INT_HIGH           0x3
#define INPUT_INT_LOW            0x4
#define INPUT_INT_BOTH           0x5
#define MULTI_FUNC_CFG           0x6

/* GPIO-input command number */
#define NORMAL_INPUT            0x0
#define INT_STATUS_INPUT				0x1
#define INT_ENABLE_INPUT				0x2
#define OUTPUT_INPUT						0x4
#define DIRECT_INPUT						0x5

/* GPIO-output command number */
#define NORMAL_OUTPUT            0x0
#define OPEN_DRAIN_OUTPUT        0x1
#define POWER_OUTPUT             0x2
#define BLINK_OUTPUT             0x3
#define DEBOUNCE_OUTPUT          0x4
#define CLR_INT_OUTPUT           0x5
#define EVENT_TYPE_OUTPUT        0x6
#define EVENT_POLARITY_OUTPUT    0x7
#define EVENT_ENABLE_OUTPUT      0x8
#define GPIO_SELECTED_OUTPUT     0x9
#define EVENT_TYPE2_OUTPUT    	 0xa
#define RESET_TOL_OUTPUT         0xb

/* Command value and mask */
#define SET_GPIO_OUTPUT_LOW          0x00000000
#define SET_GPIO_OUTPUT_HIGH         0x00000001

#define Function_Enabled         0x00000001
#define Function_Disabled        0x00000001


/* ioctl definitions */
#define AESS_GPIODRV_IOC_MAGIC    0xB5
#define AESS_GPIODRV_R           _IOWR(AESS_GPIODRV_IOC_MAGIC, 0, int)
#define AESS_GPIODRV_W           _IOWR(AESS_GPIODRV_IOC_MAGIC, 1, int)
#define AESS_GPIODRV_CFG         _IOWR(AESS_GPIODRV_IOC_MAGIC, 2, int)

/******************************************************************************
*   STRUCT      :   sGPIOData
******************************************************************************/
/**
 *  @brief   Structure to GPIO driver related data parameter.
 *
 *****************************************************************************/
typedef struct
{
    /** Set read/write command type */
    UINT8 u8CommandType;
    
    /** Set command number */
    UINT8 u8CommandNum;	
    
    /** Set GPIO port number */
    UINT8 u8PortNum;       
    
    /** Set GPIO Pin number */
    UINT8 u8PinNum;        
    
    /** Data buffer */
    void *pData;			    

} sGPIOData;

#define GPIO_PIN_MAX                 64+8


/***********************************************************/
#ifdef AESSGPIODRV_C
/***********************************************************/

//#ifdef AST2100

#include <asm/ioctl.h>
#include <linux/spinlock.h>
#include <asm/arch/irqs.h>
#include <asm/uaccess.h>
#include <asm/arch/hardware.h>
#include <asm/io.h>

//#endif //AST2100

#include <asm-arm/arch-ast2050/hw_regs.h>

#define MAX_CONFIG_COMMAND      6
#define MAX_READ_COMMAND      	3
#define MAX_WRITE_COMMAND       12  

/* For temporary use, because no type.h now */
#define STATUS_OK     0
#define STATUS_FAIL   1

/* init flag */
#define AESSGPIO_NOT_INIT 0
#define AESSGPIO_INIT_OK  1


/* Command value and mask */
#define SET_GPIO_INPUT_INT_OFF       0x00000002
#define SET_GPIO_INPUT_INT_RISING    0x00000006
#define SET_GPIO_INPUT_INT_FAILING   0x0000000A
#define SET_GPIO_INPUT_INT_BOTH      0x0000000E

#define GPIO_DATA_MASK               0x00000001
#define GPIO_DIRECTION_MASK          0x00000002

#define GPIO_PORT_MASK               0x000000E0
#define GPIO_PIN_MASK                0x0000001F
#define GPIO_PORT_SHIFT              0x5

#define GPIO_BLINK_DATA_MASK         0x7


/* 
 *  All register address defined move to /kernel/include/asm-arm/arch-ast2050/hw_regs.h
 */
 
/* GPIO port base address */
/*
#define GPIO_PORT_BASE_ADDR         IO_ADDRESS(ASPEED_GPIO_BASE)

#define VIC_BASE_ADDR               IO_ADDRESS(ASPEED_VIC_BASE)
#define SCU_BASE_ADDR               IO_ADDRESS(ASPEED_SCU_BASE)

#define SCU_PKR                     (SCU_BASE_ADDR + 0x00)
#define SCU_MULTI_FUNTION1_ADDR     (SCU_BASE_ADDR + 0x74)

#define VIC_IRQ_STATUS_ADDR         (VIC_BASE_ADDR + 0x0)
#define VIC_IRQ_RAW_STATUS_ADDR     (VIC_BASE_ADDR + 0x08)
#define VIC_IRQ_SELECTION_ADDR      (VIC_BASE_ADDR + 0x0c)
#define VIC_IRQ_ENABLE_ADDR         (VIC_BASE_ADDR + 0x10)
#define VIC_IRQ_SENS_ADDR           (VIC_BASE_ADDR + 0x24)
#define VIC_IRQ_TRIGGER_ADDR        (VIC_BASE_ADDR + 0x28)
#define VIC_IRQ_EVENT_ADDR          (VIC_BASE_ADDR + 0x2c)
*/
/* GPIO port base address */
/*
#define REG_DATA_VALUE    (GPIO_PORT_BASE_ADDR + 0x0) 
#define REG_DIRECTION     (GPIO_PORT_BASE_ADDR + 0x4)
#define REG_INT_ENABLE    (GPIO_PORT_BASE_ADDR + 0x8)
#define REG_INT_TYPE0     (GPIO_PORT_BASE_ADDR + 0xc)
#define REG_INT_TYPE1     (GPIO_PORT_BASE_ADDR + 0x10)
#define REG_INT_TYPE2     (GPIO_PORT_BASE_ADDR + 0x14)
#define REG_INT_STATUS    (GPIO_PORT_BASE_ADDR + 0x18)
#define REG_RESET_TOLERANT (GPIO_PORT_BASE_ADDR + 0x1c)

#define REG_EXT_DATA_VALUE (GPIO_PORT_BASE_ADDR + 0x20)
#define REG_EXT_DIRECTION  (GPIO_PORT_BASE_ADDR + 0x24)
#define REG_EXT_INT_ENABLE (GPIO_PORT_BASE_ADDR + 0x28)
#define REG_EXT_INT_TYPE0  (GPIO_PORT_BASE_ADDR + 0x2c)
#define REG_EXT_INT_TYPE1  (GPIO_PORT_BASE_ADDR + 0x30)
#define REG_EXT_INT_TYPE2  (GPIO_PORT_BASE_ADDR + 0x34)
#define REG_EXT_INT_STATUS (GPIO_PORT_BASE_ADDR + 0x38)
#define REG_EXT_RESET_TOLERANT  (GPIO_PORT_BASE_ADDR + 0x3c)

#define REG_DEBOUNCE1      (GPIO_PORT_BASE_ADDR + 0x40)
#define REG_DEBOUNCE2      (GPIO_PORT_BASE_ADDR + 0x44)

#define REG_EXT_DEBOUNCE1  (GPIO_PORT_BASE_ADDR + 0x48)
#define REG_EXT_DEBOUNCE2  (GPIO_PORT_BASE_ADDR + 0x4c)

#define REG_DEBOUNCE_TIME1 (GPIO_PORT_BASE_ADDR + 0x50)
#define REG_DEBOUNCE_TIME2 (GPIO_PORT_BASE_ADDR + 0x54)
#define REG_DEBOUNCE_TIME3 (GPIO_PORT_BASE_ADDR + 0x58)
*/

/******************************************************************************
*   STRUCT      :   Function Prototype 
******************************************************************************/
/**
 *  @brief   Prototype for each private function.
 *
 *****************************************************************************/
static int aess_gpio_write( 
                            /** GPIO Data Structure */
                            sGPIOData *psdata
                          );
  

static int aess_gpio_read( 
                           /** GPIO Data Structure */
                           sGPIOData *psdata
                         );

static int aess_gpio_config( 
                             /** GPIO Data Structure */
                             sGPIOData *psdata
                           );

int aess_gpio_commander(
                        /** bit 7~4->port, 3-0->pin */
                        UINT8 u8PortPin,  

                        /** Reference to command list */
                        UINT8 u8CommandType,

                        /** Reference to command list */
                        UINT8 u8CommandNum,

                        /** Data buffer, right now is UINT8 */
                        void *pData
                      );

#else
/* For external API to use */
extern int aess_gpio_commander(
                                /** bit 7~4->port, 3-0->pin */
                                UINT8 u8PortPin, 

                                /** Reference to command list */
                                UINT8 u8CommandType,

                                /** Reference to command list */
                                UINT8 u8CommandNum,

                                /** Data buffer, right now is UINT8 */
                                void *pData
                              );

//extern int aess_gpio_GetEventPin();
extern int aess_gpio_GetEventPin( UINT32* pValue ,UINT32* pValueExt);

#endif   /* AESSGPIODRV_C */

#endif   /* AESSGPIODRV_H */


/* End of code */
