/**
 * $Id$
 *
 *<center>
 *               Avocent Corporation. Proprietary Information.
 * \n<em>
 *      This software is supplied under the terms of a license agreement or
 *      nondisclosure agreement with Avocent Corporation, or  one of its 
 *      affiliates, andaffiliates, and may not be used, disseminated, or 
 *      distributed except in accordance with the terms of that agreement.
 *
 *      3541 Gateway Boulevard Fremont, San Jose, CA 94538 U.S.A.
 *\n
 *                  US phone: 510-771-6100
 *
 *        Copyright &copy; 2001-2009 Avocent Corporation. All rights reserved.
 *</em> </center>
 *----------------------------------------------------------------------------\n
 *  MODULES     Linux device Driver                                           \n
 *----------------------------------------------------------------------------\n
 *  @file   aess_kcsdrv.h
 *  @brief  KCS driver and interrupt service routine
 *  @internal
 *  $RCSfile: aess_kcsdrv.h,v $
 *  $Revision: 1.1.1.1 $
 *  $Date: 2009/03/10 13:44:48 $
 *  $Author: Taiyi Kuo $
 *  $Source: //tpe-subversion/AESSTPE/puti/puti_fw/ProjectBase/trunk/platform/ast2050/evb/src/drivers/kcs/kcs_driver,v $
 *  $Name:  $
 *----------------------------------------------------------------------------*/

 
 
#ifndef AESSKCSDRV_H
#define AESSKCSDRV_H

/* IOCTL command */
#define AESS_KCSDRV_IOC_MAGIC    0xBA
#define AESS_KCSDRV_INIT         _IOWR(AESS_KCSDRV_IOC_MAGIC, 0, int)
#define AESS_KCSDRV_READ         _IOWR(AESS_KCSDRV_IOC_MAGIC, 1, int)
#define AESS_KCSDRV_WRITE        _IOWR(AESS_KCSDRV_IOC_MAGIC, 2, int)
#define AESS_KCSDRV_SWSMI        _IOWR(AESS_KCSDRV_IOC_MAGIC, 3, int)
#define AESS_KCSDRV_SETCBID      _IOWR(AESS_KCSDRV_IOC_MAGIC, 4, int)

#define HIF_MAX_HOOK_FUNC_NUM 4
typedef void (* HIFLRSTHookAPIPtr) (void);


/* Driver related description */
#define DRIVER_MANDATORY
#define KCS_DRIVER_NAME "aess_kcsdrv"
#define KCS_DRIVER_MODULE "aess_kcsdrv.ko"
#define DRIVER_TYPE "CHAR"

#define KCS_NOT_INIT               1
#define KCS_INIT_OK                0

#define KCS_CH_INIT                1
/* Host interface definition */

/* KCS system attention bits definition */
#define SMS_ATN_SET         1

/* KCS interface package and structure definition */
#define KCS_PACKAGE_MAX             0x28
#define KCS_TOTAL_CHANNEL           0x03



/******************************************************************************
*   STRUCT      :    sKCSInfo
******************************************************************************/
/**
 *  @brief    KCS Parameters definition from User space
 *
 *****************************************************************************/
typedef struct 
{
    /** Receive complete event */
    UINT32 u32KCSRxOKEvent;
    
    /** Transmit done event */
    UINT32 u32KCSTxOKEvent;
    
    /** Transmit fail event */
    UINT32 u32KCSTxFailEvent;

    /** Callback Function event ID */
    UINT32 u32CBFunEventID;

    /** Read data size */
    UINT8 *pu8ReadLength;    
    
    /** Data buffer, IPMI command length only 40 bytes */
    UINT8 *pu8Data;

    /** KCS base address */
    UINT16 u16BaseAddress;

    /** Event group Driver ID */
    UINT16  u16DriverID;

    /** Callback Function event group Driver ID */
    UINT16 u16CBFunDriverID;

    /** Reserve */
    UINT16 u16Reserve;

    /** Channel number */
    UINT8 u8Channel;        
    
    /** Set for SMS_ATTEN Bit */
    UINT8 u8Control;
  
    /** Write data length */
    UINT8 u8WriteLength;
    
    /** Reserve */
    UINT8 u8Reserve;
    
} sKCSInfo;


#ifdef AESSKCSDRV_C 

/* For temporary use, because no type.h now */
#define STATUS_OK     0
#define STATUS_FAIL   1

/******************************************************************************
*   FUNCTION        :   aess_kcsdrv_bh 
******************************************************************************/
/**
 *  @brief      notify event handler driver.
 *
 *  @return     None
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Private function\n
 *
 *****************************************************************************/
static void aess_kcsdrv_bh(
                            /** Unused data */
                            UINT32 u32Data
                          );

#endif   /* AESSKCSDRV_C */

#endif   /* AESSKCSDRV_H */


/* End of code */
