/**
 *
 *               Avocent Corporation. Proprietary Information.
 *      This software is supplied under the terms of a license agreement or
 *      nondisclosure agreement with Avocent Corporation, or one of its affiliates, and
 *      may not be used, copied, disseminated, or distributed except in accordance
 *      with the terms of that agreement.
 *
 *      3541 Gateway Boulevard, Fremont, California, 94538 U.S.A.
 *                  US Phone: 510-771-6100
 *      Copyright 2001-2009 Avocent Corporation. All rights reserved.
 *
 *----------------------------------------------------------------------------
 *  MODULES     AESS MTD SPI NOR Flash Driver
 *----------------------------------------------------------------------------
 * @file    aess_mtddrv_spinor.c
 * @brief   c source file for mtd spi nor flash driver
 *----------------------------------------------------------------------------
 */
/****************************************************************************** 
 * Content
 * ----------------
 *       ast2050_smc_cs_low()	-   
 *       ast2050_smc_cs_high()	-
 *       ast2050_get_dev()	-
 *       ast2050_get_baddr()	-
 *       ast2050_get_smc_baddr()    -
 *       ast2050_smc_unlock_write_protection()	-
 *       ast2050_smc_uma_read()	    -
 *       ast2050_smc_uma_write()    -
 *       ast2050_smc_uma_page_write()	-
 *       ast2050_smc_wait_for_ready()	-
 *       spi_nor_read()	-
 *       spi_nor_write()	-
 *       spi_nor_erase()	-
 *       ast2050_smc_init() -
 *       spi_nor_probe()	-
 *       spi_nor_init()	-
 *       spi_nor_cleanup()	-
 *       aess_spi_init()    -
 *       aess_write_spi_nor_flash() -
 *       aess_read_spi_nor_flash()  -
 *
 *******************************************************************************/

#define AESS_MTDDRV_SPINOR_C

#include <linux/slab.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/delay.h>

#include <linux/mtd/partitions.h>
#include <asm/io.h>
#include <asm/sizes.h>

#include <asm/hardware.h>
#include <asm-arm/io.h>
#include <asm/uaccess.h>
#include "aess_mtddrv_spinor.h"
#include <linux/vmalloc.h>
#include <asm/arch-ast2050/hw_regs.h>
#include <linux/spinlock.h>

/* implement debug message with debugfs */
#include <linux/aess_debugfs.h>


#define CONFIG_AST2050_EVB


#ifdef CONFIG_MTD_PARTITIONS
#define	mtd_has_partitions()	(1)
#else
#define	mtd_has_partitions()	(0)
#endif


static struct mtd_info *spi_nor_mtd = NULL;
static u8 spi_flash_id = 0;
static struct flash_chip *spi_nor_mtd_lock = NULL;
struct flash_t *flash_info = NULL;

#ifdef CONFIG_AST2050_EVB

#define  FLASH_SIZE_16M        	SZ_16M
#define  FLASH_SECT_SIZE_64K    SZ_64K
#define  FLASH_SECT_SIZE_256K   SZ_256K

static flash_t spi_flash_data[] = 
{
    /* MF_ID,               Capacity ID,       Chip name,   Chip Size,      Sector Size */
    {FLASH_S25FL128P_MF_ID, FLASH_CAPACITY_ID, "s25fl128p", FLASH_SIZE_16M, FLASH_SECT_SIZE_64K},
    {FLASH_MX25L128D_MF_ID, FLASH_CAPACITY_ID, "mx25L128d", FLASH_SIZE_16M, FLASH_SECT_SIZE_64K},
    {FLASH_STM25P128_MF_ID, FLASH_CAPACITY_ID, "stm25p128", FLASH_SIZE_16M, FLASH_SECT_SIZE_256K}
};


#define  FLASH_CE0_DEVICE_ID        0x01
#define  FLASH_CE1_DEVICE_ID        0x02
#define  FLASH_CE2_DEVICE_ID        0x00
#define  FLASH_DEV_ERROR            0xFF
#define  BASE_ADDR_ERROR            0xFFFFFFFF

static u8 ast2050_flash_enable_list[] = {FLASH_CE2_DEVICE_ID, FLASH_CE0_DEVICE_ID};
static u8 ast2050_flash_total_num = (sizeof(ast2050_flash_enable_list) / sizeof(u8));

/*
 * Define partitions for flash devices
 */
static struct mtd_partition s25fl128_spi_partition_info[] = {
	{ .name     = "AST2050 SPI FLASH (U-BOOT)",
	  .offset   = 0x000000,
	  .size     = 256 * SZ_1K },
	{ .name     = "AST2050 SPI FLASH (Private Storage)",
	  .offset   = 0x040000, 
	  .size     = 768 * SZ_1K },
	{ .name     = "AST2050 SPI FLASH (Kernel)",
	  .offset   = 0x100000,    
	  .size     = 2048 * SZ_1K },
	{ .name     = "AST2050 SPI FLASH (Rootfs)",
	  .offset   = 0x300000,
	  .size     = 13312 * SZ_1K },
};

static struct mtd_partition mx25L128d_spi_partition_info[] = {
	{ .name     = "AST2050 SPI FLASH (U-BOOT)",
	  .offset   = 0x000000,
	  .size     = 256 * SZ_1K },
	{ .name     = "AST2050 SPI FLASH (Private Storage)",
	  .offset   = 0x040000, 
	  .size     = 768 * SZ_1K },
	{ .name     = "AST2050 SPI FLASH (Kernel)",
	  .offset   = 0x100000,    
	  .size     = 2048 * SZ_1K },
	{ .name     = "AST2050 SPI FLASH (Rootfs)",
	  .offset   = 0x300000,
	  .size     = 13312 * SZ_1K },
};

static struct mtd_partition stm25p128_spi_partition_info[] = {
	{ .name     = "AST2050 SPI FLASH (U-BOOT)",
	  .offset   = 0x000000,
	  .size     = 256 * SZ_1K },
	{ .name     = "AST2050 SPI FLASH (Private Storage)",
	  .offset   = 0x040000, 
	  .size     = 768 * SZ_1K },
	{ .name     = "AST2050 SPI FLASH (Kernel)",
	  .offset   = 0x100000,    
	  .size     = 2048 * SZ_1K },
	{ .name     = "AST2050 SPI FLASH (Rootfs)",
	  .offset   = 0x300000,
	  .size     = 13312 * SZ_1K },
};

#endif


static char *driver_name="aess_mtddrv";
static u8   u8tmpcounter = 0;
static u16 u16tmpcounter = 0;
static u32 u32tmpcounter = 0;

// added another function layer for init to avoid build warning
int aess_spi_init (void);

/*spin lock*/
spinlock_t aess_spi_lock;


/******************************************************************************
 *       FUNCTION        :       ast2050_smc_cs_low
 ******************************************************************************/
/**
 *      @brief          Pull low chip select pin by setting the ASPEED1100 SMC registers(0x04, 0x0C)
 *
 *      @return         None
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Private function
 *
 *****************************************************************************/
static void ast2050_smc_cs_low(
				/** Select the flash device (1 for FLASH_CE0 and 2 for FLASH_CE2) */
				u8 ce
    			    )
{
    switch(ce)
    {
        case FLASH_CE0_DEVICE_ID:
            iowrite32((EN_USERMODE | USER_MODE), SMC_CE0_V_ADDR);
            break;
            
        case FLASH_CE2_DEVICE_ID:
            iowrite32((EN_USERMODE | USER_MODE), SMC_CE2_V_ADDR);
            break;	
            
        default:
            DRV_ERROR("%s, Invaild CE vaule.\n", __FUNCTION__);
            break;
    }

}	


/******************************************************************************
 *       FUNCTION        :       ast2050_smc_cs_high
 ******************************************************************************/
/**
 *      @brief          Pull high chip select pin by setting the ASPEED1100 SMC registers(0x04, 0x0C)
 *
 *      @return         None
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Private function
 *
 *****************************************************************************/
static void ast2050_smc_cs_high(
				/** Select the flash device (1 for FLASH_CE0 and 2 for FLASH_CE2) */
				u8 ce
			    )
{
    switch(ce)
    {
        case FLASH_CE0_DEVICE_ID:
            iowrite32((DIS_USERMODE | USER_MODE), SMC_CE0_V_ADDR);
            break;
            
        case FLASH_CE2_DEVICE_ID:
            iowrite32((DIS_USERMODE | USER_MODE), SMC_CE2_V_ADDR);
            break;	
            
        default:
            DRV_ERROR("%s, Invaild CE vaule.\n", __FUNCTION__);
            break;
    }

}


/******************************************************************************
 *       FUNCTION        :       ast2050_get_dev
 ******************************************************************************/
/**
 *      @brief          Get flash device number by passing the address offset
 *
 *      @return         FLASH device number (FLASH_CE0 or FLASH_CE2)
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Private function
 *
 *****************************************************************************/
static u8 ast2050_get_dev(
			    /** address offset */
			    u32 addr
			)
{
    u8 dev;
    
    dev = (addr / flash_info->chip_size);

    if (dev >= ast2050_flash_total_num)
    {
	DRV_ERROR("%s: The addr is overflow, dev: 0x%x.\n", __FUNCTION__, dev);
        return (FLASH_DEV_ERROR);
    }
    
    return (ast2050_flash_enable_list[dev]);			
}


/******************************************************************************
 *       FUNCTION        :       ast2050_get_baddr
 ******************************************************************************/
/**
 *      @brief          Get flash device virtual base address by passing the flash device number
 *
 *      @return         flash device virtual base address or negative when fail
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Private function
 *
 *****************************************************************************/
static u32 ast2050_get_baddr(
				/** Select the flash device (1 for FLASH_CE0 and 2 for FLASH_CE2) */
				u8 ce
			    )
{
    if(ce == FLASH_CE0_DEVICE_ID)
    {
        return SM_B0_V_ADDR;
    }
    
    if(ce == FLASH_CE2_DEVICE_ID)
    {
        return SM_B2_V_ADDR;
    }
    
    return BASE_ADDR_ERROR;
}


// This function is not in use now. We only keep it here in case someone need
// to use it later on.
#if 0
/******************************************************************************
 *       FUNCTION        :       ast2050_get_smc_baddr
 ******************************************************************************/
/**
 *      @brief          Get ast2050 SMC register address by passing the flash device number
 *
 *      @return         ast2050 SMC register address or negative when fail
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Private function
 *
 *****************************************************************************/
static u32 ast2050_get_smc_baddr(
				    /** Select the flash device (1 for FLASH_CE0 and 2 for FLASH_CE2) */
				    u8 ce
				)
{
    if(ce == FLASH_CE0_DEVICE_ID)
    {
        return SMC_CE0_V_ADDR;
    }
    
    if(ce == FLASH_CE2_DEVICE_ID)
    {
        return SMC_CE2_V_ADDR;
    }	
    
    return BASE_ADDR_ERROR;	
}
#endif

/******************************************************************************
 *       FUNCTION        :       ast2050_smc_unlock_write_protection
 ******************************************************************************/
/**
 *      @brief          Remove all write protection from the Firmware Boot Block
 *
 *      @return         None
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Private function
 *
 *****************************************************************************/
void ast2050_smc_unlock_write_protection(void) 
{
    int i;
    u8 dev;
    u32 baddr;
    
    for (i = 0; i < ast2050_flash_total_num; i++)
    {
        dev = ast2050_flash_enable_list[i];
        
        ast2050_smc_cs_low(dev);

        baddr = ast2050_get_baddr(dev);

	if(baddr != BASE_ADDR_ERROR)
	{
            /*write status register*/
            iowrite8(0x01, baddr);
            iowrite8(0x00, baddr);
        }
        ast2050_smc_cs_high(dev);
    }
}


/******************************************************************************
 *       FUNCTION        :       ast2050_smc_uma_read
 ******************************************************************************/
/**
 *      @brief          Read up to sizeof(u32) bytes from the flash.
 *
 *      @return         None
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Private function
 *
 *****************************************************************************/
void ast2050_smc_uma_read(
		    /** Select the flash device (0 or 1) to be accessed */
		    u8 device,
		    /** Specify the SPI transaction code */
		    u8 transaction_code,
		    /** Location on the flash , in the flash address space */
		    u32 address,
		    /** if TRUE, 3 bytes address will be passed in else 
			(FALSE), no address for this SPI transaction */
		    BOOL address_size,
		    /** a pointer to a data buffer to hold the read data. */
		    u8 * data,
		    /** buffer size. Legal sizes are 1~sizeof(u32)' */
		    u32 data_size
			)
{
	u8 reg;
	u32 usID;
	u32 baddr = ast2050_get_baddr(device);

	if(baddr == BASE_ADDR_ERROR)
	{
	    DRV_ERROR("%s: Invalid device number 0x%x.\n", __FUNCTION__, device);
	    return;
	}

	switch (transaction_code){
		
		case SPI_READ_DATA_CMD:

			if(address_size ){
				u32 i;

				ast2050_smc_cs_low(device);  /*chip select*/
				iowrite8( SPI_READ_DATA_CMD ,baddr);
				iowrite8( ((address&0xff0000)>>16),	baddr);
				iowrite8( ((address&0xff00)>>8),	baddr);
				iowrite8( (address&0xff),			baddr);	
				for( i=0 ; i<data_size ; i++){
				  data[i] = ioread8(baddr);
				} 
				ast2050_smc_cs_high(device);  /*chip select*/
			}     
			break;
			
	 	case SPI_READ_STATUS_REG_CMD:		
			/*Check register status*/
			DRV_MSG("%s: Check register status.\n", __FUNCTION__);
			ast2050_smc_cs_low(device);      /*chip select*/
			iowrite8( SPI_READ_STATUS_REG_CMD,baddr);	/*SM_V_ADDR*/
			reg =	ioread8(baddr);			/*SM_V_ADDR*/
			*data = reg;
			ast2050_smc_cs_high(device);	/*chip disselect*/
			break;
  
		case SPI_READ_JEDEC_ID_CMD:
			DRV_MSG("%s: Read jedec ID.\n", __FUNCTION__);
			ast2050_smc_cs_low(device);      /*chip select*/
			/*Write to base*/
			iowrite8( SPI_READ_JEDEC_ID_CMD,baddr);

			usID = ioread32(baddr);	
			if(data_size){
				int i;
				/*Becareful that BIG ENDIAN and LITTLE ENDDIAN*/
				for (i=data_size; i>0; i--){
				    data[i-1] = (usID >>(8*(i-1)))&0xff;
				}
			}		
			ast2050_smc_cs_high(device);	/*chip disselect*/	
			
			break;

		default:
			DRV_ERROR("%s: Invaild transaction_code vaule 0x%x.\n", __FUNCTION__, transaction_code);
			break;
	}
}

/******************************************************************************
 *       FUNCTION        :       ast2050_smc_uma_write
 ******************************************************************************/
/**
 *      @brief          Write up to 4 bytes to the flash. however, this function
 *			is not only for read, it also support sector erase, 
 *			write enable and disable, write status register.
 *
 *      @return         None
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Private function
 *
 *****************************************************************************/
void ast2050_smc_uma_write(
		/** Select the flash device (1 or 2) to be accessed */
		int device,
		/** Specify the SPI transaction code */
		u8 transaction_code,
		/** Location on the flash, in the flash address space */
		u32 address,
		/** if TRUE, 3 bytes address will be passed in 
		     else (FALSE), no address for this SPI transaction */
		BOOL address_size,
		/** a pointer to a data buffer (buffer of bytes)*/
		u8 * data,
		/** data buffer size in bytes. Legal sizes are 0,1,2,3,4 */
		int data_size
			) 
{
	u32 baddr = ast2050_get_baddr(device);

	if(baddr == BASE_ADDR_ERROR)
	{
            DRV_ERROR("%s: Invalid device number 0x%x.\n", __FUNCTION__, device);
	    return;
	}

	switch (transaction_code){
		
		case SPI_WRITE_ENABLE_CMD:
			if(address_size==FALSE){
  				/*Write enable*/
				DRV_MSG("%s: Write enable.\n", __FUNCTION__);
  				ast2050_smc_cs_low(device);      /*chip select*/			
				iowrite8( SPI_WRITE_ENABLE_CMD,baddr);
			    ast2050_smc_cs_high(device);	/*chip disselect*/
				
			}
			break;

		case SPI_WRITE_DISABLE_CMD:
			if(address_size==FALSE){
				/*Write disable*/
				DRV_MSG("%s: Write disable.\n", __FUNCTION__);
  				ast2050_smc_cs_low(device);      /*chip select*/			
  				iowrite8( SPI_WRITE_DISABLE_CMD,baddr); 
			    ast2050_smc_cs_high(device);	/*chip disselect*/
				
			}
			break;
			
		case SPI_WRITE_STATUS_REG_CMD:
			if(address_size==FALSE){
				/*Write status register*/
				DRV_MSG("%s: Write status register.\n", __FUNCTION__);
				ast2050_smc_cs_low(device);				
				iowrite8( SPI_WRITE_STATUS_REG_CMD,baddr);/*write status register*/
				iowrite8( *data,baddr);
				ast2050_smc_cs_high(device);		
			}
			break;	
		case SPI_SECTOR_ERASE_CMD:
			if(address_size){
				/*erase sector*/
				DRV_MSG("%s: Erase sector.\n", __FUNCTION__);
				ast2050_smc_cs_low(device);  /*chip select*/				
				iowrite8( SPI_SECTOR_ERASE_CMD,baddr);/*Sector erase*/
				iowrite8( ((address>>16)&0xff),baddr);
				iowrite8( ((address>>8)&0xff),baddr);
				iowrite8( (address&0xff),baddr);
				ast2050_smc_cs_high(device);/*chip disselect*/			
			}
			break;	
		case SPI_PAGE_PRGM_CMD:
			if(address_size){
				int i;
				/*Write program*/
				ast2050_smc_cs_low(device);				

				iowrite8( SPI_PAGE_PRGM_CMD,baddr);/*Page mode*/
				iowrite8( ((address>>16)&0xff),baddr);
				iowrite8( ((address>>8)&0xff),baddr);
				iowrite8( (address&0xff),baddr);

				/* Data write here is only meant to write very
				 * small amount of data (a few bytes). If you
				 * use data write here, please be sure the data
				 * size is less than 256 bytes.
				 * To write large amount of data, use 
				 * ast2050_smc_uma_page_write() instead.
				 */
				for(i=0; i<data_size; i++){
					iowrite8( data[i],baddr);
				}
				ast2050_smc_cs_high(device);		
			}
			break;			
			

		default:
			DRV_ERROR("%s: Invaild transaction_code vaule 0x%x.\n", __FUNCTION__, transaction_code);
			break;
	}
}

/******************************************************************************
 *       FUNCTION        :       ast2050_smc_uma_page_write
 ******************************************************************************/
/**
 *      @brief          Write up to 256 bytes to the flash. however, this
 *			function is not only for read, it also support sector
 *			erase, write enable and disable, write status register.
 *
 *      @return         None
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Private function
 *
 *****************************************************************************/
void ast2050_smc_uma_page_write(
			/** Select the flash device (1 or 2) to be accessed */
                        int device,
			/** Location on the flash, in the flash address space */
                        u32 address,
			/** a pointer to a data buffer (buffer of bytes) */
                        u8 * data,
			/** data buffer size in bytes. Legal sizes are 0~256 */
                        int data_size
			    )
{
	u32 i;
	u32 baddr = ast2050_get_baddr(device);
	u8 from_addr[3];

	DRV_INFO("Enter %s, device=%d, data =0x%x, data_size=%d.\n", __FUNCTION__, device, *data, data_size);
	
	if(baddr == BASE_ADDR_ERROR)
	{
		DRV_ERROR("%s: Invalid device number 0x%x.\n", __FUNCTION__, device);
		return;
	}

	if((data_size == 0)  || data_size%sizeof(u32)){
		DRV_ERROR("%s: Invalid byte alignment.\n", __FUNCTION__);
		return;
	}
	if(data_size > 256){
		DRV_ERROR("%s: Invalid byte length.\n", __FUNCTION__);
		return;
	}

	from_addr[0] = (address >> 16)&0xff;
	from_addr[1] = (address >> 8)&0xff;
	from_addr[2] = address&0xff;
	
	ast2050_smc_cs_low(device);      /*chip select*/

	iowrite8( SPI_PAGE_PRGM_CMD,baddr);/*Page mode*/
  	iowrite8( from_addr[0],baddr);
  	iowrite8( from_addr[1],baddr);
  	iowrite8( from_addr[2],baddr);

	for( i =0 ; i< data_size; i++)
	{		
		iowrite8(data[i],baddr);  //write to location
	}
  	ast2050_smc_cs_high(device);	  /*chip disselect*/

}

/******************************************************************************
 *       FUNCTION        :       ast2050_smc_wait_for_ready
 ******************************************************************************/
/**
 *      @brief          Read the status register to wait for ready
 *
 *      @return         0 if ready or -1 if not
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Private function
 *
 *****************************************************************************/
static int  ast2050_smc_wait_for_ready(
					/** Select the flash device (1 or 2) to be accessed */
					u8 device
				    ) 
{
    int count;
    u8 sr;

    for (count = 0; count < MAX_READY_WAIT_COUNT; count++)
    {
        ast2050_smc_uma_read(device,
                     SPI_READ_STATUS_REG_CMD,        
                     0, //NULL,
                     FALSE,
                     &sr,
                     1);
        
        if (!(sr & SR_WIP))
        {
            return (STATUS_OK);
        }
		
    }	
    return (STATUS_BUSY);
}

/******************************************************************************
 *       FUNCTION        :       spi_nor_read
 ******************************************************************************/
/**
 *      @brief          Read callback function for mtd_info
 *
 *      @return         0 if success
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Private function
 *
 *****************************************************************************/
static int spi_nor_read(
				/** mtd_info pointer */
				struct mtd_info *mtd,
				/** start address */
				loff_t from,
				/** length of reading */
				size_t len,
				/** number of byte read */
				size_t *retlen,
				/** a pointer to a data buffer (buffer of bytes) */
				u_char *buf
			    )
{
    u32 addr = (u32) from;
    u32 cnt = (u32) len;
    struct flash_chip *flash = mtd->priv;

    u8 dev;
    int align_size;
    int rest_size;
    u32 r_addr;
    u32 r_bytes = 0;
    u32 chip_addr;

    DRV_INFO("%s %s 0x%08x, len %zd.\n", __FUNCTION__, "from", (u32)from, len);

    /* Byte count starts at zero. */
    if (retlen)
    {
        *retlen = 0;
    }

    /* sanity checks */
    if (!len)
    {
        return(0);
    }

    if ((from + len) > mtd->size)
    {
        return -EINVAL;
    }
    
    down(&flash->lock);

    /* get flash bank information (find out which flash is going to be read ) */
    dev = ast2050_get_dev(addr);

    if (FLASH_DEV_ERROR == dev)
    {
        DRV_ERROR("%s, line %d: Wrong flash address access\n", __FUNCTION__, __LINE__);
	    up(&flash->lock);
	    return -EINVAL;
    }
    
    /* check if flash is busy */
    if (ast2050_smc_wait_for_ready(dev))
    {
        /* REVISIT status return?? */
        up(&flash->lock);
        *retlen = 0;
        return 1;
    }

    /* 1: Process even address. Ready a single byte*/
    if (addr & 0x01)
    {
	    DRV_MSG("%s: 1: Process even address: addr=0x%x.\n", __FUNCTION__, addr);
        ast2050_smc_uma_read(dev,          
                             SPI_READ_DATA_CMD, 
                             addr,
                             TRUE,  
                             &buf[0],   
                             1);   
        
        buf++;
        addr++;
        cnt--;  
    }
    
    /* set up destination of writing address */
    r_addr = addr;
    
    /* get flash bank information */
    dev = ast2050_get_dev(r_addr);

    if (FLASH_DEV_ERROR == dev)
    {
        DRV_ERROR("%s, line %d: Wrong flash address access\n", __FUNCTION__, __LINE__);
	    up(&flash->lock);
	    return -EINVAL;
    }

    align_size = cnt & (~0x03); /* amount whole 32-bit words */
    rest_size  = cnt & 0x03;    /* amount non 32-bit rest bytes */
    
    if (align_size != 0)
    {
        if ((addr & 0x000000FF) != 0)
        {
            r_bytes = 0x00000100 - (addr & 0x000000FF);
            
            if (r_bytes > align_size)
            {
                r_bytes = align_size;
            }
            
            /* 2. Process Partial Page */
	        DRV_MSG("%s: 2. Process Partial Page from addr=0x%x len=0x%x.\n", __FUNCTION__, r_addr, r_bytes);
            chip_addr = r_addr - (dev * flash_info->chip_size);
            ast2050_smc_uma_read(dev, SPI_READ_DATA_CMD, chip_addr, TRUE, (u8*)buf, (u32) r_bytes);  
            
            buf += r_bytes;
            r_addr += r_bytes;
            align_size -= r_bytes;

        }

        /* 3. Process Whole Page */
	    DRV_MSG("%s: 3. Process Whole Page from addr=0x%x len=0x%x.\n", __FUNCTION__, r_addr, align_size);
        while (align_size > 0)
        {
            /* get flash bank information */
            dev = ast2050_get_dev(r_addr);
            
            if (FLASH_DEV_ERROR == dev)
            {
		        DRV_ERROR("%s, line %d: Wrong flash address access\n", __FUNCTION__, __LINE__);
		        up(&flash->lock);
		        return -EINVAL;
            }

            /* page size is 256 bytes */
            if (align_size >= 0x100)
            {
                r_bytes = 0x100;
            }
            else
            {
                /* 4. Process the Finial aligned byte of Partial Page */
		        DRV_MSG("%s: 4. Process the Finial aligned byte of Partial Page from addr=0x%x len=0x%x.\n", __FUNCTION__, r_addr, align_size);
                r_bytes = align_size;
            }
            if (ast2050_smc_wait_for_ready(dev))
            {
                /* REVISIT status return?? */
                up(&flash->lock);
                *retlen = 0;               
                return 1;
            }

            chip_addr = r_addr - (dev * flash_info->chip_size);
            ast2050_smc_uma_read(dev, SPI_READ_DATA_CMD, chip_addr, TRUE, (u8*)buf, (u32)r_bytes);  
           
            buf += r_bytes;
            r_addr += r_bytes;
            align_size -= r_bytes;
            
        }
    }

    /* 5. Process the Finial non-aligned byte of Partial Page */
    if(rest_size != 0)
    {
	    DRV_MSG("%s: 5. Process the Finial non-aligned byte of Partial Page from addr=0x%x len=0x%x.\n", __FUNCTION__, r_addr, rest_size);
        for(; r_addr < (addr+cnt); r_addr++)
        {
            /* get flash bank information */
            dev = ast2050_get_dev(r_addr);

            if (FLASH_DEV_ERROR == dev)
            {
		        DRV_ERROR("%s, line %d: Wrong flash address access\n", __FUNCTION__, __LINE__);
		        up(&flash->lock);
		        return -EINVAL;
            }
            
            if (ast2050_smc_wait_for_ready(dev))
            {
                /* REVISIT status return?? */
                up(&flash->lock);
                *retlen = 0;           
                return 1;
            }

            chip_addr = r_addr - (dev * flash_info->chip_size);
            ast2050_smc_uma_read(dev, SPI_READ_DATA_CMD, chip_addr,TRUE,(u8*)buf,1);                     
            buf++;
        }
    }
    
    *retlen = len;
    up(&flash->lock);
	
    return 0;
}

/******************************************************************************
 *       FUNCTION        :       spi_nor_write
 ******************************************************************************/
/**
 *      @brief          Write callback function for mtd_info
 *
 *      @return         0 if success
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Private function
 *
 *****************************************************************************/
static int spi_nor_write(
				/** mtd_info pointer */
				struct mtd_info *mtd,
				/** start address */
				loff_t to,
				/** length of write */
				size_t len,
				/** number of byte written */
                            	size_t *retlen,
				/** a pointer to a data buffer (buffer of bytes) */
				const u_char *buf
			    )
{
    u32 addr = (u32) to;
    u32 cnt = (u32) len;
    struct flash_chip *flash = mtd->priv;
    u8 dev;
    int align_size;
    int rest_size;
    u32 w_addr;
    u32 w_bytes = 0;
    u32 chip_addr;

    DRV_INFO("%s %s 0x%08x, len %zd\n", __FUNCTION__, "to", (u32)to, len);

    /* Byte count starts at zero. */
    if (retlen)
    {
        *retlen = 0;
    }

    /* sanity checks */
    if (!len)
    {
        return(0);
    }

    if ((to + len) > mtd->size)
    {
        return -EINVAL;
    }
    
    down(&flash->lock);

    /* get flash bank information (find out which flash is going to be read )*/
    dev = ast2050_get_dev(addr);

    if (FLASH_DEV_ERROR == dev)
    {
	    DRV_ERROR("%s, line %d: Wrong flash address access\n", __FUNCTION__, __LINE__);
	    up(&flash->lock);
	    return -EINVAL;
    }

    /* check if flash is busy */
    if (ast2050_smc_wait_for_ready(dev))
    {
        /* REVISIT status return?? */
        up(&flash->lock);
        *retlen = 0;
        return 1;
    }

    /* 1: Process even address */
    if (addr & 0x01)
    {       
        ast2050_smc_uma_write(
                      dev,                   // only one flash device
                      SPI_WRITE_ENABLE_CMD,  // write enable transaction code
                      0,                     // address irrelevant            
                      FALSE,                 // no address for transaction    
                      NULL,                  // no write data                 
                      0);                    // no data                       
        
        if (ast2050_smc_wait_for_ready(dev))
        {
            /* REVISIT status return?? */
            up(&flash->lock);
            *retlen = 0;
            
            return 1;
        }

        ast2050_smc_uma_write(dev,               // only one flash device
                      SPI_PAGE_PRGM_CMD,            // write transaction code
                      addr,
                      TRUE,            // transaction has address 
                      (u8 *) buf,      // write data
                      1);              // data size
        
        buf++;
        addr++;
        cnt--;  
        
        if (ast2050_smc_wait_for_ready(dev))
        {
            /* REVISIT status return?? */
            up(&flash->lock);
            *retlen = 0;
            
            return 1;
        }
    }
    
    /* set up destination of writing address */
    w_addr = addr;
    
    /* get flash bank information */
    dev = ast2050_get_dev(w_addr);

    if (FLASH_DEV_ERROR == dev)
    {
	DRV_ERROR("%s, line %d: Wrong flash address access\n", __FUNCTION__, __LINE__);
	up(&flash->lock);
	return -EINVAL;
    }

    align_size = cnt & (~0x03); /* amount whole 32-bit words */
    rest_size  = cnt & 0x03;    /* amount non 32-bit rest bytes */
	
    if (align_size != 0)
    {
        if ((addr & 0x000000FF) != 0)
        {
			ast2050_smc_uma_write(
                          dev,                   // flash device num
                          SPI_WRITE_ENABLE_CMD,  // write enable transaction code
                          0,                     // address irrelevant
                          FALSE,                 // no address for transaction 
                          NULL,                  // no write data
                          0);                    // no data
            w_bytes = 0x00000100 - (addr & 0x000000FF);
            
            if (w_bytes > align_size)
            {
                w_bytes = align_size;
            }
            
            if (ast2050_smc_wait_for_ready(dev))
            {
                /* REVISIT status return?? */
                up(&flash->lock);
                *retlen = 0;
                
                return 1;
            }
            
            /* 2. Process Partial Page */
            chip_addr = w_addr - (dev * flash_info->chip_size);

            ast2050_smc_uma_page_write(dev, chip_addr, (u8*)buf, (int) w_bytes);
            
            buf += w_bytes;
            w_addr += w_bytes;
            align_size -= w_bytes;
            
            if (ast2050_smc_wait_for_ready(dev))
            {
                /* REVISIT status return?? */
                up(&flash->lock);
                *retlen = 0;
                
                return 1;
            }
        }

        /* 3. Process Whole Page */
        while (align_size > 0)
        {
            /* get flash bank information */
            dev = ast2050_get_dev(w_addr);

            if (FLASH_DEV_ERROR == dev)
            {
		        DRV_ERROR("%s, line %d: Wrong flash address access\n", __FUNCTION__, __LINE__);
		        up(&flash->lock);
		        return -EINVAL;
            }
            
            ast2050_smc_uma_write(
                           dev,                   // flash device num
                           SPI_WRITE_ENABLE_CMD,  // write enable transaction code
                           0,                     // address irrelevant
                           FALSE,                 // no address for transaction 
                           NULL,                  // no write data
                           0);                    // no data
            
            /* page size is 256 bytes */
            if (align_size >= 0x100)
            {
                w_bytes = 0x100;
            }
            else
            {
                /* 4. Process the Finial aligned byte of Partial Page */
                w_bytes = align_size;
            }
            if (ast2050_smc_wait_for_ready(dev))
            {
                /* REVISIT status return?? */
                up(&flash->lock);
                *retlen = 0;               
                return 1;
            }

            chip_addr = w_addr - (dev * flash_info->chip_size);
            ast2050_smc_uma_page_write(dev, chip_addr, (u8*)buf, (int) w_bytes);
            
            buf += w_bytes;
            w_addr += w_bytes;
            align_size -= w_bytes;
            
            if (ast2050_smc_wait_for_ready(dev))
            {
                /* REVISIT status return?? */
                up(&flash->lock);
                *retlen = 0;                
                return 1;
            }
        }
    }

    /* 5. Process the Finial non-aligned byte of Partial Page */
    if(rest_size != 0)
    {
        for(; w_addr < (addr+cnt); w_addr++)
        {
            /* get flash bank information */
			dev = ast2050_get_dev(w_addr);

            if (FLASH_DEV_ERROR == dev)
            {
          	    DRV_ERROR("%s, line %d: Wrong flash address access\n", __FUNCTION__, __LINE__);
          	    up(&flash->lock);
                return -EINVAL;
            }
            
            ast2050_smc_uma_write(
                          dev,                   // only one flash device
                          SPI_WRITE_ENABLE_CMD,  // write enable transaction code
                          0,                     // address irrelevant
                          FALSE,                 // no address for transaction 
                          NULL,                  // no write data
                          0);                    // no data
            
            if (ast2050_smc_wait_for_ready(dev))
            {
                /* REVISIT status return?? */
                up(&flash->lock);
                *retlen = 0;           
                return 1;
            }

            chip_addr = w_addr - (dev * flash_info->chip_size);
            ast2050_smc_uma_write(dev,               // only one flash device
                          SPI_PAGE_PRGM_CMD,            // write transaction code
                          chip_addr,
                          TRUE,            // transaction has address 
                          (u8 *) buf,      // write data
                          1);              // data size         
            buf++;
            
            if (ast2050_smc_wait_for_ready(dev))
            {
                /* REVISIT status return?? */
                up(&flash->lock);
                *retlen = 0;               
                return 1;
            }
        }
    }
    
    *retlen = len;
    up(&flash->lock);
    return 0;
}

/******************************************************************************
 *       FUNCTION        :       spi_nor_erase
 ******************************************************************************/
/**
 *      @brief          Erase callback function for mtd_info
 *
 *      @return         0 if success
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Private function
 *
 *****************************************************************************/
static int spi_nor_erase(
				/** mtd_info pointer */
				struct mtd_info *mtd,
				/** erase_info pointer */
				struct erase_info *instr
			    )
{
    struct flash_chip *flash = mtd->priv;
    u32 addr;
    int len;
    u8 dev;
    u32 chip_addr;

    /* sanity checks */
    if ((instr->addr + instr->len) > mtd->size)
    {
	DRV_ERROR("%s, line %d: Invalid erase address/length.\n", __FUNCTION__, __LINE__);
        return -EINVAL;
    }
    
    if (((instr->addr % mtd->erasesize) != 0) || ((instr->len % mtd->erasesize) != 0))
    {
	DRV_ERROR("%s, line %d: Invalid erase address/length.\n", __FUNCTION__, __LINE__);
        return -EINVAL;
    }

    addr = instr->addr;
    len = instr->len;

    down(&flash->lock);

    /* now erase those sectors */
    while (len)
    {
        dev = ast2050_get_dev(addr);		

        if (FLASH_DEV_ERROR == dev)
        {
            DRV_ERROR("%s: Wrong flash address access.\n", __FUNCTION__);
	    up(&flash->lock);
	    return -EINVAL;
        }
        
        ast2050_smc_uma_write(
                       dev,                     // only one flash device
                       SPI_WRITE_ENABLE_CMD,    // write enable transaction code
                       0,                       // address irrelevant
                       FALSE,                   // no address for transaction 
                       NULL,                    // no write data
                       0);                      // no data

        if(ast2050_smc_wait_for_ready(dev))
        {
            instr->state = MTD_ERASE_FAILED;
            up(&flash->lock);
            return -EIO;
        }     
        
        chip_addr = addr - (dev * flash_info->chip_size);

        ast2050_smc_uma_write(
                       dev,                     // only one flash device
                       SPI_SECTOR_ERASE_CMD,    // sector erase transaction code
                       chip_addr,                    // address relevant
                       TRUE,                    // address for transaction 
                       NULL,                    // no write data
                       0);                      // no data
      
        if(ast2050_smc_wait_for_ready(dev))
        {
            instr->state = MTD_ERASE_FAILED;
            up(&flash->lock);
            return -EIO;
        }     

        addr += mtd->erasesize;
        len -= mtd->erasesize;
    }

    up(&flash->lock);

    instr->state = MTD_ERASE_DONE;
    mtd_erase_callback(instr);

    return 0;
}

/******************************************************************************
 *       FUNCTION        :       ast2050_smc_init
 ******************************************************************************/
/**
 *      @brief          Initiallize ASPEED SMC register, unlock flash protection
 *
 *      @return         None
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Private function
 *
 *****************************************************************************/
void ast2050_smc_init(
				/** flash_chip pointer */
				struct flash_chip *this
			    )

{
    int i;
	int dev_idx;
    u8 status_reg_val;
    
    /* Initialize the semaphore */
    init_MUTEX(&this->lock);

    /* Setup controller 0x00h register*/
    iowrite32( CE0_RD_WR | CE2_RD_WR | CE0_SPI_FLASH | CE2_SPI_FLASH | \
               SEGMENT_SIZE_16MB, SMC_V_ADDR);

    ast2050_smc_unlock_write_protection();

    for (i = 0; i < ast2050_flash_total_num; i++)
    {

        dev_idx = ast2050_flash_enable_list[i];
        
        ast2050_smc_uma_write(dev_idx,              // flash device num
                              SPI_WRITE_ENABLE_CMD, // write enable transaction code
                              0,                    // address irrelevant
                              FALSE,                // no address for transaction 
                              NULL,                 // no write data
                              0);                   // no data

        ast2050_smc_wait_for_ready(dev_idx);

        ast2050_smc_uma_read(dev_idx, SPI_READ_STATUS_REG_CMD, 0, FALSE, &status_reg_val, sizeof(u32));

        ast2050_smc_wait_for_ready(dev_idx);

        status_reg_val &= 0xC3; /* set to 0 bits 2,3,4,5 that move all chip to unprotected mode */

        ast2050_smc_uma_write(dev_idx,               // flash device num
                              SPI_WRITE_ENABLE_CMD,  // write enable transaction code
                              0,                     // address irrelevant
                              FALSE,                 // no address for transaction 
                              NULL,                  // no write data
                              0);                    // no data

        ast2050_smc_wait_for_ready(dev_idx);

        ast2050_smc_uma_write(dev_idx, 
                              SPI_WRITE_STATUS_REG_CMD,
                              0,
                              FALSE,
                              &status_reg_val,
                              sizeof(u8));

        ast2050_smc_wait_for_ready(dev_idx);
    }
}


/******************************************************************************
 *       FUNCTION        :       spi_nor_probe
 ******************************************************************************/
/**
 *      @brief          Initiallize mtd_info data members and callbacks.
 *			Register the static MTD patition to kernel. Register
 *			the driver to mtd
 *
 *      @return         0 if success
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Private function
 *
 *****************************************************************************/
int spi_nor_probe (
			/** flash_chip pointer */
			struct mtd_info *mtd,
			/** maximun chips number on the EVB */
			int maxchips
		    )
{
    
    int i;
	u32 reg_val = 0;
	struct mtd_partition *parts = NULL;
    int nr_parts = 0;
	
	ast2050_smc_uma_read(FLASH_CE2_DEVICE_ID, SPI_READ_JEDEC_ID_CMD, 0, FALSE, &reg_val, sizeof(u32));

	printk("SPI Flash ID : 0x%x \n",reg_val);
	
    switch ( reg_val & 0xff)
    {
        case FLASH_S25FL128P_MF_ID :
			printk("Detect SPI Flash ID : S25FL128P \n");
			spi_flash_id = FLASH_S25FL128P_MF_ID;
		    flash_info = &spi_flash_data[0];
			parts = s25fl128_spi_partition_info;
            nr_parts = sizeof(s25fl128_spi_partition_info) / sizeof(struct mtd_partition);
		    break;
		case FLASH_MX25L128D_MF_ID :
			printk("Detect SPI Flash ID : MX25L128D \n");
			spi_flash_id = FLASH_MX25L128D_MF_ID;
		    flash_info = &spi_flash_data[1];	
			parts = mx25L128d_spi_partition_info;
			nr_parts = sizeof(mx25L128d_spi_partition_info) / sizeof(struct mtd_partition);
		    break;
    	case FLASH_STM25P128_MF_ID :
			printk("Detect SPI Flash ID : STM25P128 \n");
			spi_flash_id = FLASH_STM25P128_MF_ID;
		    flash_info = &spi_flash_data[2];	
			parts = stm25p128_spi_partition_info;
            nr_parts = sizeof(stm25p128_spi_partition_info) / sizeof(struct mtd_partition);
		    break;
		default:
			printk("This flash type [0x%x] doesn't support now.\n", reg_val);
			return (-1);
    }
	
    mtd->name = flash_info->name;
    mtd->type = MTD_NORFLASH;
    mtd->flags = MTD_CAP_NORFLASH;
    mtd->size = flash_info->chip_size * ast2050_flash_total_num;
    mtd->erasesize = flash_info->block_size;
    mtd->erase = spi_nor_erase;
    mtd->read = spi_nor_read;
    mtd->write = spi_nor_write;
    mtd->writesize =1;
    
    /* partitions should match sector boundaries; and it may be good to
     * use readonly partitions for writeprotected sectors (BP2..BP0).
     */
    if (mtd_has_partitions())
    {
        #ifdef CONFIG_MTD_CMDLINE_PARTS
        static const char *part_probes[] = { "cmdlinepart", NULL, };
        nr_parts = parse_mtd_partitions(mtd, part_probes, &parts, 0);
        #endif

        if (nr_parts <= 0)
        {
            /* Use default Partition */
            parts = s25fl128_spi_partition_info;
            nr_parts = sizeof(s25fl128_spi_partition_info) / sizeof(struct mtd_partition);
        }

        if (nr_parts > 0)
        {
            for (i = 0; i < nr_parts; i++)
            {
                DEBUG(MTD_DEBUG_LEVEL0, "partitions[%d] = "
                       "{.name = %s, .offset = 0x%.8x, "
                       ".size = 0x%.8x (%uK) }\n",
                       i, parts[i].name,
                       parts[i].offset,
                       parts[i].size,
                       parts[i].size / 1024);
            }
            return add_mtd_partitions(mtd, parts, nr_parts);
        }
    }

	return add_mtd_device(mtd);
}

/******************************************************************************
 *       FUNCTION        :       spi_nor_init
 ******************************************************************************/
/**
 *      @brief          Initiallize this driver
 *
 *      @return         0 if success
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Private function
 *
 *****************************************************************************/
int __init spi_nor_init (void)
{
    return aess_spi_init();
}

/******************************************************************************
 *       FUNCTION        :       spi_nor_cleanup
 ******************************************************************************/
/**
 *      @brief          unregister this driver from MTD and kernel
 *
 *      @return         None
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Private function
 *
 *****************************************************************************/
static void __exit spi_nor_cleanup (void)
{
#ifdef CONFIG_MTD_PARTITIONS
	/* Deregister partitions */
	del_mtd_partitions(spi_nor_mtd);
#endif
	/* Deregister the device */
	del_mtd_device (spi_nor_mtd);

	/* Free the MTD device structure */
	kfree (spi_nor_mtd_lock);
	kfree (spi_nor_mtd);
	
	DRV_INFO("%s: Remove module SPI NOR Flash peacefully!\n", __FUNCTION__);

    /* remove debugfs module */
    aess_debugfs_remove(driver_name, NULL, 0, 0);
}

/******************************************************************************
 *       FUNCTION        :       aess_spi_init
 ******************************************************************************/
/**
 *      @brief          added another function layer for init to avoid build warning
 *
 *      @return         0 if success
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Private function
 *
 *****************************************************************************/
int aess_spi_init (void)
{
    int err = 0;

    // avoid multiple initialization
    if(spi_nor_mtd != NULL)
    {
        return 0;
    }

    /* Allocate/initialize memory for MTD device structure and private data */
    spi_nor_mtd = kzalloc(sizeof(struct mtd_info), GFP_KERNEL);
    spi_nor_mtd_lock = kzalloc(sizeof(struct flash_chip), GFP_KERNEL);
   
    if ((!spi_nor_mtd) || (!spi_nor_mtd_lock))
    {
	    if(spi_nor_mtd)
	    {
	        kfree(spi_nor_mtd);
	    }

	    if(spi_nor_mtd_lock)
	    {
	        kfree(spi_nor_mtd_lock);
	    }

        DEBUG(MTD_DEBUG_LEVEL0, "Unable to allocate AST2050 SPI Flash MTD device structure.\n");
        return -ENOMEM;
    }	

    /* Link the private data with the MTD structure */
    spi_nor_mtd->priv = spi_nor_mtd_lock;

    /* AST2050 SMC Initialization */
    ast2050_smc_init(spi_nor_mtd_lock);

    if (spi_nor_probe(spi_nor_mtd, 1))
    {
        return -ENXIO;
    }

    spin_lock_init(&aess_spi_lock);

    printk("Insert SPI NOR Flash module success!\n");

    /* initial debugfs related resource */
    aess_debugfs_default_create(driver_name, NULL, 0, 0);
    aess_debugfs_create_file(driver_name, "u32", (u32 *)&u32tmpcounter, DBG_TYPE32, 0);
    aess_debugfs_create_file(driver_name, "u16", (u16 *)&u16tmpcounter, DBG_TYPE16, 0);
    aess_debugfs_create_file(driver_name, "u8", (u8 *)&u8tmpcounter, DBG_TYPE8, 0);

    return err;
}

/******************************************************************************
 *       FUNCTION        :       aess_write_spi_nor_flash
 ******************************************************************************/
/**
 *      @brief          flash write function for aess
 *
 *      @return         0 if success
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Private function
 *
 *****************************************************************************/

int aess_write_spi_nor_flash(
				/** address to write to */
				unsigned int dest_addr, 
				/** pointer to the data buffer (bytes) */
			        unsigned char *buf,
				/** length to write */
				int len
			    )
{
    struct flash_chip *flash = spi_nor_mtd->priv;
    unsigned int erasesize = spi_nor_mtd->erasesize;
    u8 dev;
    void *tmpBuf = NULL;
    int rlt;
    size_t retlen;
    // addr to write to in the block
    unsigned int to_addr = dest_addr % erasesize;
    // block to write to
    unsigned int block_addr = dest_addr - to_addr;
    int len_remain = len; // remaining length to write
    int wlen = 0; // write length 

    // initialize spi if it is not initialized yet
    if(spi_nor_mtd == NULL)
    {
        aess_spi_init();
    }

    if ((dest_addr + len) > spi_nor_mtd->size)
    {
        DRV_ERROR("%s - incorrect address/length used:\n"
                "block addr: 0x%x\n" "to addr: 0x%x\n"
                "len: %d\n" "mtd size: 0x%x\n" "block size: 0x%x.\n",
                __FUNCTION__, block_addr, to_addr, len, 
                spi_nor_mtd->size, erasesize);

        return -EINVAL;
    }

    tmpBuf = vmalloc(erasesize);
    if(tmpBuf == NULL)
    {
        DRV_ERROR("%s: vmalloc failed.\n", __FUNCTION__);
        return 1;
    }

    /* repeat the steps until remaining length is 0;
       1. read from flash to create backup copy
       2. erase the block
       3. modified the backup copy according to the buffer passed
       4. write the modified backup copy back to flash */
    spin_lock_irq(aess_spi_lock);
    while (len_remain > 0)
    {
        rlt = spi_nor_read(spi_nor_mtd, block_addr, 
                    erasesize, &retlen, tmpBuf);
        if(rlt != 0)
        {
            DRV_ERROR("%s: spi_nor_read failed - len: %d, retlen: %d\n", 
                __FUNCTION__, len, retlen);
            vfree(tmpBuf);
            return rlt;
        }

        down(&flash->lock);

        // do the flash erase
        dev = ast2050_get_dev(block_addr);

        ast2050_smc_uma_write(
                       dev,                     // only one flash device
                       SPI_WRITE_ENABLE_CMD,    // write enable transaction code
                       0,                       // address irrelevant
                       FALSE,                   // no address for transaction 
                       NULL,                    // no write data
                       0);                      // no data

        if(ast2050_smc_wait_for_ready(dev))
        {
            up(&flash->lock);
            vfree(tmpBuf);
            return -EIO;
        }     
        
        ast2050_smc_uma_write(
                       dev,                     // only one flash device
                       SPI_SECTOR_ERASE_CMD,    // sector erase transaction code
                       block_addr,              // address of MAC located block
                       TRUE,                    // address for transaction 
                       NULL,                    // no write data
                       0);                      // no data
      
        if(ast2050_smc_wait_for_ready(dev))
        {
            up(&flash->lock);
            vfree(tmpBuf);
            return -EIO;
        }     
        // flash erase done

        up(&flash->lock);

        // write the change to the backup copy, then write it back to
        // flash
        if(len_remain > (erasesize - to_addr))
        {
            wlen = erasesize - to_addr;
        }
        else
        {
            wlen = len_remain;
        }
        len_remain -= wlen;

        memcpy((tmpBuf + to_addr), buf, wlen);

        rlt = spi_nor_write(spi_nor_mtd, block_addr, 
                    erasesize, &retlen, tmpBuf);

        if(rlt != 0)
        {
            DRV_ERROR("%s: spi_nor_write failed - len: %d, retlen: %d\n",
                __FUNCTION__, len, retlen);
            vfree(tmpBuf);
            return rlt;
        }

        block_addr += erasesize;
        to_addr = 0;
    }
    spin_unlock_irq(aess_spi_lock);

    vfree(tmpBuf);
    return 0;
}

/******************************************************************************
 *       FUNCTION        :       aess_read_spi_nor_flash
 ******************************************************************************/
/**
 *      @brief          flash read function for aess
 *
 *      @return         0 if success
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Private function
 *
 *****************************************************************************/
int aess_read_spi_nor_flash(
			    /** physical address to read from */
			    unsigned int from_addr,
			    /** pointer to the data buffer (bytes) */
		            unsigned char *buf,
			    /** length to read */
			    int len
			)
{
    int rlt;
    size_t retlen;

    // initialize spi if it is not initialized yet
    if(spi_nor_mtd == NULL)
    {
        aess_spi_init();
    }

    rlt = spi_nor_read(spi_nor_mtd,
        from_addr, len, &retlen, buf);

    if(rlt != 0)
    {
        DRV_ERROR("%s: spi_nor_read failed - len: %d, retlen: %d\n",
            __FUNCTION__, len, retlen);
    }

    return rlt;
}

EXPORT_SYMBOL(aess_write_spi_nor_flash);
EXPORT_SYMBOL(aess_read_spi_nor_flash);

module_init(spi_nor_init);
module_exit(spi_nor_cleanup);

MODULE_AUTHOR("Andy Chang");
MODULE_DESCRIPTION("SPI NOR flash device driver");
MODULE_VERSION("0.1.0.1");
MODULE_LICENSE("GPL");

/* End of code */

