/**
 *
 *               Avocent Corporation. Proprietary Information.
 *      This software is supplied under the terms of a license agreement or
 *      nondisclosure agreement with Avocent Corporation, or one of its affiliates, and
 *      may not be used, copied, disseminated, or distributed except in accordance
 *      with the terms of that agreement.
 *
 *      3541 Gateway Boulevard, Fremont, California, 94538 U.S.A.
 *                  US Phone: 510-771-6100
 *      Copyright 2001-2009 Avocent Corporation. All rights reserved.
 *
 *----------------------------------------------------------------------------
 * MODULES      AESS MTD SPI NOR Flash Driver
 *----------------------------------------------------------------------------
 *  @file   aess_mtddrv_spinor.h
 *  @brief  header file for MTD SPI NOR flash dirver
 *
 *----------------------------------------------------------------------------*/

#include <asm/sizes.h>
#include <asm/io.h>
#include <asm/sizes.h>
#include <asm/hardware.h>
#include <asm-arm/io.h>

#ifndef	AESS_MTDDRV_SPINOR_H
#define AESS_MTDDRV_SPINOR_H

/**************************************************************
 * Private Local Macros
 **************************************************************/
	
	/* Define max times to check status register before we give up. */
#define	MAX_READY_WAIT_COUNT	1000000

/*Macros for SMC00 register*/
#define CE_RD_O     (0)
#define CE2_RD_WR   (1 << 12)
#define CE1_RD_WR   (1 << 11)
#define CE0_RD_WR   (1 << 10)

#define CE2_NOR_FLASH   (0)
#define CE2_NAND_FLASH  (1 << 8)
#define CE2_SPI_FLASH   (2 << 8)

#define CE1_NOR_FLASH   0
#define CE1_NAND_FLASH  (1 << 6)
#define CE1_SPI_FLASH   (2 << 6)

#define CE0_NOR_FLASH   0
#define CE0_NAND_FLASH  (1 << 4)
#define CE0_SPI_FLASH   (2 << 4)

#define SEGMENT_SIZE_32MB   0
#define SEGMENT_SIZE_16MB   1
#define SEGMENT_SIZE_8MB    2
#define SEGMENT_SIZE_4MB    3

/* Macros for CE0~CE2 (SPI FLASH) registers */
#define EN_READ_CMMD_MERGE  0

#define HLCK_DV16 0
#define HLCK_DV14 (1 << 8)
#define HLCK_DV12 (2 << 8)
#define HLCK_DV10 (3 << 8)
#define HLCK_DV8  (4 << 8)
#define HLCK_DV6  (5 << 8)
#define HLCK_DV4  (6 << 8)
#define HLCK_DV2  (7 << 8)

#define DUMMY_0BYTE 0
#define DUMMY_1BYTE (1 << 6)
#define DUMMY_2BYTE (2 << 6)
#define DUMMY_3BYTE (3 << 6)

#define MSB_FIRST 0
#define LSB_FIRST (1 << 5)

#define CLOCK_MODE0 0
#define CLOCK_MODE3 (1 << 4)

#define EN_1BIT_CLK 0
#define EN_2BIT_CLK (1 << 3)

#define DIS_USERMODE (1 << 2)
#define EN_USERMODE 0
#define USERMODE_MASK (1 << 2)

#define NORMAL_READ 0
#define FAST_READ 1
#define NORMAL_WRITE 2
#define USER_MODE  3

/* Manufacture ID */
#define FLASH_S25FL128P_MF_ID	 0x01	/* Spansion */
#define FLASH_STM25P128_MF_ID	 0x20	/* ST Microelectronics */
#define FLASH_MX25L128D_MF_ID	 0xc2	/* MXIC */

#define FLASH_CAPACITY_ID   	 0x16

/* SPI NOR FLASH COMMAND SET */
#define SPI_READ_JEDEC_ID_CMD    0x9F
#define SPI_WRITE_ENABLE_CMD     0x06
#define SPI_WRITE_DISABLE_CMD    0x04 
#define SPI_READ_STATUS_REG_CMD  0x05
#define SPI_WRITE_STATUS_REG_CMD 0x01
#define SPI_READ_DATA_CMD        0x03
#define SPI_PAGE_PRGM_CMD        0x02
#define SPI_SECTOR_ERASE_CMD     0xD8
#define SPI_READ_PID_CMD         0x90

/* Status Register bits. */
#define	SR_WIP			1	/* Write in progress */
#define	SR_WEL			2	/* Write enable latch */
#define	SR_BP0			4	/* Block protect 0 */
#define	SR_BP1			8	/* Block protect 1 */
#define	SR_BP2			0x10	/* Block protect 2 */
#define	SR_SRWD			0x80	/* SR write protect */

#define STATUS_OK       0
#define STATUS_BUSY     1

/*Macros read/write*/

/******************************************************************************
 *       STRUCT        :       flash_chip
 ******************************************************************************/
/**
 *      @brief          Private data structure
 *
 *****************************************************************************/
struct flash_chip {
	/** smeaphore lock */
	struct semaphore	lock;
};

/******************************************************************************
 *       STRUCT        :       flash_t
 ******************************************************************************/
/**
 *      @brief          flash type structure
 *
 *****************************************************************************/
typedef struct flash_t
{
    /** manufacture ID */
    u32 mf_id;
    /** capacity */
    u32 capacity;
    /** name */
    u8*  name;
    /** chip size */
    u32 chip_size;
    /** block size */
    u32 block_size;
} flash_t;

typedef enum {
	FALSE = 0,
	TRUE  = 1
} BOOL;

#endif

/* End of code */

