// --------------------------------------------------------------------

#ifndef FTMAC100_H
#define FTMAC100_H

#define CACHE_ABLE    1

#define IP_COUNT                1
#define BUFPOOL_SIZE		0x20000
#define BUFPOOL_OFFSET_RXDESC	0
#define BUFPOOL_OFFSET_RXBUF	(BUFPOOL_OFFSET_RXDESC+0x1000)
#define BUFPOOL_OFFSET_TXDESC	(BUFPOOL_OFFSET_RXBUF+0x10000)
#define BUFPOOL_OFFSET_TXBUF	(BUFPOOL_OFFSET_TXDESC+0x1000)


typedef unsigned char			byte;
typedef unsigned short			word;
typedef unsigned long int 		dword;


// --------------------------------------------------------------------
//		FTMAC100 hardware related defenition
// --------------------------------------------------------------------
#define CONFIG_FTGMAC_DES_FMT_NC1
#ifdef CONFIG_FTGMAC_DES_FMT_NC1
#define ZeroCopy
// Defining _PHY_MARVELL includes code that isn't needed for Broadcom.
#define CONFIG_FTGMAC_PHY_MARVELL
#define HAVE_MULTICAST
#endif

#ifdef CONFIG_CPU_FA52x_DCE
#define DescpCacheable
#endif


//base address and interrupt information
#define NC_GMAC1_BASE		0xE0400000
#define IRQ_GMAC1		6
#define NC_GMAC2_BASE		0xE0500000
#define IRQ_GMAC2		5


#define ISR_REG				0x00				// interrups status register
#define IER_REG				0x04				// interrupt maks register
#define MAC_MADR_REG		0x08				// MAC address (Most significant)
#define MAC_LADR_REG		0x0c				// MAC address (Least significant)

#define MAHT0_REG			0x10				// Multicast Address Hash Table 0 register
#define MAHT1_REG			0x14				// Multicast Address Hash Table 1 register
#define TXPD_REG			0x18				// Transmit Poll Demand register
#define RXPD_REG			0x1c				// Receive Poll Demand register
#define TXR_BADR_REG		0x20				// Transmit Ring Base Address register
#define RXR_BADR_REG		0x24				// Receive Ring Base Address register

#define HPTXPD_REG			0x28	//
#define HPTXR_BADR_REG		0x2c	//

#define ITC_REG				0x30				// interrupt timer control register
#define APTC_REG			0x34				// Automatic Polling Timer control register
#define DBLAC_REG			0x38				// DMA Burst Length and Arbitration control register

#define DMAFIFOS_REG		0x3c	//
#define FEAR_REG			0x44	//
#define TPAFCR_REG			0x48	//
#define RBSR_REG			0x4c	//for NC Body
#define MACCR_REG			0x50				// MAC control register
#define MACSR_REG			0x54				// MAC status register
#define PHYCR_REG			0x60				// PHY control register
#define PHYDATA_REG			0x64				// PHY Write Data register
#define FCR_REG				0x68				// Flow Control register
#define BPR_REG				0x6c				// back pressure register
#define WOLCR_REG			0x70				// Wake-On-Lan control register
#define WOLSR_REG			0x74				// Wake-On-Lan status register
#define WFCRC_REG			0x78				// Wake-up Frame CRC register
#define WFBM1_REG			0x80				// wake-up frame byte mask 1st double word register
#define WFBM2_REG			0x84				// wake-up frame byte mask 2nd double word register
#define WFBM3_REG			0x88				// wake-up frame byte mask 3rd double word register
#define WFBM4_REG			0x8c				// wake-up frame byte mask 4th double word register

#define NPTXR_PTR_REG		0x90	//
#define HPTXR_PTR_REG		0x94	//
#define RXR_PTR_REG			0x98	//


// --------------------------------------------------------------------
//		ISR_REG  IMR_REG 
// --------------------------------------------------------------------
#define HPTXBUF_UNAVA_bit	(1UL<<10)
#define PHYSTS_CHG_bit		(1UL<<9)
#define AHB_ERR_bit			(1UL<<8)
#define TPKT_LOST_bit		(1UL<<7)
#define NPTXBUF_UNAVA_bit	(1UL<<6)
#define TPKT2F_bit			(1UL<<5)
#define TPKT2E_bit			(1UL<<4)
#define RPKT_LOST_bit		(1UL<<3)
#define RXBUF_UNAVA_bit		(1UL<<2)
#define RPKT2F_bit			(1UL<<1)
#define RPKT2B_bit			(1UL<<0)


// --------------------------------------------------------------------
//	APTC_REG 
// --------------------------------------------------------------------

#ifndef CONFIG_CPU_BIG_ENDIAN 

typedef struct
{
	u32 RXPOLL_CNT:4;
	u32 RXPOLL_TIME_SEL:1;
	u32 Reserved1:3;
	u32 TXPOLL_CNT:4;
	u32 TXPOLL_TIME_SEL:1;
	u32 Reserved2:19;
}FTGMAC100_APTCR_Status;

#else

typedef struct
{
	u32 Reserved2:19;
	u32 TXPOLL_TIME_SEL:1;
	u32 TXPOLL_CNT:4;
	u32 Reserved1:3;
	u32 RXPOLL_TIME_SEL:1;
	u32 RXPOLL_CNT:4;
}FTGMAC100_APTCR_Status;

#endif

// --------------------------------------------------------------------
//		PHYCR_REG
// --------------------------------------------------------------------
#define PHY_RE_AUTO_bit			(1UL<<9)
#define PHY_READ_bit			(1UL<<26)
#define PHY_WRITE_bit			(1UL<<27)
// --------------------------------------------------------------------
//		PHYCR_REG 
// --------------------------------------------------------------------
#define PHY_AUTO_OK_bit			(1UL<<5)
// --------------------------------------------------------------------
//		PHY INT_STAT_REG 
// --------------------------------------------------------------------
#define PHY_SPEED_CHG_bit		(1UL<<14)
#define PHY_DUPLEX_CHG_bit		(1UL<<13)
#define PHY_LINK_CHG_bit		(1UL<<10)
#define PHY_AUTO_COMP_bit		(1UL<<11)
// --------------------------------------------------------------------
//		PHY SPE_STAT_REG 
// --------------------------------------------------------------------
#define PHY_RESOLVED_bit		(1UL<<11)
#define PHY_SPEED_mask			0xC000
#define PHY_SPEED_10M			0x0
#define PHY_SPEED_100M			0x1
#define PHY_SPEED_1G			0x2
#define PHY_DUPLEX_mask                 0x2000
//#define PHY_FULLDUPLEX                  0x1
#define PHY_SPEED_DUPLEX_MASK           0x01E0
#define PHY_100M_DUPLEX                 0x0100
#define PHY_100M_HALF                   0x0080
#define PHY_10M_DUPLEX                  0x0040
#define PHY_10M_HALF                    0x0020
#define LINK_STATUS                     0x04


// --------------------------------------------------------------------
//		MACCR_REG
// --------------------------------------------------------------------

#define SW_RST_bit			(1UL<<31)				// software reset/
#define DIRPATH_bit			(1UL<<21)
#define RX_IPCS_FAIL_bit	(1UL<<20)	//
#define SPEED_100_bit   	        (1UL<<19)	//
#define RX_UDPCS_FAIL_bit	(1UL<<18)	//
#define RX_BROADPKT_bit		(1UL<<17)				// Receiving broadcast packet
#define RX_MULTIPKT_bit		(1UL<<16)				// receiving multicast packet
#define RX_HT_EN_bit		(1UL<<15)
#define RX_ALLADR_bit		(1UL<<14)				// not check incoming packet's destination address
#define JUMBO_LF_bit		(1UL<<13)	//
#define RX_RUNT_bit			(1UL<<12)				// Store incoming packet even its length is les than 64 byte
#define CRC_CHK_bit			(1UL<<11)	//
#define CRC_APD_bit			(1UL<<10)				// append crc to transmit packet
#define GMAC_MODE_bit		(1UL<<9)	//
#define FULLDUP_bit			(1UL<<8)				// full duplex
#define ENRX_IN_HALFTX_bit	(1UL<<7)	//
#define LOOP_EN_bit			(1UL<<6)				// Internal loop-back
#define HPTXR_EN_bit		(1UL<<5)	//
#define REMOVE_VLAN_bit		(1UL<<4)	//
//#define MDC_SEL_bit		(1UL<<13)				// set MDC as TX_CK/10
//#define RX_FTL_bit		(1UL<<11)				// Store incoming packet even its length is great than 1518 byte
#define RXMAC_EN_bit		(1UL<<3)				// receiver enable
#define TXMAC_EN_bit		(1UL<<2)				// transmitter enable
#define RXDMA_EN_bit		(1UL<<1)				// enable DMA receiving channel
#define TXDMA_EN_bit		(1UL<<0)				// enable DMA transmitting channel


// --------------------------------------------------------------------
//		SCU_REG
// --------------------------------------------------------------------
#define  SCU_PROTECT_KEY_REG            0x0
#define  SCU_PROT_KEY_MAGIC             0x1688a8a8
#define  SCU_RESET_CONTROL_REG          0x04
#define  SCU_RESET_MAC1                 (1u << 11)
#define  SCU_RESET_MAC2                 (1u << 12)

#define  SCU_HARDWARE_TRAPPING_REG      0x70
#define  SCU_HT_MAC_INTF_LSBIT          6
#define  SCU_HT_MAC_INTERFACE           (0x7u << SCU_HT_MAC_INTF_LSBIT)
#define  MAC_INTF_SINGLE_PORT_MODES     (1u<<0/*GMII*/ | 1u<<3/*MII_ONLY*/ | 1u<<4/*RMII_ONLY*/)
#define  SCU_HT_MAC_GMII                0x0u
// MII and MII mode
#define  SCU_HT_MAC_MII_MII             0x1u
#define  SCU_HT_MAC_MII_ONLY            0x3u
#define  SCU_HT_MAC_RMII_ONLY           0x4u

#define  SCU_MULTIFUNCTION_PIN_REG      0x74
#define  SCU_MFP_MAC2_PHYLINK           (1u << 26)
#define  SCU_MFP_MAC1_PHYLINK           (1u << 25)
#define  SCU_MFP_MAC2_MII_INTF          (1u << 21)
#define  SCU_MFP_MAC2_MDC_MDIO          (1u << 20)
#define  SCU_SILICON_REVISION_REG       0x7C

// --------------------------------------------------------------------
//		Receive Ring descriptor structure
// --------------------------------------------------------------------


#ifdef CONFIG_CPU_BIG_ENDIAN 
typedef struct
{
	// RXDES0
	u32 RXPKT_RDY:1;		// 1 ==> owned by FTMAC100, 0 ==> owned by software
	u32 Reserved3:1;
	u32 FRS:1;
	u32 LRS:1;
	u32 Reserved2:2;
	u32 PAUSE_FRAME:1;
	u32 PAUSE_OPCODE:1;
	u32 FIFO_FULL:1;
	u32 RX_ODD_NB:1;
	u32 RUNT:1;
	u32 FTL:1;
	u32 CRC_ERR:1;
	u32 RX_ERR:1;
	u32 BROARDCAST:1;
	u32 MULTICAST:1;
	u32 EDORR:1;
	u32 Reserved1:1;   
	u32 VDBC:14;
	
	
	// RXDES1
	u32 Reserved5:4;
	u32 IPCS_FAIL:1;
	u32 UDPCS_FAIL:1;
	u32 TCPCS_FAIL:1;
	u32 VLAN_AVA:1;	
	u32 DF:1;
	u32 LLC_PKT:1;
	u32 PROTL_TYPE:2;
	u32 Reserved4:4;
	u32 VLAN_TAGC:16;
	
	
	// RXDES2
	u32 Reserved6:32;
	
	// RXDES3
	u32 RXBUF_BADR;
				
	u32 VIR_RXBUF_BADR;			// not defined, the virtual address of receive buffer is placed here
	
	u32 RESERVED; 
	u32 RESERVED1; 
	u32 RESERVED2; 
}RX_DESC;

#else // little endian

#ifdef CONFIG_FTGMAC_DES_FMT_NC1
typedef struct
{
	// RXDES0
	u32 VDBC:14;//0~10
	u32 Reserved1:1;          //11~15
	u32 EDORR:1;
	u32 MULTICAST:1;          //16
	u32 BROARDCAST:1;         //17
	u32 RX_ERR:1;             //18
	u32 CRC_ERR:1;            //19
	u32 FTL:1;
	u32 RUNT:1;
	u32 RX_ODD_NB:1;
	u32 FIFO_FULL:1;
	u32 PAUSE_OPCODE:1;
	u32 PAUSE_FRAME:1;
	u32 Reserved2:2;
	u32 LRS:1;
	u32 FRS:1;
	u32 Reserved3:1;
	u32 RXPKT_RDY:1;		// 1 ==> owned by FTMAC100, 0 ==> owned by software
	
	// RXDES1
	u32 VLAN_TAGC:16;
	u32 Reserved4:4;
	u32 PROTL_TYPE:2;
	u32 LLC_PKT:1;
	u32 DF:1;
	u32 VLAN_AVA:1;	
	u32 TCPCS_FAIL:1;
	u32 UDPCS_FAIL:1;
	u32 IPCS_FAIL:1;
	u32 Reserved5:4;
	
	// RXDES2
	u32 Reserved6:32;
	
	// RXDES3
	u32 RXBUF_BADR;
				
	u32 VIR_RXBUF_BADR;			// not defined, the virtual address of receive buffer is placed here
	
	u32 RESERVED; 
	u32 RESERVED1; 
	u32 RESERVED2;	
}RX_DESC;

#else //NC WholeChip

typedef struct
{
	// RXDES0
	u32 VDBC:14;//0~10
	u32 Reserved1:2;          //11~15
	u32 MULTICAST:1;          //16
	u32 BROARDCAST:1;         //17
	u32 RX_ERR:1;             //18
	u32 CRC_ERR:1;            //19
	u32 FTL:1;
	u32 RUNT:1;
	u32 RX_ODD_NB:1;
	u32 FIFO_FULL:1;
	u32 PAUSE_OPCODE:1;
	u32 PAUSE_FRAME:1;
	u32 Reserved2:2;
	u32 LRS:1;
	u32 FRS:1;
	u32 Reserved3:1;
	u32 RXPKT_RDY:1;		// 1 ==> owned by FTMAC100, 0 ==> owned by software
	
	
	// RXDES1
	u32 VLAN_TAGC:16;
	u32 VLAN_AVA:1;	
	u32 TCPCS_FAIL:1;
	u32 UDPCS_FAIL:1;
	u32 IPCS_FAIL:1;
	u32 PROTL_TYPE:2;
	u32 LLC_PKT:1;
	u32 Reserved4:9;
	
	// RXDES2
	u32 RXBUF_Size:14;			
	u32 Reserved5:17;
	u32 EDORR:1;
	
	// RXDES3
	u32 RXBUF_BADR;
				
	u32 VIR_RXBUF_BADR;			// not defined, the virtual address of receive buffer is placed here
	
	u32 RESERVED; 
	u32 RESERVED1; 
	u32 RESERVED2; 
}RX_DESC;

#endif // end of #ifdef CONFIG_FTGMAC_DES_FMT_NC1

#endif


#ifdef CONFIG_CPU_BIG_ENDIAN 

typedef struct
{
	// TXDES0
	u32 TXDMA_OWN:1;
	u32 Reserved4:1;
	u32 FTS:1;
	u32 LTS:1;
	u32 Reserved3:8;
	u32 CRC_ERR:1;
	u32 Reserved2:3;
	u32 EDOTR:1;
	u32 Reserved1:1;
	u32 TXBUF_Size:14;
	
	// TXDES1
	u32 TXIC:1;	
	u32 TX2FIC:1;
	u32 Reserved6:7;
	u32 LLC_PKT:1;
	u32 Reserved5:2;
	u32 IPCS_EN:1;
	u32 UDPCS_EN:1;
	u32 TCPCS_EN:1;
	u32 INS_VLAN:1;
	u32 VLAN_TAGC:16;
	
	// TXDES2
	u32 Reserved7:32;

	// TXDES3
	u32 TXBUF_BADR;

	u32 VIR_TXBUF_BADR;			// Reserve, the virtual address of transmit buffer is placed here

	u32 RESERVED;
	u32 RESERVED1; 
	u32 RESERVED2; 
}TX_DESC;

#else //little endian

#ifdef CONFIG_FTGMAC_DES_FMT_NC1

typedef struct
{
	// TXDES0
	u32 TXBUF_Size:14;
	u32 Reserved1:1;
	u32 EDOTR:1;
	u32 Reserved2:3;
	u32 CRC_ERR:1;
	u32 Reserved3:8;
	u32 LTS:1;
	u32 FTS:1;
	u32 Reserved4:1;
	u32 TXDMA_OWN:1;
	
	// TXDES1
	u32 VLAN_TAGC:16;
	u32 INS_VLAN:1;
	u32 TCPCS_EN:1;
	u32 UDPCS_EN:1;
	u32 IPCS_EN:1;
	u32 Reserved5:2;
	u32 LLC_PKT:1;
	u32 Reserved6:7;
	u32 TX2FIC:1;
	u32 TXIC:1;	
	
	// TXDES2
	u32 Reserved7:32;

	// TXDES3
	u32 TXBUF_BADR;

	u32 VIR_TXBUF_BADR;			// Reserve, the virtual address of transmit buffer is placed here

	u32 RESERVED;
	u32 RESERVED1; 
	u32 RESERVED2; 

}TX_DESC;

#else //NC WholeChip

typedef struct
{
	// TXDES0
	u32 TXBUF_Size:14;
	u32 Reserved1:14;
	u32 LTS:1;
	u32 FTS:1;
	u32 Reserved2:1;
	u32 TXDMA_OWN:1;
	
	// TXDES1
	u32 VLAN_TAGC:16;
	u32 INS_VLAN:1;
	u32 TCPCS_EN:1;
	u32 UDPCS_EN:1;
	u32 IPCS_EN:1;
	u32 LLC_PKT:1;
	u32 Reserved3:11;
	
	// TXDES2
	u32 Reserved4:29;
	u32 TX2FIC:1;
	u32 TXIC:1;
	u32 EDOTR:1;

	// TXDES3
	u32 TXBUF_BADR;

	u32 VIR_TXBUF_BADR;			// Reserve, the virtual address of transmit buffer is placed here

	u32 RESERVED;
	u32 RESERVED1; 
	u32 RESERVED2; 

}TX_DESC;

#endif //#ifdef CONFIG_FTGMAC_DES_FMT_NC1

#endif



// waiting to do:
#define	TXPOLL_CNT			8
#define RXPOLL_CNT			0

#define TX_OWNBY_SOFTWARE		0
#define TX_OWNBY_FTGMAC100		1


#define RX_OWNBY_SOFTWARE		1
#define RX_OWNBY_FTGMAC100		0

// --------------------------------------------------------------------
//		driver related definition
// --------------------------------------------------------------------


//#define RXDES_NUM			64//64 // we defined 32 descriptor for OTG issue 
#define RXDES_NUM			32

//#define RX_BUF_SIZE			512					//  receive buffer  512 byte
#define RX_BUF_SIZE			1536

//#define TXDES_NUM			64//64 //define 32 for OTG working
//#define TX_BUF_SIZE			2048

#define TXDES_NUM			32
#define TX_BUF_SIZE			1536

#define PHYID_VENDOR_MASK		0xfffffc00
#define PHYID_VENDOR_MARVELL		0x01410c00
#define PHYID_VENDOR_BROADCOM		0x00406000

/* store this information for the driver.. */

struct AstMacHwConfig {
    unsigned char phyAddr;	// See IP_phy_addr[] encoding
    unsigned char macId;
    unsigned char isRevA0;
    unsigned char pad[1];
    unsigned int  miiPhyId;
};

struct ftgmac100_local {

 	// these are things that the kernel wants me to keep, so users
	// can find out semi-useless statistics of how well the card is
	// performing
	struct net_device_stats stats;

	struct AstMacHwConfig ids;

	// Set to true during the auto-negotiation sequence
	int	autoneg_active;

	// Last contents of PHY Register 18
	u32		lastPhy18;

	spinlock_t lock;

	struct sk_buff *rx_skbuff[RXDES_NUM];

	volatile RX_DESC *rx_descs;			// receive ring base address
	u32		rx_descs_dma;				// receive ring physical base address
	char	*rx_buf;					// receive buffer cpu address
	int		rx_buf_dma;					// receive buffer physical address
	int		rx_idx;						// receive descriptor

	volatile TX_DESC *tx_descs;
	u32		tx_descs_dma;
	char	*tx_buf;
	int		tx_buf_dma;
	int		tx_idx;
	int		old_tx;
	struct sk_buff *tx_skbuff[TXDES_NUM];
	int     maccr_val;
        struct timer_list    timer;
};


#define FTGMAC100_STROBE_TIME			(10*HZ)
///#define FTMAC100_STROBE_TIME			1
 
#endif  /* _SMC_91111_H_ */


