/**
 *
 *               Avocent Corporation. Proprietary Information.
 *      This software is supplied under the terms of a license agreement or
 *      nondisclosure agreement with Avocent Corporation, or one of its affiliates, and
 *      may not be used, copied, disseminated, or distributed except in accordance
 *      with the terms of that agreement.
 *
 *      3541 Gateway Boulevard, Fremont, California, 94538 U.S.A.
 *                  US Phone: 510-771-6100
 *      Copyright 2001-2009 Avocent Corporation. All rights reserved.
 *
 *----------------------------------------------------------------------------
 *  MODULES     AESS Serial Driver
 *----------------------------------------------------------------------------
 * @file    aspeed_serial.c
 * @brief   c source file for aspeed serial driver
 *----------------------------------------------------------------------------
 */
/****************************************************************************** 
 * Content
 * ----------------
 *       mvp2000_uart_stop_tx()	-   stop transmitting
 *       mvp2000_uart_start_tx() -  start transmitting
 *       mvp2000_uart_stop_rx()  -  stop receiving
 *       mvp2000_uart_enable_ms() - enable modem status for hardware control
 *       mvp2000_uart_rx_chars() -  receive chars
 *       mvp2000_uart_tx_chars() -  transmit chars
 *       mvp2000_modem_status() -   handle modem status for hardware control
 *       mvp2000_uart_int() -	interrupt handler
 *       mvp2000_uart_tx_empty()    -	check tx empty or not
 *       mvp2000_uart_get_mctrl()   -	get CTS state
 *       mvp2000_uart_set_mctrl()   -	set CTS/RTS state 
 *       mvp2000_uart_break_ctl()   -	set LCR break control
 *       mvp2000_uart_startup()	    -   start up uart device
 *       mvp2000_uart_shutdown()    -	stop uart device
 *       mvp2000_set_termios()	-   set terminal settings
 *       mvp2000_uart_type()	-   get UART device type
 *       mvp2000_uart_release_port()	-   release memory for device
 *       mvp2000_uart_request_port()	-   request memory for device
 *       mvp2000_uart_config_port() -	configuare UART device port
 *       mvp2000_uart_verify_port() -	verify UART device port
 *       mvp2000_uart_console_write()	-   write data to console
 *       mvp2000_uart_console_get_options() -	get console options
 *       mvp2000_uart_console_setup()	-   setup/configure console
 *       mvp2000_uart_console_init()	-   register/init console
 *       mvp2000_uart_init()	-   init function for the driver
 *       mvp2000_uart_exit()	-   exit function for the driver
 *
 *******************************************************************************/

//#include <linux/config.h>
#include <linux/module.h>
#include <linux/tty.h>
#include <linux/ioport.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/serial.h>
#include <linux/console.h>
#include <linux/sysrq.h>

#include <asm/io.h>
#include <asm/irq.h>

#if defined(CONFIG_SERIAL_ASPEED_CONSOLE) && defined(CONFIG_MAGIC_SYSRQ)
//#define SUPPORT_SYSRQ
#endif

#include <linux/serial_core.h>

#include <asm/hardware/aspeed_serial.h>
#include <asm/arch/hardware.h>
#include <asm/arch-ast2050/hw_regs.h>

/* 
 * Implement debug message with debugfs 
 * If debugfs is to be used here, it is recommanded that the output
 * is redirected to file instead of console, because it will create
 * way too much interrupt and message here in the serial driver
 * */
#include <linux/aess_debugfs.h>

#define UART_NR		2

#define SERIAL_MVP2000_CONSLE_NAME	"ttyS"
#define SERIAL_MVP2000_TTY_NAME		"ttyS"
#define SERIAL_MVP2000_DEVFS_NAME	"tts/"

/* 
 * modified the major and minor number to use default 
 * tty major/minor number
 */
//#define SERIAL_MVP2000_MAJOR	204
//#define SERIAL_MVP2000_MINOR	16
#define SERIAL_MVP2000_MAJOR	TTY_MAJOR
#define SERIAL_MVP2000_MINOR	64

#define SERIAL_MVP2000_NR	UART_NR

#define CALLOUT_MVP2000_NAME	"cuaam"
#define CALLOUT_MVP2000_MAJOR	205
#define CALLOUT_MVP2000_MINOR	16
#define CALLOUT_MVP2000_NR	UART_NR


#ifdef SUPPORT_SYSRQ
static struct console mvp2000_console;
#endif

/* Avocent --- AndyC start */
//#define MVP2000_ISR_PASS_LIMIT	256
#define MVP2000_ISR_PASS_LIMIT   16

// added for build time serial hardware configuration
#ifndef CONFIG_SERIAL_ASPEED_DEBUG_PORT
#define CONFIG_SERIAL_ASPEED_DEBUG_PORT 1 // set default DEBUG port to port 1
#endif

#ifndef CONFIG_SERIAL_ASPEED_IPMI_PORT
#define CONFIG_SERIAL_ASPEED_IPMI_PORT 0 // set default IPMI port to port 0
#endif

// dev/ttyS0 is only used for debug console.
// When configuration serial hardware, only the base address it maps
// to changes. PORT0_xxx stores the actual hardware port information
// ttyS0 uses
#if CONFIG_SERIAL_ASPEED_DEBUG_PORT == 0
#define PORT0_VIRTUAL ASPEED_UART0_VIRTUAL
#define PORT0_BASE ASPEED_UART0_BASE
#define PORT0_IRQ IRQ_UARTINT0
#else
#define PORT0_VIRTUAL ASPEED_UART1_VIRTUAL
#define PORT0_BASE ASPEED_UART1_BASE
#define PORT0_IRQ IRQ_UARTINT1
#endif

// dev/ttyS1 is only used by IPMI.
// When configuration serial hardware, only the base address it maps
// to changes. PORT1_xxx stores the actual hardware port information
// ttyS1 uses
#if CONFIG_SERIAL_ASPEED_IPMI_PORT == 0
#define PORT1_VIRTUAL ASPEED_UART0_VIRTUAL
#define PORT1_BASE ASPEED_UART0_BASE
#define PORT1_IRQ IRQ_UARTINT0
#else
#define PORT1_VIRTUAL ASPEED_UART1_VIRTUAL
#define PORT1_BASE ASPEED_UART1_BASE
#define PORT1_IRQ IRQ_UARTINT1
#endif

// added for Hardware flow control
#define UART_GET_MCR(p)     readl((p)->membase + UART_MCR)
#define UART_PUT_MCR(p, v)  writel((v), (p)->membase + UART_MCR)
#define UART_GET_MSR(p)     readl((p)->membase + UART_MSR)

/*
 * Access macros for the UARTs
 */
#define UART_GET_CHAR(p)	readl((p)->membase + UART_RBR)
#define UART_PUT_CHAR(p, v)	writel((v), (p)->membase + UART_THR)
#define UART_GET_DLL(p)		readl((p)->membase + UART_DLL)
#define UART_PUT_DLL(p, v)	writel((v), (p)->membase + UART_DLL)
#define UART_GET_DLH(p)		readl((p)->membase + UART_DLH)
#define UART_PUT_DLH(p, v)	writel((v), (p)->membase + UART_DLH)
#define UART_GET_IER(p)		readl((p)->membase + UART_IER)
#define UART_PUT_IER(p, v)	writel((v), (p)->membase + UART_IER)
#define UART_GET_IIR(p)		readl((p)->membase + UART_IIR)
#define UART_GET_FCR(p)		readl((p)->membase + UART_FCR)
#define UART_PUT_FCR(p, v)	writel((v), (p)->membase + UART_FCR)
#define UART_GET_LCR(p)		readl((p)->membase + UART_LCR)
#define UART_PUT_LCR(p, v)	writel((v), (p)->membase + UART_LCR)
#define UART_GET_LSR(p)		readl((p)->membase + UART_LSR)

#define UART_DUMMY_RSR_RX	256
#define UART_PORT_SIZE		64

#define U_CTS       0x10
#define U_RTSE      0x02

static char *driver_name="mvp2000_serial";
static u8   u8tmpcounter = 0;
static u16 u16tmpcounter = 0;
static u32 u32tmpcounter = 0;

static void mvp2000_uart_set_mctrl(struct uart_port *port, unsigned int mctrl);

/******************************************************************************
 *       STRUCT        :       uart_mvp2000_port
 ******************************************************************************/
/**
 *      @brief          We wrap our port structure around the generic uart_port.
 *
 *****************************************************************************/
struct uart_mvp2000_port {
	/** uart port structure provided by kernel*/
	struct uart_port	port;
};

/******************************************************************************
 *       FUNCTION        :       mvp2000_uart_stop_tx
 ******************************************************************************/
/**
 *      @brief          stop transmitting
 *
 *      @return         None
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Global function
 *
 *****************************************************************************/
static void mvp2000_uart_stop_tx(
				    /** which UART port */
				    struct uart_port *port
				)
{
	unsigned int cr;

	DRV_MSG("%s\n", __FUNCTION__);

	cr = UART_GET_IER(port);
	cr &= ~UART_IER_ETEI;
	UART_PUT_IER(port, cr);
}

/******************************************************************************
 *       FUNCTION        :       mvp2000_uart_start_tx
 ******************************************************************************/
/**
 *      @brief          start transmitting
 *
 *      @return         None
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Global function
 *
 *****************************************************************************/
static void mvp2000_uart_start_tx(
				    /** which UART port */
				    struct uart_port *port
				)
{
	unsigned int cr;

        DRV_MSG("%s\n", __FUNCTION__);

	cr = UART_GET_IER(port);
	cr |= UART_IER_ETEI;
	UART_PUT_IER(port, cr);
}

/******************************************************************************
 *       FUNCTION        :       mvp2000_uart_stop_rx
 ******************************************************************************/
/**
 *      @brief          stop receiving
 *
 *      @return         None
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Global function
 *
 *****************************************************************************/
static void mvp2000_uart_stop_rx(
				    /** which UART port */
				    struct uart_port *port
				)
{
	unsigned int cr;

	DRV_MSG("%s\n", __FUNCTION__);

	cr = UART_GET_IER(port);
	cr &= ~UART_IER_ERDI;
	UART_PUT_IER(port, cr);
}

/******************************************************************************
 *       FUNCTION        :       mvp2000_uart_enable_ms
 ******************************************************************************/
/**
 *      @brief          enable modem status for hardware control
 *
 *      @return         None
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Global function
 *
 *****************************************************************************/
static void mvp2000_uart_enable_ms(
				    /** which UART port */
				    struct uart_port *port
				)
{
        unsigned int ier_status;

        DRV_MSG("%s\n", __FUNCTION__);

        ier_status = UART_GET_IER(port);
        ier_status |= UART_IER_EMSI;
        UART_PUT_IER(port, ier_status);
}

/******************************************************************************
 *       FUNCTION        :       mvp2000_uart_rx_chars
 ******************************************************************************/
/**
 *      @brief          receive chars
 *
 *      @return         None
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Private function
 *
 *****************************************************************************/
static void
#ifdef SUPPORT_SYSRQ
mvp2000_uart_rx_chars(struct uart_port *port, struct pt_regs *regs)
#else
mvp2000_uart_rx_chars(struct uart_port *port)
#endif
{
	struct tty_struct *tty = port->info->tty;
	unsigned int status, ch, flag, lsr, max_count = 256;

	status = UART_GET_LSR(port);;
	while ((status & UART_LSR_DR) && max_count--) {
		
		ch = UART_GET_CHAR(port);
		flag = TTY_NORMAL;
		port->icount.rx++;

		/*
		 * Note that the error handling code is
		 * out of the main execution path
		 */
		lsr = UART_GET_LSR(port); 
		if (unlikely(lsr & UART_LSR_ANY)) {
			if (lsr & UART_LSR_BE) {
				lsr &= ~(UART_LSR_FE | UART_LSR_PE);
				port->icount.brk++;
				if (uart_handle_break(port))
					goto ignore_char;
			} else if (lsr & UART_LSR_PE)
				port->icount.parity++;
			else if (lsr & UART_LSR_FE)
				port->icount.frame++;
			if (lsr & UART_LSR_OE)
				port->icount.overrun++;

			lsr &= port->read_status_mask;

			if (lsr & UART_LSR_BE)
				flag = TTY_BREAK;
			else if (lsr & UART_LSR_PE)
				flag = TTY_PARITY;
			else if (lsr & UART_LSR_FE)
				flag = TTY_FRAME;
		}

#ifdef SUPPORT_SYSRQ
		if (uart_handle_sysrq_char(port, ch & 255, regs))
#else
	        if (uart_handle_sysrq_char(port, ch & 255))
#endif
		{
			goto ignore_char;
		}

		uart_insert_char(port, lsr, UART_LSR_OE, ch, flag);

	ignore_char:
		status = UART_GET_LSR(port);
	}
	tty_flip_buffer_push(tty);
	return;
}

/******************************************************************************
 *       FUNCTION        :       mvp2000_uart_tx_chars
 ******************************************************************************/
/**
 *      @brief          transmit chars
 *
 *      @return         None
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Private function
 *
 *****************************************************************************/
static void mvp2000_uart_tx_chars(
				    /** which UART port */
				    struct uart_port *port
				)
{
	struct circ_buf *xmit = &port->info->xmit;
	int count;

	if (port->x_char) {
		UART_PUT_CHAR(port, port->x_char);
		port->icount.tx++;
		port->x_char = 0;
		return;
	}
	if (uart_circ_empty(xmit) || uart_tx_stopped(port)) {
		mvp2000_uart_stop_tx(port);
		return;
	}

	count = port->fifosize >> 1;
	do {
		UART_PUT_CHAR(port, xmit->buf[xmit->tail]);
		xmit->tail = (xmit->tail + 1) & (UART_XMIT_SIZE - 1);
		port->icount.tx++;
		if (uart_circ_empty(xmit))
			break;
	} while (--count > 0);

	if (uart_circ_chars_pending(xmit) < WAKEUP_CHARS)
		uart_write_wakeup(port);

	if (uart_circ_empty(xmit))
		mvp2000_uart_stop_tx(port);
}

/******************************************************************************
 *       FUNCTION        :       mvp2000_modem_status
 ******************************************************************************/
/**
 *      @brief          handle modem status for hardware control
 *
 *      @return         None
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Private function
 *
 *****************************************************************************/
static void mvp2000_modem_status(
				    /** which UART port */
				    struct uart_port* port
				)
{
        unsigned int status;

        status = UART_GET_MSR(port); /* Clear modem status intr */

        DRV_MSG("%s: modem status : 0x%x\n", __FUNCTION__, status);

        status = (status & U_CTS) >> 4;
        uart_handle_cts_change(port, status);
        wake_up_interruptible(&port->info->delta_msr_wait);
}

/******************************************************************************
 *       FUNCTION        :       mvp2000_uart_int
 ******************************************************************************/
/**
 *      @brief          interrupt handler
 *
 *      @return         irqreturn_t
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Global function
 *
 *****************************************************************************/
static irqreturn_t mvp2000_uart_int(
				    /** irq number */
				    int irq,

#ifndef SUPPORT_SYSRQ
				    /** device ID */
				    void *dev_id
#else
				    /** device ID */
				    void *devid,

				    /** only used with SUPPORT_SYSRQ */
				    struct pt_regs *regs
#endif
				)
{
	struct uart_port *port;
	volatile unsigned int status;
	volatile unsigned int iir;
	volatile unsigned int pass_counter;

	spin_lock(&port->lock);

	port = dev_id;
	pass_counter = MVP2000_ISR_PASS_LIMIT;

	status = UART_GET_LSR(port);

        DRV_MSG("%s: Line Status : 0x%x\n", __FUNCTION__, status);

	do {
		if (status & UART_LSR_DR)
#ifdef SUPPORT_SYSRQ
			mvp2000_uart_rx_chars(port, regs);
#else
			mvp2000_uart_rx_chars(port);
#endif

	        if (port->flags & UPF_CONS_FLOW)
	        {
	            if ((UART_GET_IIR(port) & 0xF) == 0)
	            {
	                mvp2000_modem_status(port);
	            }
	        }

		if (status & UART_LSR_THRE) {
			mvp2000_uart_tx_chars(port);
                	iir = UART_GET_IIR(port);
	        }

		if (pass_counter-- == 0)
			break;

		status = UART_GET_LSR(port);
	} while (status & (UART_LSR_THRE|UART_LSR_DR));
	
	spin_unlock(&port->lock);
	
	return IRQ_HANDLED;
}

/******************************************************************************
 *       FUNCTION        :       mvp2000_uart_tx_empty
 ******************************************************************************/
/**
 *      @brief          check tx empty or not
 *
 *      @return         0 if transmitter register is 0
 *			TIOCSER_TEMT otherwise
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Global function
 *
 *****************************************************************************/
static unsigned int mvp2000_uart_tx_empty(
					    /** which port */
					    struct uart_port *port
					)
{
	return UART_GET_LSR(port) & UART_LSR_TEMT ? TIOCSER_TEMT : 0;
}

/******************************************************************************
 *       FUNCTION        :       mvp2000_uart_get_mctrl
 ******************************************************************************/
/**
 *      @brief          get CTS state
 *
 *      @return         CTS state
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Global function
 *
 *****************************************************************************/
static unsigned int mvp2000_uart_get_mctrl(
					    /** which port */
					    struct uart_port *port
					)
{
        unsigned int result = 0;

        if (port->flags & UPF_CONS_FLOW)
        {
            result = UART_GET_MSR(port);
            if((result & U_CTS) == U_CTS)
	    {
                result |= TIOCM_CTS;
	    }
        }

        return result;
}

/******************************************************************************
 *       FUNCTION        :       mvp2000_uart_set_mctrl
 ******************************************************************************/
/**
 *      @brief          set CTS/RTS state
 *
 *      @return         None
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Global function
 *
 *****************************************************************************/
static void mvp2000_uart_set_mctrl(
				    /** which port */
				    struct uart_port *port,

				    /** mctrl state */
				    unsigned int mctrl
				)
{
        unsigned int ctrl = 0;

        DRV_MSG("%s: set to : 0x%x\n", __FUNCTION__, mctrl);

        ctrl = UART_GET_MCR(port);
        if (mctrl & TIOCM_RTS)
        {
            /*Set RTS */
            ctrl |= U_RTSE;
        }
        else
        {
            /*Clear RTS*/
            ctrl &= ~U_RTSE;
        }
        UART_PUT_MCR(port, ctrl);
}

/******************************************************************************
 *       FUNCTION        :       mvp2000_uart_break_ctl
 ******************************************************************************/
/**
 *      @brief          set LCR break control
 *
 *      @return         None
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Global function
 *
 *****************************************************************************/
static void mvp2000_uart_break_ctl(
				    /** which port */
				    struct uart_port *port,

				    /** break state */
				    int break_state
				)
{
	unsigned long flags;
	unsigned int lcr;

	spin_lock_irqsave(&port->lock, flags);
	lcr = UART_GET_LCR(port);
	if (break_state == -1)
		lcr |= UART_LCR_BRK;
	else
		lcr &= ~UART_LCR_BRK;
	UART_PUT_LCR(port, lcr);
	spin_unlock_irqrestore(&port->lock, flags);
}

/******************************************************************************
 *       FUNCTION        :       mvp2000_uart_startup
 ******************************************************************************/
/**
 *      @brief          start up uart device
 *
 *      @return         0 if successful
 *			not 0 otherwise
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Global function
 *
 *****************************************************************************/
static int mvp2000_uart_startup(
				/** which port */
				struct uart_port *port
			    )
{
	int retval;

	/*
	 * Allocate the IRQ
	 */
	retval = request_irq(port->irq, mvp2000_uart_int, IRQF_SHARED, "mvp2000_serial", port);
	if (retval)
	{
	        printk("%s: Can't Get IRQ\n", __FUNCTION__);

		return retval;
	}

	/*
	 * Finally, enable interrupts
	 */
	IRQ_SET_HIGH_LEVEL(port->irq);
	IRQ_SET_LEVEL_TRIGGER(port->irq);
	UART_PUT_IER(port, UART_IER_ERDI);

        DRV_INFO("%s: IER state : 0x%x\n", __FUNCTION__, UART_GET_IER(port));

	return 0;
}

/******************************************************************************
 *       FUNCTION        :       mvp2000_uart_shutdown
 ******************************************************************************/
/**
 *      @brief          stop uart device
 *
 *      @return         None
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Global function
 *
 *****************************************************************************/
static void mvp2000_uart_shutdown(
				    /** which port */
				    struct uart_port *port
				)
{
	/*
	 * Free the interrupt
	 */
	free_irq(port->irq, port);

	/*
	 * disable all interrupts, disable the port
	 */
	UART_PUT_IER(port, 0);

	/* disable break condition and fifos */
	UART_PUT_LCR(port, UART_GET_LCR(port)&(~UART_LCR_BRK));
	UART_PUT_FCR(port, UART_GET_FCR(port)&(~UART_FCR_FIFOE));
}

/******************************************************************************
 *       FUNCTION        :       mvp2000_set_termios
 ******************************************************************************/
/**
 *      @brief          set terminal settings
 *
 *      @return         None
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Global function
 *
 *****************************************************************************/
static void mvp2000_set_termios(
				/** which port */
				struct uart_port *port,

				/** which terminal */
				struct ktermios *termios,

				/** current terminal */
				struct ktermios *old
			    )
{
	unsigned int lcr, fcr = 0;
	unsigned long flags;
	unsigned int baud, quot;
	int ch, i;

    if(termios->c_cflag & CRTSCTS)
    {
        port->flags |= UPF_CONS_FLOW;
        UART_PUT_IER(port, UART_GET_IER(port) | UART_IER_EMSI);
        mvp2000_uart_set_mctrl(port, TIOCM_RTS);
    }
    else
    {
        port->flags &= (~UPF_CONS_FLOW);
        if((UART_GET_IER(port) & UART_IER_EMSI) != 0)
        {
            UART_PUT_IER(port, UART_GET_IER(port) & (~UART_IER_EMSI));
            mvp2000_uart_set_mctrl(port, (~TIOCM_RTS));
        }
    }
 
	/*
	 * Ask the core to calculate the divisor for us.
	 */
	baud = uart_get_baud_rate(port, termios, old, 0, port->uartclk/16);
	quot = port->uartclk / (16 * baud);

	switch (termios->c_cflag & CSIZE) {
	case CS5:
		lcr = UART_LCR_WLEN_5;
		break;
	case CS6:
		lcr = UART_LCR_WLEN_6;
		break;
	case CS7:
		lcr = UART_LCR_WLEN_7;
		break;
	default: // CS8
		lcr = UART_LCR_WLEN_8;
		break;
	}
	if (termios->c_cflag & CSTOPB)
		lcr |= UART_LCR_STOP;
	if (termios->c_cflag & PARENB) {
		lcr |= UART_LCR_PEN;
		if (!(termios->c_cflag & PARODD))
			lcr |= UART_LCR_EPS;
	}
	if (port->fifosize > 1)
		fcr |= (UART_FCR_XMITR|UART_FCR_RCVRR|UART_FCR_FIFOE);


	spin_lock_irqsave(&port->lock, flags);

	/*
	 * Update the per-port timeout.
	 */
	uart_update_timeout(port, termios->c_cflag, baud);

	port->read_status_mask = UART_LSR_OE;
	if (termios->c_iflag & INPCK)
		port->read_status_mask |= UART_LSR_FE | UART_LSR_PE;
	if (termios->c_iflag & (BRKINT | PARMRK))
		port->read_status_mask |= UART_LSR_BE;

	/*
	 * Characters to ignore
	 */
	port->ignore_status_mask = 0;
	if (termios->c_iflag & IGNPAR)
		port->ignore_status_mask |= UART_LSR_FE | UART_LSR_PE;
	if (termios->c_iflag & IGNBRK) {
		port->ignore_status_mask |= UART_LSR_BE;
		/*
		 * If we're ignoring parity and break indicators,
		 * ignore overruns too (for real raw support).
		 */
		if (termios->c_iflag & IGNPAR)
			port->ignore_status_mask |= UART_LSR_OE;
	}

	/*
	 * Ignore all characters if CREAD is not set.
	 */
	if ((termios->c_cflag & CREAD) == 0)
		port->ignore_status_mask |= UART_DUMMY_RSR_RX;


	/* Set baud rate */
	UART_PUT_LCR(port, UART_LCR_DLAB);	/* enable Divisor Latach Address Bit */
	UART_PUT_DLH(port, ((quot >> 8) & 0xFF));
	UART_PUT_DLL(port, (quot & 0xff));

	UART_PUT_FCR(port, fcr);
	UART_PUT_LCR(port, lcr);
	for (i = 0; i < 16; i++) {
		ch = UART_GET_CHAR(port);  /* Clear TIMEOUT Interrupt */
	}

        DRV_INFO("%s: IER state : 0x%x\n", __FUNCTION__, UART_GET_IER(port));

	spin_unlock_irqrestore(&port->lock, flags);
}

/******************************************************************************
 *       FUNCTION        :       mvp2000_uart_type
 ******************************************************************************/
/**
 *      @brief          get UART device type
 *
 *      @return         "ASPEED UART" if type is PORT_GUCMVP2000
 *			NULL otherwise
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Global function
 *
 *****************************************************************************/
static const char *mvp2000_uart_type(
					/** which UART port */
					struct uart_port *port
				    )
{
	return port->type == PORT_GUCMVP2000 ? "ASPEED UART" : NULL;
}

/******************************************************************************
 *       FUNCTION        :       mvp2000_uart_release_port
 ******************************************************************************/
/**
 *      @brief          release the memory region(s) being used by 'port'
 *
 *      @return         None
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Global function
 *
 *****************************************************************************/
static void mvp2000_uart_release_port(
					/** which port */
					struct uart_port *port
				    )
{
	release_mem_region(port->mapbase, UART_PORT_SIZE);
}

/******************************************************************************
 *       FUNCTION        :       mvp2000_uart_request_port
 ******************************************************************************/
/**
 *      @brief          Request the memory region(s) being used by 'port'
 *
 *      @return         0 if successful
 *			-EBUSY otherwise
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Global function
 *
 *****************************************************************************/
static int mvp2000_uart_request_port(
					/** which UART port */
					struct uart_port *port
				    )
{
	return request_mem_region(port->mapbase, UART_PORT_SIZE, "serial_gucmvp2000")
			!= NULL ? 0 : -EBUSY;
}

/******************************************************************************
 *       FUNCTION        :       mvp2000_uart_config_port
 ******************************************************************************/
/**
 *      @brief          Configure/autoconfigure the port.
 *
 *      @return         None
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Global function
 *
 *****************************************************************************/
static void mvp2000_uart_config_port(
					/** which UART port */
					struct uart_port *port,

					/** flags to be used */
					int flags
				    )
{
	if (flags & UART_CONFIG_TYPE) {
		port->type = PORT_GUCMVP2000;
		mvp2000_uart_request_port(port);
	}
}

/******************************************************************************
 *       FUNCTION        :       mvp2000_uart_verify_port
 ******************************************************************************/
/**
 *      @brief          verify the new serial_struct (for TIOCSSERIAL).
 *
 *      @return         0 if no error
 *			-EINVAL otherwise
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Global function
 *
 *****************************************************************************/
static int mvp2000_uart_verify_port(
					/** which UART port */
					struct uart_port *port,

					/** serial info */
					struct serial_struct *ser
				)
{
	int ret = 0;
	if (ser->type != PORT_UNKNOWN && ser->type != PORT_GUCMVP2000)
		ret = -EINVAL;
	if (ser->irq < 0 || ser->irq >= NR_IRQS)
		ret = -EINVAL;
	if (ser->baud_base < 9600)
		ret = -EINVAL;
	return ret;
}

static struct uart_ops mvp2000_pops = {
	.tx_empty	= mvp2000_uart_tx_empty,
	.set_mctrl	= mvp2000_uart_set_mctrl,
	.get_mctrl	= mvp2000_uart_get_mctrl,
	.stop_tx	= mvp2000_uart_stop_tx,
	.start_tx	= mvp2000_uart_start_tx,
	.stop_rx	= mvp2000_uart_stop_rx,
	.enable_ms	= mvp2000_uart_enable_ms,
	.break_ctl	= mvp2000_uart_break_ctl,
	.startup	= mvp2000_uart_startup,
	.shutdown	= mvp2000_uart_shutdown,
	.set_termios	= mvp2000_set_termios,
	.type		= mvp2000_uart_type,
	.release_port	= mvp2000_uart_release_port,
	.request_port	= mvp2000_uart_request_port,
	.config_port	= mvp2000_uart_config_port,
	.verify_port	= mvp2000_uart_verify_port,
};

static struct uart_mvp2000_port mvp2000_ports[UART_NR] = {
	{
		.port	= {
            /* Avocent --- AndyC
             * modified ASPEED_UART0_VIRTUAL, ASPEED_UART0_BASE and
             * IRQ_UARTINT0 to allow build time serial hardware
             * configuration */
			.membase	= (void *) PORT0_VIRTUAL, // (void *) ASPEED_UART0_VIRTUAL,
			.mapbase	= PORT0_BASE, //ASPEED_UART0_BASE,
			.iotype		= SERIAL_IO_MEM,
			.irq		= PORT0_IRQ, //IRQ_UARTINT0,
			.uartclk	= (24*1000000L),
			.fifosize	= 16,
			.ops		= &mvp2000_pops,
			.flags		= ASYNC_BOOT_AUTOCONF,
			.line		= 0,
		},
	},
	{
		.port	= {
            /* Avocent --- AndyC
             * modified ASPEED_UART1_VIRTUAL, ASPEED_UART1_BASE and
             * IRQ_UARTINT1 to allow build time serial hardware
             * configuration */
			.membase	= (void *) PORT1_VIRTUAL, //(void *) ASPEED_UART1_VIRTUAL,
			.mapbase	= PORT1_BASE, //ASPEED_UART1_BASE,
			.iotype		= SERIAL_IO_MEM,
			.irq		= PORT1_IRQ, //IRQ_UARTINT1,
			.uartclk	= (24*1000000L),
			.fifosize	= 16,
			.ops		= &mvp2000_pops,
			.flags		= ASYNC_BOOT_AUTOCONF,
			.line		= 1,
		},
	}
};

#ifdef CONFIG_SERIAL_ASPEED_CONSOLE

/******************************************************************************
 *       FUNCTION        :       mvp2000_uart_console_write
 ******************************************************************************/
/**
 *      @brief           write data to console
 *
 *      @return         None
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Global function
 *
 *****************************************************************************/
static void mvp2000_uart_console_write(
					/** which console */
					struct console *co,

					/** input string */
					const char *s,

					/** number of chars in string*/
					unsigned int count
				    )
{
	struct uart_port *port = &mvp2000_ports[co->index].port;
	unsigned int status, old_ier;
	int i;

	/*
	 *	First save the IER then disable the interrupts
	 */
	old_ier = UART_GET_IER(port);
	UART_PUT_IER(port, 0);

	/*
	 *	Now, do each character
	 */
	for (i = 0; i < count; i++) {
		do {
			status = UART_GET_LSR(port);
		} while (!(status & UART_LSR_THRE));
		UART_PUT_CHAR(port, s[i]);
		if (s[i] == '\n') {
			do {
				status = UART_GET_LSR(port);
			} while (!(status & UART_LSR_THRE));
			UART_PUT_CHAR(port, '\r');
		}
	}

	/*
	 *	Finally, wait for transmitter to become empty
	 *	and restore the IER
	 */
	do {
		status = UART_GET_LSR(port);
	} while (!(status & UART_LSR_TEMT));
	UART_PUT_IER(port, old_ier);
}

/******************************************************************************
 *       FUNCTION        :       mvp2000_uart_console_get_options
 ******************************************************************************/
/**
 *      @brief          get console options
 *
 *      @return         None
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Private function
 *
 *****************************************************************************/
static void __init 
mvp2000_uart_console_get_options(
				    /** which port */
				    struct uart_port *port,

				    /** baud rate */
				    int *baud,

				    /** parity */
				    int *parity,

				    /** bits */
				    int *bits
				)
{
	if (UART_GET_IER(port) & UART_IER_ERDI) {
		unsigned int lcr, quot;
		lcr = UART_GET_LCR(port);

		*parity = 'n';
		if (lcr & UART_LCR_PEN) {
			if (lcr & UART_LCR_EPS)
				*parity = 'e';
			else
				*parity = 'o';
		}

		switch (lcr & UART_LCR_WLEN_MASK) {
		case UART_LCR_WLEN_8:
		default:
		    *bits = 8;
		    break;
		case UART_LCR_WLEN_7:
		    *bits = 7;
		    break;
		case UART_LCR_WLEN_6:
		    *bits = 6;
		    break;
		case UART_LCR_WLEN_5:
		    *bits = 5;
		    break;
		}

		UART_PUT_LCR(port, UART_LCR_DLAB);	/* enable Divisor Latach Address Bit */
		quot = UART_GET_DLL(port) | (UART_GET_DLH(port) << 8);
		*baud = port->uartclk / (16 * quot);
		UART_PUT_LCR(port, lcr);
	}
}

/******************************************************************************
 *       FUNCTION        :       mvp2000_uart_console_setup
 ******************************************************************************/
/**
 *      @brief          setup/configure console
 *
 *      @return         0 if successful
 *			non 0 otherwise
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Global function
 *
 *****************************************************************************/
static int __init mvp2000_uart_console_setup(
						/** which console */
						struct console *co,
						/** options to be used */
						char *options
					    )
{
	struct uart_port *port;
	int baud = CONFIG_SERIAL_ASPEED_CONSOLE_BAUD;
	int bits = 8;
	int parity = 'n';
	int flow = 'n';

	/*
	 * Check whether an invalid uart number has been specified, and
	 * if so, search for the first available port that does have
	 * console support.
	 */
	if (co->index >= UART_NR)
		co->index = 0;
	port = &mvp2000_ports[co->index].port;


	if (options)
		uart_parse_options(options, &baud, &parity, &bits, &flow);
	else	
	        mvp2000_uart_console_get_options(port, &baud, &parity, &bits);


	return uart_set_options(port, co, baud, parity, bits, flow);
}

static struct uart_driver mvp2000_reg;

static struct console mvp2000_console = {
	.name		= SERIAL_MVP2000_CONSLE_NAME,
	.write		= mvp2000_uart_console_write,
	.device		= uart_console_device,
	.setup		= mvp2000_uart_console_setup,
	.flags		= CON_PRINTBUFFER,
	.index		= -1,
	.data		= &mvp2000_reg,
};

/******************************************************************************
 *       FUNCTION        :       mvp2000_uart_console_init
 ******************************************************************************/
/**
 *      @brief          register/init console
 *
 *      @return         0
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: private function
 *
 *****************************************************************************/
int __init mvp2000_uart_console_init(void)
{
	register_console(&mvp2000_console);
	return 0;
}
console_initcall(mvp2000_uart_console_init);


#define MVP2000_CONSOLE	&mvp2000_console
#else
#define MVP2000_CONSOLE	NULL
#endif


static struct uart_driver mvp2000_reg = {
	.owner			= THIS_MODULE,	
	.major			= SERIAL_MVP2000_MAJOR,
	.minor			= SERIAL_MVP2000_MINOR,	
//	.devfs_name		= SERIAL_MVP2000_DEVFS_NAME,    //AVCT: FOR 2.6.18.6 linux kernel
	.dev_name		= SERIAL_MVP2000_TTY_NAME,
	.nr			= UART_NR,
	.cons			= MVP2000_CONSOLE,
};

/******************************************************************************
 *       FUNCTION        :       mvp2000_uart_init
 ******************************************************************************/
/**
 *      @brief          init function for the driver
 *
 *      @return         0 if successful
 *			non 0 otherwise
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Global function
 *
 *****************************************************************************/
static int __init mvp2000_uart_init(void)
{
	int ret;
	int i;

	ret = uart_register_driver(&mvp2000_reg);
	if (ret == 0) {
		for (i = 0; i < UART_NR; i++)
			uart_add_one_port(&mvp2000_reg, &mvp2000_ports[i].port);
	}

        /* initial debugfs related resource */
        aess_debugfs_default_create(driver_name, NULL, 0, 0);
        aess_debugfs_create_file(driver_name, "u32", (u32 *)&u32tmpcounter, DBG_TYPE32, 0);
        aess_debugfs_create_file(driver_name, "u16", (u16 *)&u16tmpcounter, DBG_TYPE16, 0);
        aess_debugfs_create_file(driver_name, "u8", (u8 *)&u8tmpcounter, DBG_TYPE8, 0);

	return ret;
}

/******************************************************************************
 *       FUNCTION        :       mvp2000_uart_exit
 ******************************************************************************/
/**
 *      @brief          exit function for the driver
 *
 *      @return         None
 *
 *      @dependency	None
 *
 *      @limitation     None
 *
 *      @warning        None
 *
 *      @note           None
 *
 *      @internal       Function Type: Global function
 *
 *****************************************************************************/
static void __exit mvp2000_uart_exit(void)
{
	int i;

	for (i = 0; i < UART_NR; i++)
		uart_remove_one_port(&mvp2000_reg, &mvp2000_ports[i].port);

	uart_unregister_driver(&mvp2000_reg);

        /* remove debugfs module */
        aess_debugfs_remove(driver_name, NULL, 0, 0);
}

module_init(mvp2000_uart_init);
module_exit(mvp2000_uart_exit);

MODULE_AUTHOR("Andy Chang");
MODULE_DESCRIPTION("ASPEED serial port driver");
MODULE_VERSION("0.1.0.1");
MODULE_LICENSE("GPL");

/* End of code */

