/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __address_h
#define __address_h                     1

/* Base Address */
#define ASPEED_SM_BASE                  0x10000000	/* STATIC MEMORY */
#define ASPEED_SMC_CONTROLLER_BASE      0x16000000	/* SMC CONTROLLER */
#define ASPEED_AHB_CONTROLLER_BASE      0x1E600000	/* AHB CONTROLLER */
#define ASPEED_MAC1_BASE                0x1E660000      /* MAC1 */ 
#define ASPEED_MAC2_BASE                0x1E680000      /* MAC2 */
#define ASPEED_USB20_BASE               0x1E6A0000	/* USB 2.0 */
#define ASPEED_VIC_BASE                 0x1E6C0000	/* VIC */
//#define ASPEED_APB1_BASE                0x1E6E0000	/* APB BRIDGE 1 */
#define ASPEED_MMC_BASE                	0x1E6E0000	/* MMC */
#define ASPEED_USB11_BASE              	0x1E6E1000	/* USB11 */
#define ASPEED_SCU_BASE                	0x1E6E2000	/* SCU */
#define ASPEED_CRYPTO_BASE              0x1E6E3000	/* Crypto */
#define ASPEED_ADPCM_BASE               0x1E6E4000	/* ADPCM */
#define ASPEED_I2S_BASE                	0x1E6E5000	/* I2S */
#define ASPEED_GRAPHIC_BASE             0x1E6E6000	/* Graphics */
#define ASPEED_VIDEO_BASE               0x1E700000	/* VIDEO ENGINE */
#define ASPEED_AHB_TO_PBUS_BASE         0x1E720000	/* APB -> PBUS */
#define ASPEED_MDMA_BASE                0x1E740000	/* MDMA */
#define ASPEED_2D_BASE                  0x1E760000	/* 2D */
//#define ASPEED_APB2_BASE                0x1E780000	/* APB BRIDGE 2 */
#define ASPEED_GPIO_BASE               	0x1E780000	/* MMC */
#define ASPEED_RTC_BASE               	0x1E781000	/* RTC */
#define ASPEED_TIMER_BASE               0x1E782000	/* TIMER */
#define ASPEED_UART0_BASE               0x1E783000	/* UART1 */
#define ASPEED_UART1_BASE               0x1E784000	/* UART2 */
#define ASPEED_WDT_BASE               	0x1E785000	/* WDT */
#define ASPEED_PWM_BASE               	0x1E786000	/* PWM */
#define ASPEED_VUART0_BASE              0x1E787000	/* VUART1 */
#define ASPEED_VUART1_BASE              0x1E788000	/* VUART2 */
#define ASPEED_LPC_BASE               	0x1E789000	/* LPC */
#define ASPEED_I2C_BASE               	0x1E78A000	/* I2C */
#define ASPEED_PECI_BASE               	0x1E78B000	/* PECI */
#define ASPEED_PCIARBITER_BASE          0x1E78C000	/* PCI ARBITER */
#define ASPEED_PCI_MASTER1_BASE         0x60000000	/* PCI MASTER 1 */
#define ASPEED_PCI_MASTER2_BASE         0x80000000	/* PCI MASTER 2 */

#define ASPEED_SDRAM_BASE               0x40000000	/* SDRAM */

#define ASPEED_IO_START                 0x10000000
#define ASPEED_IO_SIZE                  0x0F000000


/*
 * VIC
 */
/* Interrupt Controllers */
#define ASPEED_VIC_STATUS_OFFSET	0x00
#define ASPEED_VIC_RAW_STATUS_OFFSET    0x08
#define ASPEED_VIC_INT_SEL_OFFSET	0x0C
#define ASPEED_VIC_ENABLE_SET_OFFSET    0x10
#define ASPEED_VIC_ENABLE_CLEAR_OFFSET  0x14
#define ASPEED_VIC_SENSE_OFFSET         0x24
#define ASPEED_VIC_BOTH_EDGE_OFFSET	0x28
#define ASPEED_VIC_EVENT_OFFSET         0x2C
#define ASPEED_VIC_EDGE_CLEAR_OFFSET    0x38

#define ASPEED_VIC_STATUS               (ASPEED_VIC_BASE+ASPEED_VIC_STATUS_OFFSET)
#define ASPEED_VIC_RAW_STATUS           (ASPEED_VIC_BASE+ASPEED_VIC_RAW_STATUS_OFFSET)
#define ASPEED_VIC_ENABLE_SET           (ASPEED_VIC_BASE+ASPEED_VIC_ENABLE_SET_OFFSET)
#define ASPEED_VIC_ENABLE_CLEAR         (ASPEED_VIC_BASE+ASPEED_VIC_ENABLE_CLEAR_OFFSET)
#define ASPEED_VIC_SENSE                (ASPEED_VIC_BASE+ASPEED_VIC_SENSE_OFFSET)
#define ASPEED_VIC_BOTH_EDGE		(ASPEED_VIC_BASE+ASPEED_VIC_BOTH_EDGE_OFFSET)
#define ASPEED_VIC_EVENT                (ASPEED_VIC_BASE+ASPEED_VIC_EVENT_OFFSET)
#define ASPEED_VIC_EDGE_CLEAR           (ASPEED_VIC_BASE+ASPEED_VIC_EDGE_CLEAR_OFFSET)

/* Interrupt Controllers 2 */
#define ASPEED_VIC2_STATUS_OFFSET	0x38
#define ASPEED_VIC2_RAW_STATUS_OFFSET   0x38		/* no use */
#define ASPEED_VIC2_INT_SEL_OFFSET	0x0C		/* no use */
#define ASPEED_VIC2_ENABLE_SET_OFFSET   0x28
#define ASPEED_VIC2_ENABLE_CLEAR_OFFSET 0x28
#define ASPEED_VIC2_SENSE_OFFSET        0x30
#define ASPEED_VIC2_BOTH_EDGE_OFFSET	0x34
#define ASPEED_VIC2_EVENT_OFFSET        0x2C
#define ASPEED_VIC2_EDGE_CLEAR_OFFSET   0x38

#define ASPEED_VIC2_STATUS              (ASPEED_GPIO_BASE+ASPEED_VIC2_STATUS_OFFSET)
#define ASPEED_VIC2_RAW_STATUS          (ASPEED_GPIO_BASE+ASPEED_VIC2_RAW_STATUS_OFFSET)
#define ASPEED_VIC2_ENABLE_SET          (ASPEED_GPIO_BASE+ASPEED_VIC2_ENABLE_SET_OFFSET)
#define ASPEED_VIC2_ENABLE_CLEAR        (ASPEED_GPIO_BASE+ASPEED_VIC2_ENABLE_CLEAR_OFFSET)
#define ASPEED_VIC2_SENSE               (ASPEED_GPIO_BASE+ASPEED_VIC2_SENSE_OFFSET)
#define ASPEED_VIC2_BOTH_EDGE		(ASPEED_GPIO_BASE+ASPEED_VIC2_BOTH_EDGE_OFFSET)
#define ASPEED_VIC2_EVENT               (ASPEED_GPIO_BASE+ASPEED_VIC2_EVENT_OFFSET)
#define ASPEED_VIC2_EDGE_CLEAR          (ASPEED_GPIO_BASE+ASPEED_VIC2_EDGE_CLEAR_OFFSET)

/* Interrupt numbers and bit positions */
/* on VIC1 */
#define INT_ECC				0
#define INTMASK_ECC			(1 << INT_ECC)

#define INT_MIC				1
#define INTMASK_MIC			(1 << INT_MIC)

#define INT_MAC1           		2
#define INTMASK_MAC1       		(1 << INT_MAC1)

#define INT_MAC2           		3
#define INTMASK_MAC2       		(1 << INT_MAC2)

#define INT_HAC           		4
#define INTMASK_HAC       		(1 << INT_HAC)

#define INT_USB20           		5
#define INTMASK_USB20       		(1 << INT_USB20)

#define INT_MDMA           		6
#define INTMASK_MDMA       		(1 << INT_MDMA)

#define INT_VIDEO           		7
#define INTMASK_VIDEO       		(1 << INT_VIDEO)

#define INT_LPC				8
#define INTMASK_LPC			(1 << INT_LPC)

#define INT_UARTINT0            	9
#define INTMASK_UARTINT0        	(1 << INT_UARTINT0)

#define INT_UARTINT1            	10
#define INTMASK_UARTINT1        	(1 << INT_UARTINT1)

#define INT_I2S           	        11
#define INTMASK_I2S       		(1 << INT_I2S)

#define INT_I2C				12
#define INTMASK_I2C			(1 << INT_I2C)

#define INT_USB11           		13
#define INTMASK_USB11       		(1 << INT_USB11)

#define INT_ADPCM           		14
#define INTMASK_ADPDM       		(1 << INT_ADPCM)

#define INT_PECI           		15
#define INTMASK_PECI       		(1 << INT_PECI)

#define INT_TIMERINT0           	16
#define INTMASK_TIMERINT0       	(1 << INT_TIMERINT0)

#define INT_TIMERINT1           	17
#define INTMASK_TIMERINT1       	(1 << INT_TIMERINT1)

#define INT_TIMERINT2           	18
#define INTMASK_TIMERINT2       	(1 << INT_TIMERINT2)

#define INT_FLASH		      	19
#define INTMASK_FLASH       		(1 << INT_FLASH)

#define INT_GPIO           		20
#define INTMASK_GPIO       		(1 << INT_GPIO)

#define INT_SCU          		21
#define INTMASK_SCU      		(1 << INT_SCU)

#define INT_RTC_SEC			22
#define INTMASK_RTC_SEC			(1 << INT_RTC_SEC)

#define INT_RTC_DAY			23
#define INTMASK_RTC_DAY			(1 << INT_RTC_DAY)

#define INT_RTC_HOUR			24
#define INTMASK_RTC_HOUR		(1 << INT_RTC_HOUR)

#define INT_RTC_MIN			25
#define INTMASK_RTC_MIN			(1 << INT_RTC_MIN)

#define INT_RTC_ALARM			26
#define INTMASK_RTC_ALARM		(1 << INT_RTC_ALARM)

#define INT_WDT           		27
#define INTMASK_WDT        		(1 << INT_WDT)

#define INT_TACHO           		28
#define INTMASK_TACHO       		(1 << INT_TACHO)

#define INT_2D           		29
#define INTMASK_2D       		(1 << INT_2D)

#define INT_PCI           		30
#define INTMASK_PCI       		(1 << INT_PCI)

#define INT_AHBC			31
#define INTMASK_AHBC			(1 << INT_AHBC)

/* Mask of valid system controller interrupts */
/* VIC means VIC1 */
#define ASPEED_VIC_VALID_INTMASK	( \
					 INTMASK_ECC | \
                                         INTMASK_MIC | \
                                         (INTMASK_MAC1 | INTMASK_MAC2) | \
                                         INTMASK_HAC | \
                                         INTMASK_USB20 | \
                                         INTMASK_MDMA | \
                                         INTMASK_VIDEO | \
                                         INTMASK_LPC | \
                                         (INTMASK_UARTINT0 | INTMASK_UARTINT1) | \
                                         INTMASK_I2S | \
                                         INTMASK_I2C | \
                                         INTMASK_USB11 | \
                                         INTMASK_ADPDM | \
                                         INTMASK_PECI | \
                                         (INTMASK_TIMERINT0 | INTMASK_TIMERINT1 | INTMASK_TIMERINT2) | \
                                         INTMASK_FLASH | \
                                         INTMASK_GPIO | \
                                         INTMASK_SCU | \
					 (INTMASK_RTC_SEC | INTMASK_RTC_DAY | INTMASK_RTC_HOUR | INTMASK_RTC_MIN | INTMASK_RTC_ALARM) | \
                                         INTMASK_WDT | \
                                         INTMASK_TACHO | \
                                         INTMASK_2D | \
                                         INTMASK_PCI | \
                                         INTMASK_AHBC \
					)

#define MAXIRQNUM                       31
#define MAXFIQNUM                       29

/*
 * Timer
 */
#define ASPEED_TIMER0_OFFSET		0x0000		/* Timer0 Offset */
#define ASPEED_TIMER1_OFFSET		0x0010		/* Timer1 Offset */
#define ASPEED_TIMER2_OFFSET		0x0020		/* Timer2 Offset */
#define ASPEED_TIMERRC_OFFSET		0x0030		/* Timer RC Offset */

#define ASPEED_TIMER0_VA_BASE		(IO_ADDRESS(ASPEED_TIMER_BASE)+ASPEED_TIMER0_OFFSET)
#define ASPEED_TIMER1_VA_BASE		(IO_ADDRESS(ASPEED_TIMER_BASE)+ASPEED_TIMER1_OFFSET)
#define ASPEED_TIMER2_VA_BASE		(IO_ADDRESS(ASPEED_TIMER_BASE)+ASPEED_TIMER2_OFFSET)
#define ASPEED_TIMERC_VA_BASE		(IO_ADDRESS(ASPEED_TIMER_BASE)+ASPEED_TIMERRC_OFFSET)

#define ASPEED_TIMER_CLKRATE		(ASPEED_EXTCLK)

/*
 * AVOCENT - AndyC begin
 * modifiedd for kernel upgrade from 2.6.18.6 to 2.6.23.1
 *
 */
#define ASPEED_EXTCLK			(1*1000*1000)	/* 1M */
//#define ASPEED_EXTCLK			(1*1024*1024)	/* 1M */
/* AVOCENT - AndyC end */
#define ASPEED_CRYPTO_VA_BASE       IO_ADDRESS(ASPEED_CRYPTO_BASE)
/*
 * UART
 */
#define ASPEED_UART0_VIRTUAL		(IO_ADDRESS(ASPEED_UART0_BASE))
#define ASPEED_UART1_VIRTUAL		(IO_ADDRESS(ASPEED_UART1_BASE))

/*
 * Ticks
 */
//#define TICKS_PER_uSEC                  40 // IP Cam
//#define TICKS_PER_uSEC                  24 // FPGA
#define TICKS_PER_uSEC                  1		/* ASPEED_EXTCLK / 10 ^ 6 */

#define mSEC_1                          1000
#define mSEC_5                          (mSEC_1 * 5)
#define mSEC_10                         (mSEC_1 * 10)
#define mSEC_25                         (mSEC_1 * 25)
#define SEC_1                           (mSEC_1 * 1000)

/*
 * RTC
 */
#define ASPEED_RTC_VA_BASE		(IO_ADDRESS(ASPEED_RTC_BASE))
 
/*
 * Watchdog
 */
#define ASPEED_WDT_VA_BASE		(IO_ADDRESS(ASPEED_WDT_BASE))

#endif


#ifndef hw_regs_h
#define hw_regs_h                     1

/*
 * Serial register address
 */
#define UART_RBR            0x00    /* Receiver Buffer Register */
#define UART_THR            0x00    /* Transmit Holding Register */
#define UART_DLL            0x00    /* Divisor Latch Low Register */
#define UART_DLH            0x04    /* Divisor Latch High Register */
#define UART_IER            0x04    /* Interrupt Enable Register */
#define UART_IIR            0x08    /* Interrupt Identity Register */
#define UART_FCR            0x08    /* FIFO Control Register */
#define UART_LCR            0x0C    /* Line Control Register */
#define UART_MCR            0x10    /* Modem Control Register */
#define UART_LSR            0x14    /* Line Status Register */
#define UART_MSR            0x18    /* Modem Status Register */
#define UART_SCR            0x1C    /* Scratch Register */

/*
 * Watchdog register address
 */
#define WDT_BASE_VA     ASPEED_WDT_VA_BASE
#define WDT_CntSts              (WDT_BASE_VA+0x00)
#define WDT_Reload              (WDT_BASE_VA+0x04)
#define WDT_Restart             (WDT_BASE_VA+0x08)
#define WDT_Ctrl                (WDT_BASE_VA+0x0C)
#define WDT_TimeOut             (WDT_BASE_VA+0x10)
#define WDT_Clr                 (WDT_BASE_VA+0x14)
#define WDT_RstWd               (WDT_BASE_VA+0x18)

/*
 * MTD register address
 */
#define SM_B0_BASE_ADDR (ASPEED_SM_BASE)
#define SM_B2_BASE_ADDR (ASPEED_SM_BASE + (SZ_16M * 2))

/*Static memory virtural address*/
#define SM_B0_V_ADDR    (IO_ADDRESS(SM_B0_BASE_ADDR))
#define SM_B2_V_ADDR    (IO_ADDRESS(SM_B2_BASE_ADDR))

/*Static memory controller virtural address*/
#define SMC_V_ADDR      IO_ADDRESS(ASPEED_SMC_CONTROLLER_BASE)
#define SMC_CE0_V_ADDR  (SMC_V_ADDR + 0x04)
#define SMC_CE1_V_ADDR  (SMC_V_ADDR + 0x08)
#define SMC_CE2_V_ADDR  (SMC_V_ADDR + 0x0C)

/*
 * GPIO register address 
 */

#define GPIO_PORT_BASE_ADDR         IO_ADDRESS(ASPEED_GPIO_BASE)

#define VIC_BASE_ADDR               IO_ADDRESS(ASPEED_VIC_BASE)
#define SCU_BASE_ADDR               IO_ADDRESS(ASPEED_SCU_BASE)

#define SCU_PKR                     (SCU_BASE_ADDR + 0x00)
#define SCU_MULTI_FUNTION1_ADDR     (SCU_BASE_ADDR + 0x74)

#define VIC_IRQ_STATUS_ADDR         (VIC_BASE_ADDR + 0x0)
#define VIC_IRQ_RAW_STATUS_ADDR     (VIC_BASE_ADDR + 0x08)
#define VIC_IRQ_SELECTION_ADDR      (VIC_BASE_ADDR + 0x0c)
#define VIC_IRQ_ENABLE_ADDR         (VIC_BASE_ADDR + 0x10)
#define VIC_IRQ_SENS_ADDR           (VIC_BASE_ADDR + 0x24)
#define VIC_IRQ_TRIGGER_ADDR        (VIC_BASE_ADDR + 0x28)
#define VIC_IRQ_EVENT_ADDR          (VIC_BASE_ADDR + 0x2c)

/* GPIO port base address */

#define REG_DATA_VALUE    (GPIO_PORT_BASE_ADDR + 0x0) 
#define REG_DIRECTION     (GPIO_PORT_BASE_ADDR + 0x4)
#define REG_INT_ENABLE    (GPIO_PORT_BASE_ADDR + 0x8)
#define REG_INT_TYPE0     (GPIO_PORT_BASE_ADDR + 0xc)
#define REG_INT_TYPE1     (GPIO_PORT_BASE_ADDR + 0x10)
#define REG_INT_TYPE2     (GPIO_PORT_BASE_ADDR + 0x14)
#define REG_INT_STATUS    (GPIO_PORT_BASE_ADDR + 0x18)
#define REG_RESET_TOLERANT (GPIO_PORT_BASE_ADDR + 0x1c)

#define REG_EXT_DATA_VALUE (GPIO_PORT_BASE_ADDR + 0x20)
#define REG_EXT_DIRECTION  (GPIO_PORT_BASE_ADDR + 0x24)
#define REG_EXT_INT_ENABLE (GPIO_PORT_BASE_ADDR + 0x28)
#define REG_EXT_INT_TYPE0  (GPIO_PORT_BASE_ADDR + 0x2c)
#define REG_EXT_INT_TYPE1  (GPIO_PORT_BASE_ADDR + 0x30)
#define REG_EXT_INT_TYPE2  (GPIO_PORT_BASE_ADDR + 0x34)
#define REG_EXT_INT_STATUS (GPIO_PORT_BASE_ADDR + 0x38)
#define REG_EXT_RESET_TOLERANT  (GPIO_PORT_BASE_ADDR + 0x3c)

#define REG_DEBOUNCE1      (GPIO_PORT_BASE_ADDR + 0x40)
#define REG_DEBOUNCE2      (GPIO_PORT_BASE_ADDR + 0x44)

#define REG_EXT_DEBOUNCE1  (GPIO_PORT_BASE_ADDR + 0x48)
#define REG_EXT_DEBOUNCE2  (GPIO_PORT_BASE_ADDR + 0x4c)

#define REG_DEBOUNCE_TIME1 (GPIO_PORT_BASE_ADDR + 0x50)
#define REG_DEBOUNCE_TIME2 (GPIO_PORT_BASE_ADDR + 0x54)
#define REG_DEBOUNCE_TIME3 (GPIO_PORT_BASE_ADDR + 0x58)



/*
 * LPC register address 
 */

#define ASPEED_SCU_VIRTUAL  	IO_ADDRESS(ASPEED_SCU_BASE)

#define SCU_WR_PROT_KEY        	(ASPEED_SCU_VIRTUAL+0x00)
#define SCU_RESET_CTRL         	(ASPEED_SCU_VIRTUAL+0x04)
#define SCU_CLOCK_STOP_CTRL    	(ASPEED_SCU_VIRTUAL+0x0c)
#define SCU_HARDWARE_TRAPPING   (ASPEED_SCU_VIRTUAL+0x70)
#define SCU_MUTI_FUN_PIN_CTRL1  (ASPEED_SCU_VIRTUAL+0x74)
#define SCU_SILICON_REVISION    (ASPEED_SCU_VIRTUAL+0x7C)


#define	IRQ_LPC		    	INT_LPC
#define LPC_BASE_REG        IO_ADDRESS(ASPEED_LPC_BASE)

#define LPC_HICR0_REG       (LPC_BASE_REG + 0x00)
#define LPC_HICR1_REG       (LPC_BASE_REG + 0x04)
#define LPC_HICR2_REG       (LPC_BASE_REG + 0x08)
#define LPC_HICR3_REG       (LPC_BASE_REG + 0x0C)
#define LPC_HICR4_REG       (LPC_BASE_REG + 0x10)
#define LPC_LADR3H_REG      (LPC_BASE_REG + 0x14)
#define LPC_LADR3L_REG      (LPC_BASE_REG + 0x18)
#define LPC_LADR12H_REG     (LPC_BASE_REG + 0x1C)
#define LPC_LADR12L_REG     (LPC_BASE_REG + 0x20)
#define LPC_IDR1_REG        (LPC_BASE_REG + 0x24)
#define LPC_IDR2_REG        (LPC_BASE_REG + 0x28)
#define LPC_IDR3_REG        (LPC_BASE_REG + 0x2C)
#define LPC_ODR1_REG        (LPC_BASE_REG + 0x30)
#define LPC_ODR2_REG        (LPC_BASE_REG + 0x34)
#define LPC_ODR3_REG        (LPC_BASE_REG + 0x38)
#define LPC_STR1_REG        (LPC_BASE_REG + 0x3C)
#define LPC_STR2_REG        (LPC_BASE_REG + 0x40)
#define LPC_STR3_REG        (LPC_BASE_REG + 0x44)

/*
 * Biospost register address 
 */
//#define	IRQ_LPC		    	INT_LPC
//#define LPC_BASE_REG        IO_ADDRESS(ASPEED_LPC_BASE)

//#define LPC_HICR0_REG       (LPC_BASE_REG + 0x00)
//#define LPC_HICR1_REG       (LPC_BASE_REG + 0x04)
//#define LPC_HICR2_REG       (LPC_BASE_REG + 0x08)
//#define LPC_HICR3_REG       (LPC_BASE_REG + 0x0C)
//#define LPC_HICR4_REG       (LPC_BASE_REG + 0x10)
#define LPC_HICR5_REG       (LPC_BASE_REG + 0x80)
#define LPC_HICR6_REG       (LPC_BASE_REG + 0x84)
#define LPC_HICR7_REG       (LPC_BASE_REG + 0x88)
#define LPC_SNPWADR_REG     (LPC_BASE_REG + 0x90)
#define LPC_SNPWDR_REG      (LPC_BASE_REG + 0x94)


/* Fan speed base address */
#define AST2050_FAN_BASE_REG            IO_ADDRESS(ASPEED_PWM_BASE)
#define FAN_REG_GEN_CTRL_ADDR           (AST2050_FAN_BASE_REG + 0x0)
#define FAN_REG_TYPEM_CTRL0_ADDR        (AST2050_FAN_BASE_REG + 0x10)
#define FAN_REG_TYPEM_CTRL1_ADDR        (AST2050_FAN_BASE_REG + 0x14)
#define FAN_REG_TYPEN_CTRL0_ADDR        (AST2050_FAN_BASE_REG + 0x18)
#define FAN_REG_TYPEN_CTRL1_ADDR        (AST2050_FAN_BASE_REG + 0x1c)
#define FAN_REG_TACH_SOURCE_ADDR        (AST2050_FAN_BASE_REG + 0x20)
#define FAN_REG_TRIGGER_ADDR            (AST2050_FAN_BASE_REG + 0x28)
#define FAN_REG_RESULT_ADDR             (AST2050_FAN_BASE_REG + 0x2c)


/* PWM port base address */
#define AST2050_PWM_BASE_REG            IO_ADDRESS(ASPEED_PWM_BASE)
#define SCU_RESET_CONTROL_ADDR          (SCU_BASE_ADDR + 0x4)
#define PWM_REG_GEN_CONTROL_ADDR        (AST2050_PWM_BASE_REG + 0)
#define PWM_REG_CLOCK_CONTROL_ADDR      (AST2050_PWM_BASE_REG + 0x4)
#define PWM_REG_DUTY_CTRL0_ADDR         (AST2050_PWM_BASE_REG + 0x8)
#define PWM_REG_DUTY_CTRL1_ADDR         (AST2050_PWM_BASE_REG + 0xc)
#define PWM_REG_TYPEM_CTRL0_ADDR        (AST2050_PWM_BASE_REG + 0x10)
#define PWM_REG_TYPEM_CTRL1_ADDR        (AST2050_PWM_BASE_REG + 0x14)
#define PWM_REG_TYPEN_CTRL0_ADDR        (AST2050_PWM_BASE_REG + 0x18)
#define PWM_REG_TYPEN_CTRL1_ADDR        (AST2050_PWM_BASE_REG + 0x1c)
#define PWM_REG_TACH_SOURCE_ADDR        (AST2050_PWM_BASE_REG + 0x20)
#define PWM_REG_TRIGGER_ADDR            (AST2050_PWM_BASE_REG + 0x28)
#define PWM_REG_RESULT_ADDR             (AST2050_PWM_BASE_REG + 0x2c)

/* I2C base address */
#define I2C_GLB_REG     (BASE_ADDRESS_REG + 0x000)
#define I2C_DEV1        (BASE_ADDRESS_REG + 0x040)
#define I2C_DEV2        (BASE_ADDRESS_REG + 0x080)
#define I2C_DEV3        (BASE_ADDRESS_REG + 0x0C0)
#define I2C_DEV4        (BASE_ADDRESS_REG + 0x100)
#define I2C_DEV5        (BASE_ADDRESS_REG + 0x140)
#define I2C_DEV6        (BASE_ADDRESS_REG + 0x180)
#define I2C_DEV7        (BASE_ADDRESS_REG + 0x1C0)
#define I2C_BUF_POOL    (BASE_ADDRESS_REG + 0x200)

/* Device Register Definition */
#define I2C0_FUNC_CTRL_REG                  (I2C_DEV1 + 0x00)
#define I2C0_CLK_N_AC_TIMING_CTRL_REG_1     (I2C_DEV1 + 0x04)
#define I2C0_CLK_N_AC_TIMING_CTRL_REG_2     (I2C_DEV1 + 0x08)
#define I2C0_INT_CTRL_REG                   (I2C_DEV1 + 0x0C)
#define I2C0_INT_STATUS_REG                 (I2C_DEV1 + 0x10)
#define I2C0_CMD_STATUS_REG                 (I2C_DEV1 + 0x14)
#define I2C0_DEV_ADDR                       (I2C_DEV1 + 0x18)
#define I2C0_BUF_CTRL_REG                   (I2C_DEV1 + 0x1C)
#define I2C0_TRANS_RECV_BYTE_BUF            (I2C_DEV1 + 0x20)
#define I2C0_DMA_MODE_CTRL_REG              (I2C_DEV1 + 0x24)
#define I2C0_DMA_MODE_STATUS_REG            (I2C_DEV1 + 0x28)

#define I2C1_FUNC_CTRL_REG                  (I2C_DEV2 + 0x00)
#define I2C1_CLK_N_AC_TIMING_CTRL_REG_1     (I2C_DEV2 + 0x04)
#define I2C1_CLK_N_AC_TIMING_CTRL_REG_2     (I2C_DEV2 + 0x08)
#define I2C1_INT_CTRL_REG                   (I2C_DEV2 + 0x0C)
#define I2C1_INT_STATUS_REG                 (I2C_DEV2 + 0x10)
#define I2C1_CMD_STATUS_REG                 (I2C_DEV2 + 0x14)
#define I2C1_DEV_ADDR                       (I2C_DEV2 + 0x18)
#define I2C1_BUF_CTRL_REG                   (I2C_DEV2 + 0x1C)
#define I2C1_TRANS_RECV_BYTE_BUF            (I2C_DEV2 + 0x20)
#define I2C1_DMA_MODE_CTRL_REG              (I2C_DEV2 + 0x24)
#define I2C1_DMA_MODE_STATUS_REG            (I2C_DEV2 + 0x28)

#define I2C2_FUNC_CTRL_REG                  (I2C_DEV3 + 0x00)
#define I2C2_CLK_N_AC_TIMING_CTRL_REG_1     (I2C_DEV3 + 0x04)
#define I2C2_CLK_N_AC_TIMING_CTRL_REG_2     (I2C_DEV3 + 0x08)
#define I2C2_INT_CTRL_REG                   (I2C_DEV3 + 0x0C)
#define I2C2_INT_STATUS_REG                 (I2C_DEV3 + 0x10)
#define I2C2_CMD_STATUS_REG                 (I2C_DEV3 + 0x14)
#define I2C2_DEV_ADDR                       (I2C_DEV3 + 0x18)
#define I2C2_BUF_CTRL_REG                   (I2C_DEV3 + 0x1C)
#define I2C2_TRANS_RECV_BYTE_BUF            (I2C_DEV3 + 0x20)

#define I2C3_FUNC_CTRL_REG                  (I2C_DEV4 + 0x00)
#define I2C3_CLK_N_AC_TIMING_CTRL_REG_1     (I2C_DEV4 + 0x04)
#define I2C3_CLK_N_AC_TIMING_CTRL_REG_2     (I2C_DEV4 + 0x08)
#define I2C3_INT_CTRL_REG                   (I2C_DEV4 + 0x0C)
#define I2C3_INT_STATUS_REG                 (I2C_DEV4 + 0x10)
#define I2C3_CMD_STATUS_REG                 (I2C_DEV4 + 0x14)
#define I2C3_DEV_ADDR                       (I2C_DEV4 + 0x18)
#define I2C3_BUF_CTRL_REG                   (I2C_DEV4 + 0x1C)
#define I2C3_TRANS_RECV_BYTE_BUF            (I2C_DEV4 + 0x20)

#define I2C4_FUNC_CTRL_REG                  (I2C_DEV5 + 0x00)
#define I2C4_CLK_N_AC_TIMING_CTRL_REG_1     (I2C_DEV5 + 0x04)
#define I2C4_CLK_N_AC_TIMING_CTRL_REG_2     (I2C_DEV5 + 0x08)
#define I2C4_INT_CTRL_REG                   (I2C_DEV5 + 0x0C)
#define I2C4_INT_STATUS_REG                 (I2C_DEV5 + 0x10)
#define I2C4_CMD_STATUS_REG                 (I2C_DEV5 + 0x14)
#define I2C4_DEV_ADDR                       (I2C_DEV5 + 0x18)
#define I2C4_BUF_CTRL_REG                   (I2C_DEV5 + 0x1C)
#define I2C4_TRANS_RECV_BYTE_BUF            (I2C_DEV5 + 0x20)

#define I2C5_FUNC_CTRL_REG                  (I2C_DEV6 + 0x00)
#define I2C5_CLK_N_AC_TIMING_CTRL_REG_1     (I2C_DEV6 + 0x04)
#define I2C5_CLK_N_AC_TIMING_CTRL_REG_2     (I2C_DEV6 + 0x08)
#define I2C5_INT_CTRL_REG                   (I2C_DEV6 + 0x0C)
#define I2C5_INT_STATUS_REG                 (I2C_DEV6 + 0x10)
#define I2C5_CMD_STATUS_REG                 (I2C_DEV6 + 0x14)
#define I2C5_DEV_ADDR                       (I2C_DEV6 + 0x18)
#define I2C5_BUF_CTRL_REG                   (I2C_DEV6 + 0x1C)
#define I2C5_TRANS_RECV_BYTE_BUF            (I2C_DEV6 + 0x20)

#define I2C6_FUNC_CTRL_REG                  (I2C_DEV7 + 0x00)
#define I2C6_CLK_N_AC_TIMING_CTRL_REG_1     (I2C_DEV7 + 0x04)
#define I2C6_CLK_N_AC_TIMING_CTRL_REG_2     (I2C_DEV7 + 0x08)
#define I2C6_INT_CTRL_REG                   (I2C_DEV7 + 0x0C)
#define I2C6_INT_STATUS_REG                 (I2C_DEV7 + 0x10)
#define I2C6_CMD_STATUS_REG                 (I2C_DEV7 + 0x14)
#define I2C6_DEV_ADDR                       (I2C_DEV7 + 0x18)
#define I2C6_BUF_CTRL_REG                   (I2C_DEV7 + 0x1C)
#define I2C6_TRANS_RECV_BYTE_BUF            (I2C_DEV7 + 0x20)

/* peci register  definition */
#define PECI_REG_BASE_ADDR      	    IO_ADDRESS(ASPEED_PECI_BASE)

#define PECI_REG_CONTROL                (PECI_REG_BASE_ADDR + 0x0)
#define PECI_REG_TIMING_NEGOTIATION   	(PECI_REG_BASE_ADDR + 0x4)
#define PECI_REG_COMMAND                (PECI_REG_BASE_ADDR + 0x8)
#define PECI_REG_READ_WRITE_LENGTH      (PECI_REG_BASE_ADDR + 0xc)
#define PECI_REG_EXPECTED_FCS         	(PECI_REG_BASE_ADDR + 0x10)
#define PECI_REG_CAPTURED_FCS          	(PECI_REG_BASE_ADDR + 0x14)
#define PECI_REG_INTERRUPT              (PECI_REG_BASE_ADDR + 0x18)
#define PECI_REG_INTERRUPT_STATUS       (PECI_REG_BASE_ADDR + 0x1c)
#define PECI_REG_WRITE_DATA0            (PECI_REG_BASE_ADDR + 0x20)
#define PECI_REG_WRITE_DATA1            (PECI_REG_BASE_ADDR + 0x24)
#define PECI_REG_WRITE_DATA2            (PECI_REG_BASE_ADDR + 0x28)
#define PECI_REG_WRITE_DATA3            (PECI_REG_BASE_ADDR + 0x2C)
#define PECI_REG_READ_DATA0             (PECI_REG_BASE_ADDR + 0x30)
#define PECI_REG_READ_DATA1             (PECI_REG_BASE_ADDR + 0x34)
#define PECI_REG_READ_DATA2             (PECI_REG_BASE_ADDR + 0x38)
#define PECI_REG_READ_DATA3             (PECI_REG_BASE_ADDR + 0x3C)


/*Crypto Registers Definition*/
#define ASPEED_CRYPTOREG(x) (ASPEED_CRYPTO_VA_BASE + (x))
#define ASPEED_CRYPTO_CRY_SDATA_BA    ASPEED_CRYPTOREG(0x00)
#define ASPEED_CRYPTO_CRY_DDATA_BA    ASPEED_CRYPTOREG(0x04)
#define ASPEED_CRYPTO_CRY_CBUF_BA     ASPEED_CRYPTOREG(0x08)
#define ASPEED_CRYPTO_CRY_DATALEN     ASPEED_CRYPTOREG(0x0C)
#define ASPEED_CRYPTO_CRY_CMD         ASPEED_CRYPTOREG(0x10)
#define ASPEED_CRYPTO_ST              ASPEED_CRYPTOREG(0x1C)
#define ASPEED_CRYPTO_HASH_SDATA_BA   ASPEED_CRYPTOREG(0x20)
#define ASPEED_CRYPTO_HASH_WBUF_BA    ASPEED_CRYPTOREG(0x24)
#define ASPEED_CRYPTO_HMACKY_BA       ASPEED_CRYPTOREG(0x28)
#define ASPEED_CRYPTO_HASH_DATALEN    ASPEED_CRYPTOREG(0x2C)
#define ASPEED_CRYPTO_HASH_CMD        ASPEED_CRYPTOREG(0x30)

/*MAC Registers Definition*/
#define MAC_REG_BASE_ADDR1      	    IO_ADDRESS(ASPEED_MAC1_BASE)
#define MAC_REG_BASE_ADDR2      	    IO_ADDRESS(ASPEED_MAC2_BASE)
#define ISR_REG				  0x00				// interrups status register
#define IER_REG				  0x04				// interrupt maks register
#define MAC_MADR_REG		0x08				// MAC address (Most significant)
#define MAC_LADR_REG		0x0c				// MAC address (Least significant)

#define MAHT0_REG			  0x10				// Multicast Address Hash Table 0 register
#define MAHT1_REG			  0x14				// Multicast Address Hash Table 1 register
#define TXPD_REG			  0x18				// Transmit Poll Demand register
#define RXPD_REG			  0x1c				// Receive Poll Demand register
#define TXR_BADR_REG		0x20				// Transmit Ring Base Address register
#define RXR_BADR_REG		0x24				// Receive Ring Base Address register

#define HPTXPD_REG			0x28	//
#define HPTXR_BADR_REG	0x2c	//

#define ITC_REG				  0x30				// interrupt timer control register
#define APTC_REG			  0x34				// Automatic Polling Timer control register
#define DBLAC_REG			  0x38				// DMA Burst Length and Arbitration control register

#define DMAFIFOS_REG		0x3c	//
#define FEAR_REG			  0x44	//
#define TPAFCR_REG			0x48	//
#define RBSR_REG			  0x4c	//for NC Body
#define MACCR_REG			  0x50				// MAC control register
#define MACSR_REG			  0x54				// MAC status register
#define PHYCR_REG			  0x60				// PHY control register
#define PHYDATA_REG			0x64				// PHY Write Data register
#define FCR_REG				  0x68				// Flow Control register
#define BPR_REG				  0x6c				// back pressure register
#define WOLCR_REG			  0x70				// Wake-On-Lan control register
#define WOLSR_REG			  0x74				// Wake-On-Lan status register
#define WFCRC_REG			  0x78				// Wake-up Frame CRC register
#define WFBM1_REG			  0x80				// wake-up frame byte mask 1st double word register
#define WFBM2_REG			  0x84				// wake-up frame byte mask 2nd double word register
#define WFBM3_REG			  0x88				// wake-up frame byte mask 3rd double word register
#define WFBM4_REG			  0x8c				// wake-up frame byte mask 4th double word register

#define NPTXR_PTR_REG		0x90	//
#define HPTXR_PTR_REG		0x94	//
#define RXR_PTR_REG			0x98	//


#endif

