/**
 * $Id$
 *
 *<center>
 *               Avocent Corporation. Proprietary Information.
 * \n<em>
 *      This software is supplied under the terms of a license agreement or
 *      nondisclosure agreement with Avocent Corporation, or  one of its 
 *      affiliates, andaffiliates, and may not be used, disseminated, or 
 *      distributed except in accordance with the terms of that agreement.
 *
 *      3541 Gateway Boulevard Fremont, San Jose, CA 94538 U.S.A.
 *\n
 *                  US phone: 510-771-6100
 *
 *        Copyright &copy; 2001-2009 Avocent Corporation. All rights reserved.
 *</em> </center>
 *----------------------------------------------------------------------------\n
 *  MODULES     Linux device Driver                                           \n
 *----------------------------------------------------------------------------\n
 *  @file   aess_pecisensordrv.h
 *  @brief  peci sensor driver 
 *  @internal
 *  $RCSfile: aess_pecisensordrv.h,v $
 *  $Revision: 1.3 $
 *  $Date: 2008/03/30 20:07:25 $
 *  $Author: Ken Lin $
 *  $Source: $
 *  $Name:  $
 *----------------------------------------------------------------------------*/
#include <asm/ioctl.h>
#if !defined (AESSPECISENSORDRV_H)
#define AESSPECISENSORDRV_H

/* ioctl definitions */
#define AESS_PECIDRV_IOC_MAGIC  0xC3
#define AESS_PECIDRV_R             _IOR(AESS_PECIDRV_IOC_MAGIC, 0, int)
#define AESS_PECIDRV_Q             _IOR(AESS_PECIDRV_IOC_MAGIC, 1, int)
#define AESS_PECIDRV_COMMAND       _IOR(AESS_PECIDRV_IOC_MAGIC, 2, int)

#define PECI_DATA_INOUT_REG_NUMBER  16
/******************************************************************************
*   STRUCT      :   sPECICommandData
******************************************************************************/
/**
 *  @brief   Structure to PECI driver command data parameter.
 *
 *****************************************************************************/
typedef struct
{
	/** Set PECI Client Address */
	UINT8 u8ClientAddress;     

	/** Set PECI Write Length */
	UINT8 u8WriteLength;     

	/** Set PECI Read Length */
	UINT8 u8ReadLength;     
	
	/** Set PECI Command Code */
	UINT8 u8CommandCode;     	

	/** Set PECI Command Data */
	UINT8 u8CommandData[PECI_DATA_INOUT_REG_NUMBER];   
			
} sPECICommandData;

#ifdef AESSPECISENSORDRV_C

/* For temporary use, because no type.h now */
#define STATUS_OK     0

/* init status */
#define NOT_INIT   0
#define INIT_DONE  1

/* SCU related addr, used to enable PECI function */
#define SCU_BASE_ADDR    				IO_ADDRESS(ASPEED_SCU_BASE)
#define SCU_PKR    						(SCU_BASE_ADDR + 0x00)
#define SCU_SYSTEM_RESET_CONTROL    	(SCU_BASE_ADDR + 0x04)
#define SCU_MULTI_FUNTION1_ADDR  		(SCU_BASE_ADDR + 0x74)
#define UN_LOCKKEY                      0x1688A8A8
#define LOCKKEY                         0
#define LOCKED                          0
#define UNLOCKED                        1
#define ENABLE_PECI_DEVICE              0x80
#define RESET_PECI_CONTROLLER           (1 << 10)
#define PECI_READING_VALUE_MASK         0xFFFF
#define SHIFT_FRACTIONAL                6

/* Control register */
#define ENABLE_CLOCK                    (1 << 0)
#define ENABLE_AUTO_AWFCS               (1 << 1)
#define ENABLE_PECI                     (1 << 4)
#define ENABLE_BUS_CONTENTION           (1 << 5)
#define PECI_CLOCK                      8
#define PECI_READ_MODE                  12
#define PECI_READ_SAMPLING              16

/* Control register setting, this part will change by different situation */
#define CLOCK_DIVIDED_VALUE             4  /* from 2^0 to 2^7 */
#define MODE_SELECT                     0  /* 0->PS mode 1->PWC mode */
#define SAMPLING_POINT_SELECT           8  /* from 0 ~ 15 */

/* Timing Control register */
#define TIMING_OF_ADDRESS               0
#define TIMING_OF_MESSAGE               8

/* Timing control register setting, this part will change by different situation */
#define Message_Timing                  0x10  /* from 2^0 to 2^7 */
#define Address_Timing                  0x10  /* from 2^0 to 2^7 */
                                    
/* COMMAND register */
#define FIRE_COMMAND                    (1 << 0)
                                    
/* READ/WRITE length register */
#define COMMAND_TARGET_ADDRESS          0
#define COMMAND_WRITE_LENGTH            8
#define COMMAND_READ_LENGTH             16
#define COMMAND_ENABLE_AWFCS_CYCLE      31
                                    
/* Expected FCS Data Register */   
#define PECI_PROGRAM_AWFCS              24

/* Expected FCS Data Register, this part will change by different situation */
// N/A for now

/* Captured FCS Data Register */   
#define MASK_WRITE_FCS                  0xFF
#define MASK_READ_FCS                   0xFF0000

/* Interrupt register */
#define ENABLE_DONE_INT                 (1 << 0)
#define ENABLE_WRITE_FCS_ABORT_INT      (1 << 1)
#define ENABLE_WRITE_FCS_BAD_INT        (1 << 2)
#define ENABLE_BUS_CONTENT_INT          (1 << 3)
#define PECI_TIMING_SELECTION           30

/* Interrupt register setting, this part will change by different situation */
#define Timing_Negotiation_Selection    0

/* Interrupt status register */
#define INTERRUPT_OCCURE                1
#define MASK_DONE_INT                   (1 << 0)
#define MASK_WRITE_FCS_ABORT_INT        (1 << 1)
#define MASK_WRITE_FCS_BAD_INT          (1 << 2)
#define MASK_BUS_CONTENT_INT            (1 << 3)
#define CLEAR_DONE_INT                  (1 << 0)
#define CLEAR_WRITE_FCS_ABORT_INT       (1 << 1)
#define CLEAR_WRITE_FCS_BAD_INT         (1 << 2)
#define CLEAR_BUS_CONTENT_INT           (1 << 3)
#define CLEAR_ALL_INT                   0xF
#define PECI_TIMING_NEGOTIATION         16

/******************************************************************************
*   STRUCT      :   Function Prototype 
******************************************************************************/
/**
 *  @brief   Prototype for each private function.
 *
 *****************************************************************************/
 
static int aess_pecisensor_command(sPECICommandData *pPECICommandData);
#endif   /* AESSpeciSENSORDRV_C */

#endif   /* AESSpeciSENSORDRV_H */

/* End of code */
