/**
 * \n<em>
 *               Avocent Corporation. Proprietary Information.
 *      This software is supplied under the terms of a license agreement or
 *      nondisclosure agreement with Avocent Corporation, or one of its affiliates, and
 *      may not be used, copied, disseminated, or distributed except in accordance
 *      with the terms of that agreement.
 *
 *      3541 Gateway Boulevard, Fremont, California, 94538 U.S.A.
 *                  US Phone: 510-771-6100
 *      Copyright 2001-2009 Avocent Corporation. All rights reserved.
 *</em> </center>
 *----------------------------------------------------------------------------\n
 *  MODULES     aess pwm driver\n
 *----------------------------------------------------------------------------\n
 * @file    aess_pwmdrv.h 
 * @brief   fantach sensor driver header file
 *----------------------------------------------------------------------------
 */

#if !defined (AESSPWMDRV_H)
#define AESSPWMDRV_H


/******************************************************************************
*   STRUCT      :   sPWMDevConfig
******************************************************************************/
/**
 *  @brief   Structure to PWM driver config.
 *
 *****************************************************************************/
typedef struct
{    
	/** PWM Channel number */
	UINT8 u8PWMChannelNum;
	
	/** PWM Base Cycle Frequency */
	UINT8 u8PWMBaseCycleFrequency;
	
	/** PWM Frequency Divider */
	UINT8 u8PWMFrequencyDivider; 

	/** PWM Duty Cycle */
	UINT8 u8PWMDutyCycle; 
	
} sPWMDevConfig;

/* ioctl definitions */
#define AESS_PWMDRV_IOC_MAGIC           0xBE
#define AESS_PWM_CONFIG_INIT             _IOWR(AESS_PWMDRV_IOC_MAGIC, 0, sPWMDevConfig)
#define AESS_PWM_CONFIG_SET             _IOWR(AESS_PWMDRV_IOC_MAGIC, 1, sPWMDevConfig)
#define AESS_PWM_CONFIG_TEST             _IOWR(AESS_PWMDRV_IOC_MAGIC, 2, sPWMDevConfig)
#define AESS_PWM_CONFIG_INFO             _IOWR(AESS_PWMDRV_IOC_MAGIC, 3, sPWMDevConfig)

/* Define the maximum PWM channel number */
#define PWM_MAX_CHN_NUM  4

#ifdef AESSPWMDRV_C

#include <asm/arch/hardware.h>
#include <linux/spinlock.h>
#include <asm-arm/arch-ast2050/hw_regs.h>

/* TBD: For temporary use, because no type.h now */
#define STATUS_OK     0
#define STATUS_FAIL   1

/* clock souce */
#define CLK_SRC_TYPE_M	0
#define CLK_SRC_TYPE_N	1

/* type slection */
#define PWMA_TYPE	12
#define PWMB_TYPE	13
#define PWMC_TYPE	14
#define PWMD_TYPE	15

/* init flag */
#define AESSPWM_NOT_INIT 0
#define AESSPWM_INIT_OK  1


#define	PWM_TACH_READ_OK_MASK								0x80000000

//system will follow this order to init PWM related register
#define PWM_REG_BASE_FREQUENCY   		0 //base frequency #divisor 1
#define PWM_REG_RESOLUTION   				1 //frequency resolution #divisor 2
#define PWM_REG_DUTY_CYCLE  		 		2 //duty cycle
#define PWM_REG_OTHER0_SET1					3 //it is not normal seeting, depend on chipset, default set to high, set before pwm enable
#define PWM_REG_ENABLE			   			4 //enable setting
#define PWM_REG_OTHER1_SET1					5 //it is not normal seeting, depend on chipset, default set to high, set after pwm enable
#define PWM_REG_ENABLE_PIN					6 //enable pwm pin, shared with GPIO

#define PWM_REG_MAX 		7

/******************************************************************************
*   STRUCT      :   sPWMRegTable
******************************************************************************/
/**
 *  @brief   Structure to PWM driver related data parameter.
 *
 *****************************************************************************/
typedef struct
{
	/** register address */
	UINT32 address;
	
	/** register enabled/disabled */
	UINT8 enable;	
	
	/** mask to access the register */
	UINT8 mask;       
	
	/** number of bits to access the regiester */
	UINT8 bitnumber;        
	
	/** negative logic/psitive logic */
	UINT8 negative;        
} sPWMRegTable;

sPWMRegTable PWMRegMap[PWM_REG_MAX][PWM_MAX_CHN_NUM] = 
{
	// #0 base frequency #divisor 1
	{
		//PWM0~PWM3
		{(UINT32)PWM_REG_CLOCK_CONTROL_ADDR,1,0x4,0,0},
		{(UINT32)PWM_REG_CLOCK_CONTROL_ADDR,1,0x4,0,0},
		{(UINT32)PWM_REG_CLOCK_CONTROL_ADDR,1,0x4,16,0},
		{(UINT32)PWM_REG_CLOCK_CONTROL_ADDR,1,0x4,16,0}
	},
	// #1 frequency resolution #divisor 2
	{
		//PWM0~PWM3
		{(UINT32)0,0,0x1,12,0},
		{(UINT32)0,0,0x1,13,0},
		{(UINT32)0,0,0x1,14,0},
		{(UINT32)0,0,0x1,15,0}
	},
	// #2 duty cycle
	{
		//PWM0~PWM3
		{(UINT32)PWM_REG_DUTY_CTRL0_ADDR,1,0xff,8,0},
		{(UINT32)PWM_REG_DUTY_CTRL0_ADDR,1,0xff,24,0},
		{(UINT32)PWM_REG_DUTY_CTRL1_ADDR,1,0xff,8,0},
		{(UINT32)PWM_REG_DUTY_CTRL1_ADDR,1,0xff,24,0}
	},
	// #3 set before pwm enable
	{
		//PWM0~PWM3
		{(UINT32)0,0,0x1,12,0},
		{(UINT32)0,0,0x1,13,0},
		{(UINT32)0,0,0x1,14,0},
		{(UINT32)0,0,0x1,15,0}
	},
	// #4 enable setting - enable pwm port
	{
		//PWM0~PWM3
		{(UINT32)PWM_REG_GEN_CONTROL_ADDR,1,0x1,8,0},
		{(UINT32)PWM_REG_GEN_CONTROL_ADDR,1,0x1,9,0},
		{(UINT32)PWM_REG_GEN_CONTROL_ADDR,1,0x1,10,0},
		{(UINT32)PWM_REG_GEN_CONTROL_ADDR,1,0x1,11,0}
	},
	// #5 set after pwm enable - enable pwm/fan clock
	{
		//PWM0~PWM3
		{(UINT32)PWM_REG_GEN_CONTROL_ADDR,1,0x1,0,0},
		{(UINT32)PWM_REG_GEN_CONTROL_ADDR,1,0x1,0,0},
		{(UINT32)PWM_REG_GEN_CONTROL_ADDR,1,0x1,0,0},
		{(UINT32)PWM_REG_GEN_CONTROL_ADDR,1,0x1,0,0}
	},
	// #6 enable pwm pin, shared with GPIO
	{
		//PWM0~PWM3
		{(UINT32)SCU_MULTI_FUNTION1_ADDR,1,0x1,8,0},
		{(UINT32)SCU_MULTI_FUNTION1_ADDR,1,0x1,9,0},
		{(UINT32)SCU_MULTI_FUNTION1_ADDR,1,0x1,10,0},
		{(UINT32)SCU_MULTI_FUNTION1_ADDR,1,0x1,11,0}
	}
};


static int aess_pwm_config_init(sPWMDevConfig *PWMDevConfig);
static int aess_pwm_config_set(sPWMDevConfig *PWMDevConfig);

#endif   /* AESSPWMDRV_C */
#endif   /* AESSPWMDRV_H */

/* End of code */
