/*
 * $RCSfile: ast2050_fwupgrade.c,v $
 * $Revision: 1.21 $
 * $Date: 2008/06/13 21:04:00 $
 * $Author: jlee $
 *
 * AST2050 firmware upgrade driver.
 *  
 * Copyright (C) 2008 Avocent Corp.
 *
 * This file is subject to the terms and conditions of the GNU 
 * General Public License. This program is distributed in the hope 
 * that it will be useful, but WITHOUT ANY WARRANTY; without even 
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 */


#include "common.h"
#include "command.h"
#include "config.h"

#include "cdefs.h"
#include "ast2050_fwupgrade.h"
#include "ast2050_ps.h"

#include "image.h"
#include "asm/byteorder.h"

#define IMG_TYPE_AESS  0xAE

#if 1
#define PRINTD(fmt,args...) printf("FWU: " fmt ,##args)
#else
#define PRINTD(fmt,args...)
#endif

#if 0
#define FWU_DEBUG
#endif

extern int flash_write (char *src, ulong addr, ulong cnt);
extern int flash_sect_erase (ulong addr_first, ulong addr_last);
extern int flash_sect_protect (int p, ulong addr_first, ulong addr_last);


/* firmware upgarde information */
fwu_info_type fwu;


extern int do_bootm (cmd_tbl_t *, int, int, char *[]);


int fwu_block_erase(int dev, 
                    unsigned long start,     /* start block or start address */
                    lbaint_t blkcnt)
{
    UINT32 end;
    
    PRINTD("fwu_block_erase\n");
    
    switch (dev)
    {           
        case FWU_DEV_SPI:
            
            end = (start + blkcnt - 1) | 0xFFFF;
            
            /* un-protect sector */
            flash_sect_protect (0, start, end);
            
            /* erase sector */
            flash_sect_erase(start, end);
            
            /* re-protect sector */
            flash_sect_protect (1, start, end);
            
            break;
            
        default:
            break;
    }
    
    return 0;
}


int fwu_block_read(int dev, 
                   unsigned long start,     /* start block or start address */
                   lbaint_t blkcnt,
                   UINT8 *buffer)
{
    PRINTD("fwu_block_read\n");
    
    switch (dev)
    {
        case FWU_DEV_SPI:
            
            /* handle odd address */
            while (start & 0x03)
            {
                PRINTD("start address is odd\n");
                
                *(buffer) = *((UINT8 *)start);
                buffer++;
                start++;
                blkcnt--;
            }
            
            /* move 4 bytes once to increase performance */
            while (blkcnt > 3)
            {
                *((UINT32 *) buffer) = *((UINT32 *)start);
                buffer = buffer + 4;
                start = start + 4;
                blkcnt = blkcnt - 4;
            }
            
            /* handle rest of odd address */
            while (blkcnt & 0x03)
            {
                PRINTD("rest of count is odd\n");
                
                *(buffer) = *((UINT8 *)start);
                buffer++;
                start++;
                blkcnt--;
            }
            
            break;
            
        default:
            break;
    }
    
    return 0;
}


int fwu_block_write(int dev, 
                    unsigned long start,     /* start block or start address */
                    lbaint_t blkcnt,
                    UINT8 *buffer)
{
    UINT32 end;
    
    PRINTD("fwu_block_write\n");
    
    switch (dev)
    {
        case FWU_DEV_SPI:
            
            end = (start + blkcnt - 1) | 0xFFFF;
            
            /* un-protect sector */
            flash_sect_protect (0, start, end);
            
            /* erase sector */
            flash_sect_erase(start, end);
            
            /* write data to SPI flash */
            flash_write(buffer, start, blkcnt);
            
            /* re-protect sector */
            flash_sect_protect (1, start, end);
            
            break;
            
        default:
            break;
    }
    
    return 0;
}


int fwu_dev_init(int dev)
{
    switch (dev)
    {
        case FWU_DEV_SPI:
            break;
            
        default:
            break;
    }
    
    return 0;
}


int fwu_set_dev_type(int dev)
{
    switch (dev)
    {
        case FWU_DEV_SPI:
            setenv("fwudev", "spi");
            break;
            
        default:
            printf("not support fwu device\n");
            break;
    }
    
    return 0;
}


int fwu_get_dev_type(int *dev)
{
    char parameter[32];
    
    sprintf(parameter, "%s", getenv("fwudev"));
    
    PRINTD("fwudev=%s\n", getenv("fwudev"));
    
    if (strcmp(parameter, "spi") == 0)
    {
        *dev = FWU_DEV_SPI;
    }

    PRINTD("fwudev=%s, dev=%d\n", getenv("fwudev"), *dev);
    
    return 0;
}


int fwu_print_dev_list(void)
{
    printf("No.  Device\n");
    printf("---------------------\n");
    printf("%2d   SPI\n", FWU_DEV_SPI);
    
    printf("\nCurrent Device ==> %d\n\n", fwu.dev);
    
    return 0;
}


int fwu_get_image_size(int dev, UINT8 partition, UINT32 *size)
{
    UINT8 buffer[512] __attribute__ ((aligned(32)));
    image_header_t *image;
    fwu_cramfs_super_type *cramfs;
    UINT32 checksum;
    
    PRINTD("fwu_get_image_size\n");
    
    switch (dev)
    {
        case FWU_DEV_SPI:
            
            switch (partition)
            {
                case FWU_SPI_KERNEL:
                    
                    PRINTD("FWU_SPI_KERNEL\n");
                    
                    memcpy((void *) buffer, 
                           (const void *) simple_strtoul(getenv("kernel_start"),
                                                         NULL, 16), 
                           64);
                    
                    break;
                    
                case FWU_SPI_ROOTFS:
                    
                    PRINTD("FWU_SPI_ROOTFS\n");
                    
                    memcpy((void *) buffer, 
                           (const void *) simple_strtoul(getenv("rootfs_start"),
                                                         NULL, 16), 
                           64);
                    
                    break;
                    
                default:
                    
                    *size = 0;
                    return 0;
            }
            
            break;
            
        default:
            break;
    }
    
    image = (image_header_t *) buffer;
    cramfs = (fwu_cramfs_super_type *) buffer;
    
    PRINTD("image=%p, buffer=%p, partition=%d\n", 
           image, buffer, partition);
    
    /* uimage magic number is 0x27051956 */
    if (ntohl(image->ih_magic) == IH_MAGIC)
    {
        PRINTD("uimage magic number is match\n");
        
        checksum = ntohl(image->ih_hcrc);
        image->ih_hcrc = 0;
        
        if (crc32 (0, (UINT8 *) image, sizeof(image_header_t)) != checksum) 
        {
            PRINTD("checksum is %lx not %lx\n", 
                   crc32 (0, (UINT8 *) image, sizeof(image_header_t)), checksum);
            return -1;
        }
        
        /* get the image size from image header */
        *size = ntohl(image->ih_size) + sizeof(image_header_t);
    }
    
    /* cramfs magic number is 0x28cd3d45 */
    else if (cramfs->magic == FWU_CRAMFS_MAGIC)
    {
        PRINTD("cramfs magic number is match\n");
        
        if (strcmp (&cramfs->signature[0], FWU_CRAMFS_SIGNATURE))
        {
            PRINTD("cramfs magic signature is match\n");
            *size = cramfs->size;
        }
        else
        {
            return -1;
        }
    }
    else
    {
        PRINTD("uimage magic number is %lx not 0x27051956\n", image->ih_magic);
        PRINTD("cramfs magic number is %lx not 0x28cd3d45\n", cramfs->magic);
        return -1;
    }
    
    PRINTD("image size=%lx\n", *size);
    
    return 0;
}


static unsigned char fwu_bin2bcd(unsigned int n)
{
    return (((n / 10) << 4) | (n % 10));
}

int fwu_update_env(int dev)
{
    char parameter[128];
    UINT32 size;
    
    switch (dev)
    {
        case FWU_DEV_SPI:
            
            /* setup bootargs from mem and console variables */
            sprintf(parameter, 
                    "root=/dev/ram0 rw initrd=0x41530000,0xE00000 " \
                    "ramdisk_size=15360 " \
                    "mem=%s console=%s", 
                    getenv("mem"),
                    getenv("console"));

            setenv("bootargs", parameter);
            
            /* update fwu environment variables */
            
            /* get kernel start address */
            setenv("fwu_kernel_start", getenv("kernel_start"));
            
            /* update kernel size from the image */
            if (fwu_get_image_size(dev, FWU_SPI_KERNEL, &size))
            {
                PRINTD("default kernel size\n");
                /* setenv("fwu_kernel_size", getenv("kernel_size")); */
                
                /* set kernel size to 0 if the image type can't be recognized */
                setenv("fwu_kernel_size", "0");
            }
            else
            {
                /* move to next boundary and divide by 64K */
                /* size = fwu_cal_sectors(size); */
                
                PRINTD("kernel size=%lx\n", size);
                sprintf(parameter, "%lx", size);
                setenv("fwu_kernel_size", parameter);
            }
            
            /* get rootfs start address */
            setenv("fwu_rootfs_start", getenv("rootfs_start"));
            
            /* update rootfs size from the image */
            if (fwu_get_image_size(dev, FWU_SPI_ROOTFS, &size))
            {
                PRINTD("default rootfs size\n");
                /* setenv("fwu_rootfs_size", getenv("rootfs_size")); */
                
                /* set rootfs size to 0 if the image type can't be recognized */
                setenv("fwu_rootfs_size", "0");
            }
            else
            {
                /* move to next boundary and divide by 64K */
                /* size = fwu_cal_sectors(size); */
                
                PRINTD("rootfs size=%lx\n", size);
                sprintf(parameter, "%lx", size);
                setenv("fwu_rootfs_size", parameter);
            }
            
            break;
            
        default:
            break;
    }
    
    return 0;
}


int fwu_check_image(int dev, UINT8 *buffer)
{
    fwu_image_header_type *image_header;
    fwu_image_info_type *image_info;
    ps_info_type *ps;
    char parameter[32];
    UINT32 size;
    
    PRINTD("fwu_check_image\n");
    
    image_header = (fwu_image_header_type *) buffer;
    
    /* shift to image information */
    image_info 
    = (fwu_image_info_type *) (buffer + sizeof(fwu_image_header_type));
    
    PRINTD("buffer=%p, image_header=%p, image_info=%p\n",
           buffer, image_header, image_info);
    
    PRINTD("crc32=%lx, header_ver=%x, num_of_image=%x, version=%lx, img_size=%lx\n", 
           image_header->crc32,
           image_header->header_ver, 
           image_header->num_of_image, 
           image_header->version, 
           image_header->img_size);
    
    if(image_header->header_ver != 1)
    {
        printf("*** unknown image version ***\n");
        
        return -1;
    }
    
    if(image_header->img_type != IMG_TYPE_AESS)
    {
        printf("*** wrong image type ***\n");
        
        return -1;
    }
    
    printf("Checking image header CRC ... ");
    
    /* check crc32 for the header, header size is fixed to 512 bytes */
    if (crc32 (0, (buffer + 4), 508) != image_header->crc32)
    {
        PRINTD("buffer+4=%p\n", buffer + 4);
        PRINTD("crc32=%lx\n", (crc32 (0, (buffer + 4), 508)));
        
        printf("Bad CRC\n");
        
        return -1;
    }
    
    printf("OK\n");
    
    printf("Checking platform ID ........ ");
    
    /* get the start address of persistent storage */
    ps = (ps_info_type *) CFG_PS_ADDR;
    
    /* check platform ID */
    if ((*((UINT32 *) &ps->platform_id[0])) != 0xFFFFFFFF)
    {
        if (((*((UINT32 *) &image_header->platform_id[0])) != 0x00000000)
            && ((*((UINT32 *) &image_header->platform_id[0])) != 0xFFFFFFFF))
        {
            if ((strncmp((char *) &image_header->platform_id[0], 
                         CONFIG_AST2050_PLATFORM_ID, 
                         4)) 
                && (sizeof(CONFIG_AST2050_PLATFORM_ID) > 1))
            {
                printf("Fail  (%s != %c%c%c%c)\n", 
                        CONFIG_AST2050_PLATFORM_ID, 
                        (char) image_header->platform_id[0],
                        (char) image_header->platform_id[1],
                        (char) image_header->platform_id[2],
                        (char) image_header->platform_id[3]);
                
                return -1;
            }
        }
    }
    
    printf("OK\n");
    
    /* reset kernel/rootfs/uboot size variable */
    setenv("fwu_kernel_size", "0");
    setenv("fwu_rootfs_size", "0");
    setenv("fwu_uboot_size", "0");
    
    /* retrieve kernel image information, the 1st image is always kernel */
    if (image_header->num_of_image >= 1)
    {
        printf("Checking kernel image CRC ... ");
        
        PRINTD("offset=%lx, size=%lx, crc32=%lx\n", 
               image_info->offset,
               image_info->size, 
               image_info->crc32);
        
        /* check crc32 for the kernel image */
        if (image_info->crc32 
            != crc32 (0, (buffer + image_info->offset), image_info->size))
        {
            PRINTD("crc32=%lx\n", 
                   crc32 (0, (buffer + image_info->offset), image_info->size));
            
            printf("Bad CRC\n");
            
            /* setenv("fwu_kernel_size", "0"); */
            
            return -1;
        }
        
        printf("OK\n");
        
        /* update kernel load address */
        sprintf(parameter, "%lx", ((UINT32) buffer + image_info->offset));
        setenv("fwu_kernel_offset", parameter);
        
        /* get image size from image information structure */
        size = image_info->size;
        
        switch (dev)
        {
            case FWU_DEV_SPI:
                
                /* change size to sector unit */
                /* size = fwu_cal_sectors(size); */
                
                break;
                
            default:
                break;
        }
        
        PRINTD("size=%lx\n", size);
        
        sprintf(parameter, "%lx", size);
        
        setenv("fwu_kernel_size", parameter);
        
        /* retrieve rootfs image information, the 2nd image is always rootfs */
        if (image_header->num_of_image >= 2)
        {
            printf("Checking rootfs image CRC ... ");
            
            /* move to the next image information */
            image_info++;
            
            PRINTD("offset=%lx, size=%lx, crc32=%lx\n", 
                   image_info->offset,
                   image_info->size, 
                   image_info->crc32);
            
            /* check crc32 for the rootfs image */
            if (image_info->crc32 
                != crc32 (0, (buffer + image_info->offset), image_info->size))
            {
                PRINTD("crc32=%lx\n", 
                       crc32 (0, 
                              (buffer + image_info->offset), 
                              image_info->size));
                
                printf("Bad CRC\n");
                
                /* setenv("fwu_rootfs_size", "0"); */
                
                return -1;
            }
            
            printf("OK\n");
            
            /* update rootfs load address */
            sprintf(parameter, "%lx", ((UINT32) buffer + image_info->offset));
            setenv("fwu_rootfs_offset", parameter);
            
            /* get image size from image information structure */
            size = image_info->size;
            
            switch (dev)
            {
                case FWU_DEV_SPI:
                    
                    /* change size to sector unit */
                    /* size = fwu_cal_sectors(size); */
                    
                    break;
                    
                default:
                    break;
            }
            
            PRINTD("size=%lx\n", size);
            
            sprintf(parameter, "%lx", size);
            
            setenv("fwu_rootfs_size", parameter);
        }
        
        /* retrieve u-boot image information, the 3rd image is always u-boot */
        if (image_header->num_of_image >= 3)
        {
            printf("Checking u-boot image CRC ... ");
            
            /* move to the next image information */
            image_info++;
            
            PRINTD("offset=%lx, size=%lx, crc32=%lx\n", 
                   image_info->offset,
                   image_info->size, 
                   image_info->crc32);
            
            /* check crc32 for the u-boot image */
            if (image_info->crc32 
                != crc32 (0, (buffer + image_info->offset), image_info->size))
            {
                PRINTD("crc32=%lx\n", 
                       crc32 (0, 
                              (buffer + image_info->offset), 
                              image_info->size));
                
                printf("Bad CRC\n");
                
                /* setenv("fwu_uboot_size", "0"); */
                
                return -1;
            }
            
            printf("OK\n");
            
            printf("Skipping u-boot update ...... ");
            
            /* check U-Boot version */
            if ((image_header->uboot_ver[0] 
                 == fwu_bin2bcd(simple_strtoul(CONFIG_UBOOT_VERSION, 
                                               NULL, 
                                               10)))
                && (image_header->uboot_ver[1] 
                    == fwu_bin2bcd(simple_strtoul(CONFIG_UBOOT_PATCHLEVEL, 
                                                  NULL, 
                                                  10)))
                && (image_header->uboot_ver[2] 
                    == fwu_bin2bcd(simple_strtoul(CONFIG_UBOOT_SUBLEVEL, 
                                                  NULL, 
                                                  10)))
                && (image_header->uboot_ver[4] 
                    == fwu_bin2bcd(simple_strtoul(CONFIG_AVCT_VERSION, 
                                                  NULL, 
                                                  10)))
                && (image_header->uboot_ver[5] 
                    == fwu_bin2bcd(simple_strtoul(CONFIG_AVCT_PATCHLEVEL, 
                                                  NULL, 
                                                  10)))
                && (image_header->uboot_ver[6] 
                    == fwu_bin2bcd(simple_strtoul(CONFIG_AVCT_SUBLEVEL, 
                                                  NULL, 
                                                  10))))
            {
                printf("YES\n");
                
                return 0;
            }
            
            printf("NO\n");
            
            /* update u-boot load address */
            sprintf(parameter, "%lx", ((UINT32) buffer + image_info->offset));
            setenv("fwu_uboot_offset", parameter);
            
            /* get image size from image information structure */
            size = image_info->size;
            
            PRINTD("size=%lx\n", size);
            
            sprintf(parameter, "%lx", size);
            
            setenv("fwu_uboot_size", parameter);
        }
    }
    
    return 0;
}


int fwu_update_image(int dev, UINT8 *buffer)
{
    int ret;
    UINT32 size;
    UINT32 offset;
    UINT32 start;
    
    PRINTD("fwu_update_image\n");
    
    /* check image first */
    ret = fwu_check_image(dev, buffer);
    
    if (ret)
    {
        return ret;
    }
    
    /* update kernel image */
    size = simple_strtoul(getenv("fwu_kernel_size"), NULL, 16);
    
    PRINTD("kernel size=%lx\n", size);
    
    if (size)
    {
        printf("Copying kernel image ........ ");
        
        offset = simple_strtoul(getenv("fwu_kernel_offset"), NULL, 16);
        start = simple_strtoul(getenv("fwu_kernel_start"), NULL, 16);
        
        PRINTD("kernel offset=%lx, start=%lx, size=%lx\n", offset, start, size);
        
        ret = fwu_block_write(dev, 
                              start,
                              size,
                              (UINT8 *) offset);
        
        if (ret) 
        {
            printf("Fail\n");
            printf("*** fail to update kernel image ***\n");
            return ret;
        }
        
        printf("OK\n");
    }
    
    /* update rootfs image */
    size = simple_strtoul(getenv("fwu_rootfs_size"), NULL, 16);
    
    PRINTD("rootfs size=%lx\n", size);
    
    if (size)
    {
        printf("Copying rootfs .............. ");
        
        offset = simple_strtoul(getenv("fwu_rootfs_offset"), NULL, 16);
        start = simple_strtoul(getenv("fwu_rootfs_start"), NULL, 16);
        
        PRINTD("rootfs offset=%lx, start=%lx, size=%lx\n", offset, start, size);
        
        ret = fwu_block_write(dev, 
                              start,
                              size,
                              (UINT8 *) offset);
        
        if (ret) 
        {
            printf("Fail\n");
            printf("*** fail to update rootfs image ***\n");
            return ret;
        }
        
        printf("OK\n");
    }
    
    /* update u-boot image */
    size = simple_strtoul(getenv("fwu_uboot_size"), NULL, 16);
    
    PRINTD("uboot size=%lx\n", size);
    
    if (size)
    {
        printf("Copying u-boot .............. ");
        
        offset = simple_strtoul(getenv("fwu_uboot_offset"), NULL, 16);
        start = simple_strtoul(getenv("uboot_start"), NULL, 16);
        
        PRINTD("rootfs offset=%lx, start=%lx, size=%lx\n", offset, start, size);
        
        ret = fwu_block_write(FWU_DEV_SPI, 
                              start,
                              size,
                              (UINT8 *) offset);
        
        if (ret) 
        {
            printf("Fail\n");
            printf("*** fail to update u-boot image ***\n");
            return ret;
        }
        
        printf("OK\n");
    }
    
    /* update environment */
    fwu_update_env(dev);
    
    /* saveenv(); */
    
    return 0;
}


int fwu_boot_image(int dev)
{
    UINT32 load;
    UINT32 start;
    UINT32 size;
    char *bootm_args[3];
    char arg0[16] = "bootm";
    char arg1[16] = "";
    char arg2[16] = "";
    int ret;
    
    /* prepare parameter array */
    bootm_args[0] = &arg0[0];
    bootm_args[1] = &arg1[0];
    bootm_args[2] = &arg2[0];
    
    /* update environment */
    fwu_update_env(dev);
    
    load = simple_strtoul(getenv("fwu_kernel_load"), NULL, 16);
    start = simple_strtoul(getenv("fwu_kernel_start"), NULL, 16);
    size = simple_strtoul(getenv("fwu_kernel_size"), NULL, 16);
    
    PRINTD("KERNEL: load=%lx, start=%lx, size=%lx\n", load, start, size);
    
    /* copy kernel image from flash to ram */
    if ((load != 0) && (size != 0))
    {
        printf("Copying kernel image ... ");
        
        if (fwu_block_read(dev, start, size, (UINT8 *) load))
        {
            printf("Fail\n");
            printf("*** can't read from device %d ***\n", dev);
            return -1;
        }
        
        /* give kernel image address to bootm */
        sprintf(bootm_args[1], "%lx", load);
        
        PRINTD("bootm_args[1]=%s, load=%lx\n", bootm_args[1], load);
        
        printf("KERNEL copy to ram OK\n");
        
        load = simple_strtoul(getenv("fwu_rootfs_load"), NULL, 16);
        start = simple_strtoul(getenv("fwu_rootfs_start"), NULL, 16);
        size = simple_strtoul(getenv("fwu_rootfs_size"), NULL, 16);
        
        PRINTD("ROOTFS: load=%lx, start=%lx, size=%lx\n", load, start, size);
        
        /* copy rootfs image from flash to ram */
        if ((load != 0) && (size != 0))
        {
            printf("Copying rootfs image ... ");
            
            if (fwu_block_read(dev, start, size, (UINT8 *) load))
            {
                printf("Fail\n");
                printf("*** can't read from device %d ***\n", dev);
                return -1;
            }
            
            /* give rootfs image address to bootm */
            sprintf(bootm_args[2], "%lx", load);
            
            PRINTD("bootm_args[2]=%s, load=%lx\n", bootm_args[2], load);
            
            printf("ROOTFS copy to ram OK\n");
        }
        else
        {
            PRINTD("rootfs is not uimage\n");
            
            if (dev == FWU_DEV_SPI)
            {
                printf("Copying rootfs image ... ");
                
                load = simple_strtoul(getenv("rootfs_load"), NULL, 16);
                
                if (fwu_block_read(dev, start, size, (UINT8 *) load))
                {
                    printf("Fail\n");
                    printf("*** can't read from device %d ***\n", dev);
                    return -1;
                }
                
                printf("OK\n");
                
                /* bootm does not need to handle rootfs */
                load = 0;
            }
            else
            {
                /* printf("Skip rootfs image\n"); */
            }
        }
    }
    
    PRINTD("bootm_args=%p, bootm_args[1]=%s, bootm_args[2]=%s\n", 
           bootm_args, bootm_args[1], bootm_args[2]);
    
    /* check if the image size is valid */
    if (size)
    {
        /* start to boot from the kernel image */
        if (load)
        {
            ret = do_bootm(NULL, 0, 3, bootm_args);
        }
        else
        {
            ret = do_bootm(NULL, 0, 2, bootm_args);
        }
    }
    else
    {
        printf("*** image is not valid ***\n");
        ret = -1;
    }
    
    return 0;
}


/* init the firmware upgrade sub-function */
int fwu_init(void)
{
    int ret;
    
    PRINTD("fwu_init\n");
    
    /* reset device init status */
    fwu.init = 0;
    
    /* get device type */
    if (fwu_get_dev_type(&fwu.dev))
    {
        return -1;
    }
    
    PRINTD("fwu.dev=%d\n", fwu.dev);
    
    /* initiate device */
    if (fwu_dev_init(fwu.dev))
    {
        return -1;
    }
    
    /* update environment */
    fwu_update_env(fwu.dev);
    
    /* change device init status */
    fwu.init = 1;
    
    return 0;   
}


int do_fwu(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    int ret;
    UINT32 offset;
    UINT32 bytes;
    UINT8 type;
    char parameter[32];
    
    switch (argc) 
    {
        case 0:
        case 1:
            break;
            
        case 2:
        case 3:
        case 4:
        case 5:
        case 6:
            if (strcmp (argv[1], "init") == 0) 
            {
                ret = fwu_init();
                
                if (ret) 
                {
                    printf("*** fail to initiate firmware upgrade function ***\n");
                    return ret;
                }
                
                printf("Done!\n");
                
                return 0;
            }
            
            else if (strcmp (argv[1], "dev") == 0) 
            {
                if (argc < 3)
                {
                    fwu_print_dev_list();
                    
                    return 0;
                }
                
                type = (UINT8) simple_strtoul(argv[2], NULL, 16);
                
                if (type <= FWU_DEV_LAST)
                {
                    /* update environment variable */
                    fwu_set_dev_type(type);
                }
                else
                {
                    printf("*** unsupported device type ***\n");
                    return -1;
                }
                
                /* re-init device */
                ret = fwu_init();
                
                if (ret) 
                {
                    printf("*** fail to set device ***\n");
                    return ret;
                }
                
                printf("\n");
                
                fwu_print_dev_list();
                
                return 0;
            }
            
            else if (strcmp (argv[1], "check") == 0) 
            {
                /* retrieve start block variable */
                if (argc == 3)
                {
                    offset = simple_strtoul(argv[2], NULL, 16);
                }
                else
                {
                    /* get default offset from 'offset' variable */
                    sprintf(parameter, "%s", getenv("offset"));
                    
                    if (parameter != NULL) 
                    {
                        offset = simple_strtoul (parameter, NULL, 16);
                    }
                    else 
                    {
                        offset = CFG_LOAD_ADDR;
                    }
                }
                
                if (offset & 0x1FF)
                {
                    printf("Memory address is not at block boundary!\n");
                    return -1;
                }
                
                ret = fwu_check_image(fwu.dev, (UINT8 *) offset);
                
                if (ret) 
                {
                    printf("*** fail to check image ***\n");
                    return ret;
                }
                
                printf("Done!\n");
                
                return 0;
            }
            
            else if (strcmp (argv[1], "update") == 0) 
            {
                /* check device init status */
                if (fwu.init == 0)
                {
                    printf("*** device is not ready ***\n");
                    return 0;
                }
                
                /* retrieve start block variable */
                if (argc == 3)
                {
                    offset = simple_strtoul(argv[2], NULL, 16);
                }
                else
                {
                    /* get default offset from 'offset' variable */
                    sprintf(parameter, "%s", getenv("offset"));
                    
                    if (parameter != NULL) 
                    {
                        offset = simple_strtoul (parameter, NULL, 16);
                    }
                    else 
                    {
                        offset = CFG_LOAD_ADDR;
                    }
                }
                
                if (offset & 0x1FF)
                {
                    printf("Memory address is not at block boundary!\n");
                    return -1;
                }
                
                ret = fwu_update_image(fwu.dev, (UINT8 *) offset);
                
                if (ret) 
                {
                    printf("*** fail to update image ***\n");
                    return ret;
                }
                
                printf("Done!\n");
                
                return 0;
            }
            
            else if (strcmp (argv[1], "boot") == 0) 
            {
                /* check device init status */
                if (fwu.init == 0)
                {
                    printf("*** device is not ready ***\n");
                    return 0;
                }
                
                ret = fwu_boot_image(fwu.dev);
                
                if (ret) 
                {
                    printf("*** fail to boot image ***\n");
                    return ret;
                }
                
                printf("Done!\n");
                
                return 0;
            }
            
            else if (strcmp (argv[1], "cal") == 0) 
            {
                if (argc < 3)
                {
                    break;
                }
                
                bytes = simple_strtoul(argv[2], NULL, 16);
                
                printf("0x%lx bytes are equal to 0x%lx blocks = 0x%lx blocks + 0x%lx bytes\n\n", 
                       bytes,
                       (bytes & 0x1FF) ? (bytes >> 9) + 1 : bytes >> 9 ,
                       bytes >> 9,
                       bytes & 0x1FF);
                
                return 0;
            }
            
            break;
            
        default:
            break;
    }
    
    /* the command is not correct */
    printf("Usage:\n%s\n", cmdtp->usage);
    
    return 1;
}


U_BOOT_CMD(
    fwu, 6, 1, do_fwu,
    "fwu     - firmware upgrade sub-system, type 'help fwu' for details\n",
    "\nfwu dev [no]\n"
    "           - list available devices or set device 'no' \n"
    "fwu init   - parse MBR/EBR to retrieve partition and image information\n"
    "fwu check [addr]\n"
    "           - check image at memory address 'addr'\n"
    "fwu update [addr]\n"
    "           - update flash by the images at memory address 'addr'\n"
    "fwu cal size\n"
    "           - calculate how many blocks are equal to 'size' bytes\n"
    "fwu boot   - load image(s) and then boot\n"
);
