/*
 * Board initialize for ASPEED AST2050
 *
 *
 */

#include <common.h>
#include <command.h>
#include <pci.h>

extern int fwu_init(void);

int board_init (void)
{
    DECLARE_GLOBAL_DATA_PTR;
    unsigned long reg;

    /* AHB Controller */
    *((volatile ulong*) 0x1E600000)  = 0xAEED1A03;	/* unlock AHB controller */ 
    *((volatile ulong*) 0x1E60008C) |= 0x01;		/* map DRAM to 0x00000000 */

    /* Flash Controller */
#ifdef CONFIG_2SPIFLASH
    *((volatile ulong*) 0x16000000) |= 0x00001c20;	/* enable Flash Write */
#else
    *((volatile ulong*) 0x16000000) |= 0x00001c00;	/* enable Flash Write */
#endif    

    /* SCU */
    *((volatile ulong*) 0x1e6e2000) = 0x1688A8A8;	/* unlock SCU */
    reg = *((volatile ulong*) 0x1e6e2008);		/* LHCLK = HPLL/8 */
    reg &= 0x1c0fffff;                                  /* PCLK  = HPLL/8 */
    reg |= 0x61b00000;					/* BHCLK = HPLL/8 */
    *((volatile ulong*) 0x1e6e2008) = reg;     
    reg = *((volatile ulong*) 0x1e6e200c);		/* enable 2D Clk */
    *((volatile ulong*) 0x1e6e200c) &= 0xFFFFFFFD; 
                                                                          
    /* arch number */
    gd->bd->bi_arch_number = MACH_TYPE_AST2050;
                                                                                                                             
    /* adress of boot parameters */
    gd->bd->bi_boot_params = 0x40000100;
                                                                                                                             
    return 0;
}

int dram_init (void)
{
    DECLARE_GLOBAL_DATA_PTR;
                                                                                                                             
    gd->bd->bi_dram[0].start = PHYS_SDRAM_1;

#ifndef CONFIG_EMS_1_0_RAM_USE
	gd->bd->bi_dram[0].size = PHYS_SDRAM_1_SIZE; 
#else
	gd->bd->bi_dram[0].size = CONFIG_EMS_1_0_RAM_USE; 
#endif
                                                                                                                 
    return 0;
}

int checkboard(void)
{
//    char *s = getenv("serial#");
    
    puts("Board: AST2050");
    
//    /* attach platform ID */
//    if (platform_id != PF_ID_UNKNOWN)
//    {
//        printf("_%02x", platform_id);
//    }
//    
//    if (s != NULL) 
//    {
//        puts(", serial# ");
//        puts(s);
//    }
    
    putc('\n');
    
    return (0);
}

int misc_init_r(void)
{
    unsigned char reg1, reg2;

    /* Show H/W Version */
    reg1 = (unsigned char) (*((ulong*) 0x1e6e207c));
    reg2 = (unsigned char) (*((ulong*) 0x1e6e2070) >> 24);
//    puts ("H/W:   ");
//#ifdef CONFIG_AST1100_FPGA       
//    printf("FPGA130 Rev. %02x, FPGA70 Rev. %02x \n", reg1, reg2);
//#else
//    printf("Rev. %02x \n", reg1);    
//#endif

    /* detect/initiate firmware upgrade module */
    fwu_init();

    return 0;
}

int print_image_size(char* prefix, ulong size)
{
    if (size < (1024))
    {
        /* (size) */
        printf("\r%s %4ld B ", prefix, size);
    }
    else if (size < (1024 * 1024))
    {
        /* (size / 1024) */
        printf("\r%s %4ld KB", prefix, (size >> 10));
    }
    else if (size < (1024 * 1024 * 1024))
    {
        /* (size / 1024 / 1024) */
        printf("\r%s %4ld MB", prefix, (size >> 10 >> 10));
    }
    else
    {
        /* (size / 1024 / 1024 / 1024) */
        printf("\r%s %4ld GB", prefix, (size >> 10 >> 10 >> 10));
    }
    
    return 0;
}
