/*
 * $RCSfile: ast2050_ps.h,v $
 * $Revision: 1.3 $
 * $Date: 2008/05/21 20:34:54 $
 * $Author: jlee $
 *
 * AST2050 persistent storage driver.
 *  
 * Copyright (C) 2008 Avocent Corp.
 *
 * This file is subject to the terms and conditions of the GNU 
 * General Public License. This program is distributed in the hope 
 * that it will be useful, but WITHOUT ANY WARRANTY; without even 
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 */

#ifndef AST2050_PS_H
#define AST2050_PS_H

#include "cdefs.h"

#define PS_REC_MAC0             0
#define PS_REC_MAC1             1
#define PS_REC_VERSION          2
#define PS_REC_PLATFORM         3
#define PS_REC_MAC_ACT          3
#define PS_REC_MAX              3

#define PS_ERASE_ENV            0
#define PS_ERASE_REC            1

#define PS_CREATE_NEW           0
#define PS_CREATE_UPDATE        1

#define PS_PLATFORM_UNKNOWN     0xFFFFFFFF

#define PS_CURRENT_VERSION      3

#define MAXIMUM_MAC_INTF    8
#define MAC_INTF_NONE       0x00
#define MAC_INTF_RMII_PHY   0x01
#define MAC_INTF_NCSI       0x02
#define MAC_INTF_MII_PHY    0x10
#define MAC_INTF_GMII_PHY   0x20
#define MAC_INTF_IS_VAILD   (MAC_INTF_RMII_PHY | MAC_INTF_NCSI | MAC_INTF_MII_PHY |MAC_INTF_GMII_PHY)
#define MAC_INTF_IS_INVAILD   0xFF

#define MAC0    0
#define MAC1    1

#define MAC_ACTIVATE    1
#define MAC_DEACTIVATE  0

#define MAC_FROM_EEPROM  1
#define MAC_FROM_SPI     0

#define MAC0_OFFSET_IN_EEPROM  0
#define MAC1_OFFSET_IN_EEPROM  6

#pragma pack(1)

/* structure of persistent storage */
typedef struct ps_info
{
    UINT8 magic_number[4];      /* 0x1B8, 0x41 0x56 0x43 0x54 (AVCT) */
    UINT8 ps_ver;               /* version, 0x01 */
    UINT8 reserved[3];          /* 0x00, 0x00 0x00 */
    
    /* 0-5: bytes is mac address; 
     *   6: MAC INTF (0: MAC_INTF_NONE, 1: MAC_INTF_5221_PHY, 2:MAC_INTF_INTEL_NCSI)
     *   7: MAC ACT BIT: (0: MAC_DEACTIVATE; 1: MAC_ACTIVATE)
     */  
    UINT8 mac0[8];               
    UINT8 mac1[8];              
    UINT8 uboot_ver[8];         /* 0-2: major 4-6: minor, in BCD */
    UINT8 platform_id[4];       /* platfrom id, 0xFFFFFFFF -> unspecified */
} ps_info_type;

#pragma pack()

int ps_check(void);

#endif
