/*
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>

#if (CONFIG_COMMANDS & CFG_CMD_NET) && defined(CONFIG_NET_MULTI) \
	&& defined(CONFIG_FARADAYNIC)

#include <malloc.h>
#include <net.h>
#include <pci.h>
#include "../board/ast2050/ast2050_ps.h"

#include "aess_ncsi.h"

/* if S_ncsi_init = 0, the uboot need to receive the NCSI response */
static int S_ncsi_init = 1;

/* due to NCSI cannot send packet in the first init, 
   so drop the NCSI init when broad first on */
static int S_first_enter_init = 1;

#define pci_find_devices         NULL
#define pci_read_config_dword    NULL
#define SCU_BASE                 0x1E6E2000
#define SCU_RESET_CONTROL        0x04
#define SCU_HARDWARE_TRAPPING    0x70
#define SCU_PIN_MUX              0x74
#define MAC_INTERFACE            0x1C0
#define RMII                     0x100
#define RMII_AND_RMII            0x180
#define GMII                     0x0
#define MII                      0x40
#define MAC2_MDC_MDIO            (1 << 20)
#define MAC2_MII                 (1 << 21)
#define MAC1_PHY_LINK            (1 << 25)
#define MAC2_PHY_LINK            (1 << 26)

unsigned int faradaynic_iobase[2] = {0x1E660000, 0x1E680000};

#undef DEBUG_SROM
#undef DEBUG_SROM2

#undef UPDATE_SROM

/* PCI Registers.
 */
#define PCI_CFDA_PSM		0x43

#define CFRV_RN		0x000000f0	/* Revision Number */

#define WAKEUP		0x00		/* Power Saving Wakeup */
#define SLEEP		0x80		/* Power Saving Sleep Mode */

#define DC2114x_BRK	0x0020		/* CFRV break between DC21142 & DC21143 */

/* MAC chip register */
#define ISR_REG				0x00				// interrups status register
#define IER_REG				0x04				// interrupt maks register
#define MAC_MADR_REG		        0x08				// MAC address (Most significant)
#define MAC_LADR_REG		        0x0c				// MAC address (Least significant)

#define MAHT0_REG			0x10				// Multicast Address Hash Table 0 register
#define MAHT1_REG			0x14				// Multicast Address Hash Table 1 register
#define TXPD_REG			0x18				// Transmit Poll Demand register
#define RXPD_REG			0x1c				// Receive Poll Demand register
#define TXR_BADR_REG		        0x20				// Transmit Ring Base Address register
#define RXR_BADR_REG		        0x24				// Receive Ring Base Address register

#define HPTXPD_REG			0x28	//
#define HPTXR_BADR_REG		        0x2c	//

#define ITC_REG				0x30				// interrupt timer control register
#define APTC_REG			0x34				// Automatic Polling Timer control register
#define DBLAC_REG			0x38				// DMA Burst Length and Arbitration control register

#define DMAFIFOS_REG		        0x3c	//
#define FEAR_REG			0x44	//
#define TPAFCR_REG			0x48	//
#define RBSR_REG			0x4c	//for NC Body
#define MACCR_REG			0x50				// MAC control register
#define MACSR_REG			0x54				// MAC status register
#define PHYCR_REG			0x60				// PHY control register
#define PHYDATA_REG			0x64				// PHY Write Data register
#define FCR_REG				0x68				// Flow Control register
#define BPR_REG				0x6c				// back pressure register
#define WOLCR_REG			0x70				// Wake-On-Lan control register
#define WOLSR_REG			0x74				// Wake-On-Lan status register
#define WFCRC_REG			0x78				// Wake-up Frame CRC register
#define WFBM1_REG			0x80				// wake-up frame byte mask 1st double word register
#define WFBM2_REG			0x84				// wake-up frame byte mask 2nd double word register
#define WFBM3_REG			0x88				// wake-up frame byte mask 3rd double word register
#define WFBM4_REG			0x8c				// wake-up frame byte mask 4th double word register


// --------------------------------------------------------------------
//		MACCR_REG
// --------------------------------------------------------------------

#define SW_RST_bit			(1UL<<31)				// software reset/
#define DIRPATH_bit			(1UL<<21)	
#define RX_IPCS_FAIL_bit	(1UL<<20)	//
#define RX_TCPCS_FAIL_bit	(1UL<<19)	//
#define SPEED_100M_MODE_bit     (1UL<<19)               
#define RX_UDPCS_FAIL_bit	(1UL<<18)	//
#define RX_BROADPKT_bit		(1UL<<17)				// Receiving broadcast packet
#define RX_MULTIPKT_bit		(1UL<<16)				// receiving multicast packet
#define RX_HT_EN_bit		(1UL<<15)
#define RX_ALLADR_bit		(1UL<<14)				// not check incoming packet's destination address
#define JUMBO_LF_bit		(1UL<<13)	//
#define RX_RUNT_bit			(1UL<<12)				// Store incoming packet even its length is les than 64 byte
#define CRC_CHK_bit			(1UL<<11)	//
#define CRC_APD_bit			(1UL<<10)				// append crc to transmit packet
#define GMAC_MODE_bit		(1UL<<9)	//
#define FULLDUP_bit			(1UL<<8)				// full duplex
#define ENRX_IN_HALFTX_bit	(1UL<<7)	//
#define LOOP_EN_bit			(1UL<<6)				// Internal loop-back
#define HPTXR_EN_bit		(1UL<<5)	//
#define REMOVE_VLAN_bit		(1UL<<4)	//
#define RXMAC_EN_bit		(1UL<<3)				// receiver enable
#define TXMAC_EN_bit		(1UL<<2)				// transmitter enable
#define RXDMA_EN_bit		(1UL<<1)				// enable DMA receiving channel
#define TXDMA_EN_bit		(1UL<<0)				// enable DMA transmitting channel

//---------------------------------------------------
//  PHY R/W Register Bit
//---------------------------------------------------
#define MIIWR                   (1UL<<27)
#define MIIRD                   (1UL<<26)
#define MDC_CYCTHR              0x34
#define PHY_SPEED_MASK          0xC000
#define PHY_DUPLEX_MASK         0x2000
#define SPEED_1000M             0x02
#define SPEED_100M              0x01
#define SPEED_10M               0x00
#define DUPLEX_FULL             0x01
#define DUPLEX_HALF             0x00
#define RESOLVED_BIT            0x800

#define PHY_SPEED_DUPLEX_MASK   0x01E0
#define PHY_100M_DUPLEX         0x0100
#define PHY_100M_HALF           0x0080
#define PHY_10M_DUPLEX          0x0040
#define PHY_10M_HALF            0x0020



/* Descriptor bits.
 */
#define TXDMA_OWN	0x80000000	/* Own Bit */
#define RXPKT_RDY       0x00000000      
#define RXPKT_STATUS    0x80000000
//#define EDORR		0x00008000	/* Receive End Of Ring */
#define EDORR		0x40000000	/* Receive End Of Ring */
#define LRS		0x10000000	/* Last Descriptor */
#define RD_ES		0x00008000	/* Error Summary */
//#define EDOTR		0x00008000	/* Transmit End Of Ring */
#define EDOTR		0x40000000	/* Transmit End Of Ring */
#define T_OWN		0x80000000	/* Own Bit */
#define LTS		0x10000000	/* Last Segment */
#define FTS		0x20000000	/* First Segment */
#define CRC_ERR         0x00080000
#define TD_ES		0x00008000	/* Error Summary */
#define TD_SET		0x08000000	/* Setup Packet */
#define RX_ERR          0x00040000
#define FTL             0x00100000
#define RUNT            0x00200000
#define RX_ODD_NB       0x00400000

#define POLL_DEMAND	1
#define RESET_DE4X5(dev) {\
    int i;\
    i=INL(dev, MACCR_REG);\
    udelay(1000);\
    OUTL(dev, i | SW_RST_bit, MACCR_REG);\
    for (; (INL(dev, MACCR_REG ) & SW_RST_bit) != 0; ) {udelay(1000);}\
    OUTL(dev, 0, IER_REG );	\
}

#define START_MAC(dev) {\
    s32 omr; \
    omr = INL(dev, MACCR_REG);\
    omr |= RXMAC_EN_bit | TXMAC_EN_bit | RXDMA_EN_bit | TXDMA_EN_bit;\
    OUTL(dev, omr, MACCR_REG);		/* Enable the TX and/or RX */\
}

#define STOP_MAC(dev) {\
    s32 omr; \
    omr = INL(dev, MACCR_REG);\
    omr &= ~(RXMAC_EN_bit | TXMAC_EN_bit | RXDMA_EN_bit | TXDMA_EN_bit);\
    OUTL(dev, omr, MACCR_REG);		/* Disable the TX and/or RX */ \
}

#define NUM_RX_DESC PKTBUFSRX
#define NUM_TX_DESC 1			/* Number of TX descriptors   */
#define RX_BUFF_SZ  PKTSIZE_ALIGN

#define TOUT_LOOP   1000000
#define PHY_LOOP    250
#define ETH_ALEN	6

struct de4x5_desc {
	volatile s32 status;
	u32 des1;
	u32 reserved;
	u32 buf;	
};

static struct de4x5_desc rx_ring[2][NUM_RX_DESC] __attribute__ ((aligned(32))); /* RX descriptor ring         */
static struct de4x5_desc tx_ring[2][NUM_TX_DESC] __attribute__ ((aligned(32))); /* TX descriptor ring         */
static int rx_new[2];                             /* RX descriptor ring pointer */
static int tx_new[2];                             /* TX descriptor ring pointer */

static char rxRingSize[2];
static char txRingSize[2];


static unsigned short ftgmac100_read_phy_register(unsigned int ioaddr, unsigned char phyaddr, unsigned char phyreg);
static void ftgmac100_write_phy_register(unsigned int ioaddr, unsigned char phyaddr, unsigned char phyreg, unsigned short phydata);

static void  send_setup_frame(struct eth_device* dev, bd_t * bis);

static int   faradaynic_init(struct eth_device* dev, bd_t* bis);
static int   faradaynic_send(struct eth_device* dev, volatile void *packet, int length);
static int   faradaynic_recv(struct eth_device* dev);
static void  faradaynic_halt(struct eth_device* dev);
static void  set_mac_address (struct eth_device* dev, bd_t* bis);
static void  phy_write_register (struct eth_device* dev, u8 PHY_Register, u8 PHY_Address, u16 PHY_Data);
static u16   phy_read_register (struct eth_device* dev, u8 PHY_Register, u8 PHY_Address);
static void  set_mac_control_register(struct eth_device* dev);

#if defined(CONFIG_E500)
#define phys_to_bus(a) (a)
#else
#define phys_to_bus(a)	pci_phys_to_mem((pci_dev_t)dev->priv, a)
#endif

static int INL(struct eth_device* dev, u_long addr)
{
	return le32_to_cpu(*(volatile u_long *)(addr + dev->iobase));
}

static void OUTL(struct eth_device* dev, int command, u_long addr)
{
	*(volatile u_long *)(addr + dev->iobase) = cpu_to_le32(command);
}

typedef struct 
{
    UINT8 mac_num;
    UINT8 mac_intf;
    UINT8 mac_act;
    UINT8 reserved;
} faradaynic_priv;


struct eth_device	faradaynic_device[2];
static faradaynic_priv eth_priv[2];

int faradaynic_initialize(bd_t *bis)
{
    unsigned int		iobase, SCURegister;
    struct eth_device*  dev;
    char parameter[32];
    UINT8 i, mac_act = 0;

    /* unlock SCU registers */
    *(volatile ulong *)(SCU_BASE) = 0x1688A8A8;

//MAC1 RESET/PHY_LINK in SCU
        SCURegister = le32_to_cpu(*(volatile u_long *)(SCU_BASE + SCU_RESET_CONTROL));
	*(volatile u_long *)(SCU_BASE + SCU_RESET_CONTROL) = cpu_to_le32(SCURegister & ~(0x800));
#ifdef CONFIG_MAC1_PHY_LINK_INTERRUPT
        SCURegister = le32_to_cpu(*(volatile u_long *)(SCU_BASE + SCU_PIN_MUX));
	*(volatile u_long *)(SCU_BASE + SCU_PIN_MUX) = cpu_to_le32(SCURegister | (MAC1_PHY_LINK));
#endif

//MAC2
#ifdef CONFIG_MAC2_ENABLE
        SCURegister = le32_to_cpu(*(volatile u_long *)(SCU_BASE + SCU_RESET_CONTROL));
	*(volatile u_long *)(SCU_BASE + SCU_RESET_CONTROL) = cpu_to_le32(SCURegister & ~(0x1000));
        SCURegister = le32_to_cpu(*(volatile u_long *)(SCU_BASE + SCU_PIN_MUX));
	*(volatile u_long *)(SCU_BASE + SCU_PIN_MUX) = cpu_to_le32(SCURegister & ~(MAC2_MDC_MDIO));
#endif
#ifdef CONFIG_MAC2_MII_ENABLE
        SCURegister = le32_to_cpu(*(volatile u_long *)(SCU_BASE + SCU_PIN_MUX));
	*(volatile u_long *)(SCU_BASE + SCU_PIN_MUX) = cpu_to_le32(SCURegister | (MAC2_MII));
#endif
#ifdef CONFIG_MAC2_PHY_LINK_INTERRUPT
        SCURegister = le32_to_cpu(*(volatile u_long *)(SCU_BASE + SCU_PIN_MUX));
	*(volatile u_long *)(SCU_BASE + SCU_PIN_MUX) = cpu_to_le32(SCURegister | (MAC2_PHY_LINK));
#endif

    /* lock SCU registers */
    *(volatile ulong *)(SCU_BASE) = 0;

    sprintf(parameter, "%s", getenv("macact"));
    mac_act = simple_strtoul(parameter, NULL, 16);

    /* check the activate state of MAC */
    for (i=0; i<2; i++)
    {
        eth_priv[i].mac_num = i;
        
        if (MAC1 == i)    
        {
            sprintf(parameter, "%s", getenv("mac1intf"));
            eth_priv[i].mac_intf = simple_strtoul(parameter, NULL, 16);
        }
        else
        {
            sprintf(parameter, "%s", getenv("mac0intf"));
            eth_priv[i].mac_intf = simple_strtoul(parameter, NULL, 16);
        }
        
        //printf("faradaynic_initialize mac_num=%d, mac_intf=%d\n", eth_priv[i].mac_num, eth_priv[i].mac_intf);

        iobase = faradaynic_iobase[i];

        dev = &faradaynic_device[i];
    	
        sprintf(dev->name, "faradaynic#%d", eth_priv[i].mac_num);
        dev->iobase = iobase;

        dev->init = faradaynic_init;
        dev->halt = faradaynic_halt;
        dev->send = faradaynic_send;
        dev->recv = faradaynic_recv;
        dev->priv = &eth_priv[i];

        /* Ensure we're not sleeping. */
        udelay(10 * 1000);

        dev->init(dev, bis);
        
        eth_register(dev);

    }

    if (MAC1 == mac_act)    
    {
        eth_priv[0].mac_act= MAC_DEACTIVATE;
        eth_priv[1].mac_act = MAC_ACTIVATE;
    }
    else
    {
        eth_priv[0].mac_act = MAC_ACTIVATE;
        eth_priv[1].mac_act = MAC_DEACTIVATE;
    }
    
    return mac_act;
    
}

static int faradaynic_init(struct eth_device* dev, bd_t* bis)
{
    unsigned long	i;
    faradaynic_priv *priv = dev->priv;

    //printf("faradaynic_init MAC0: 0x%lx:0x%lx:0x%lx:0x%lx:0x%lx:0x%lx\n", bis->bi_enetaddr[0],bis->bi_enetaddr[1], bis->bi_enetaddr[2], bis->bi_enetaddr[3], bis->bi_enetaddr[4], bis->bi_enetaddr[5] );
    //printf("faradaynic_init MAC1: 0x%lx:0x%lx:0x%lx:0x%lx:0x%lx:0x%lx\n", bis->bi_enet1addr[0],bis->bi_enet1addr[1], bis->bi_enet1addr[2], bis->bi_enet1addr[3], bis->bi_enet1addr[4], bis->bi_enet1addr[5] );
    
    RESET_DE4X5(dev);

    /* MAC address is controlled by ast2050_ps.c */
    set_mac_address (dev, bis);
    set_mac_control_register (dev);
    
    for (i = 0; i < NUM_RX_DESC; i++) {
    	rx_ring[priv->mac_num][i].status = cpu_to_le32(RXPKT_RDY + RX_BUFF_SZ);
    	rx_ring[priv->mac_num][i].buf = cpu_to_le32((u32) NetRxPackets[i]);
    	rx_ring[priv->mac_num][i].reserved = 0;
    }

    for (i=0; i < NUM_TX_DESC; i++) {
    	tx_ring[priv->mac_num][i].status = 0;
    	tx_ring[priv->mac_num][i].des1 = 0;
    	tx_ring[priv->mac_num][i].buf = 0;
    	tx_ring[priv->mac_num][i].reserved = 0;
    }

    rxRingSize[priv->mac_num] = NUM_RX_DESC;
    txRingSize[priv->mac_num] = NUM_TX_DESC;

    rx_ring[priv->mac_num][rxRingSize[priv->mac_num] - 1].status |= cpu_to_le32(EDORR);
    tx_ring[priv->mac_num][txRingSize[priv->mac_num] - 1].status |= cpu_to_le32(EDOTR);

    OUTL(dev, ((u32) &tx_ring[priv->mac_num]), TXR_BADR_REG);
    OUTL(dev, ((u32) &rx_ring[priv->mac_num]), RXR_BADR_REG);
    START_MAC(dev);
    tx_new[priv->mac_num] = 0;
    rx_new[priv->mac_num] = 0;

    if (priv->mac_intf & MAC_INTF_NCSI)
    {
        /* The MAC interface is NCSI, begin to init NCSI */ 

        /* due to NCSI cannot send packet in the first init, 
           so drop the NCSI init when broad first on */
        if (S_first_enter_init != 1)
        {
            int err = 0;
            
            S_ncsi_init = 0;
            
            /* NC-SI init */
            ncsi_init();
        
            /* force to open pkg 0/chnl 0 now.  */  
        
            if ((err = ncsi_open(CHNL_ID(0, 0), dev->enetaddr)) != 0)
            {
                printf("ncsi_open fail err code = %x\n", err);
            }
        
            S_ncsi_init = 1;
        }    
        
        S_first_enter_init = 0;
    }
    
    return 1;
}

static void dump_msg(const char *label, const char *buf, unsigned int length)
{
	unsigned int	start, num, i;
	char		line[52], *p;

	if (length >= 512)
		return;
	printf("%s, length %u:\n", label, length);

	start = 0;
	while (length > 0) {
		num = min(length, 16u);
		p = line;
		for (i = 0; i < num; ++i) {
			if (i == 8)
				*p++ = ' ';
			sprintf(p, " %02x", buf[i]);
			p += 3;
		}
		*p = 0;
		printf("%6x: %s\n", start, line);
		buf += num;
		start += num;
		length -= num;
	}
}


static int faradaynic_send(struct eth_device* dev, volatile void *packet, int length)
{
	int		status = -1, oldlength = 0, fail = 0;
	int		i;
    faradaynic_priv *priv = dev->priv;

    if (priv->mac_intf & MAC_INTF_NCSI)
    {
        //printf("faradaynic_send: mac_num=%d, mac_intf=%d\n", priv->mac_num, priv->mac_intf);
        //dump_msg("faradaynic_send", packet, length);
    }

	if (length <= 0) {
		printf("%s: bad packet size: %d\n", dev->name, length);
		goto Done;
	}


	for(i = 0; (tx_ring[priv->mac_num][tx_new[priv->mac_num]].status & cpu_to_le32(TXDMA_OWN)) == 0x80000000; i++) {
		if (i >= TOUT_LOOP) {
			printf("%s: tx error buffer not ready\n", dev->name);
                        fail = 1;
			goto Done;
		}
	}
        
    oldlength = length;

    if (length < 60) {
        memset ((void *)cpu_to_le32((u32) (packet + length)), 0, 60 - length);
        length = 60;
    }
    
	tx_ring[priv->mac_num][tx_new[priv->mac_num]].buf    = cpu_to_le32(((u32) packet));
	tx_ring[priv->mac_num][tx_new[priv->mac_num]].status   &= (~(0x3FFF));
	tx_ring[priv->mac_num][tx_new[priv->mac_num]].status   |= cpu_to_le32(LTS | FTS | length);
	tx_ring[priv->mac_num][tx_new[priv->mac_num]].status |= cpu_to_le32(TXDMA_OWN);
        
	OUTL(dev, POLL_DEMAND, TXPD_REG);

	for (i = 0; (tx_ring[priv->mac_num][tx_new[priv->mac_num]].status & cpu_to_le32(TXDMA_OWN)) == 0x80000000; i++) 
	{
		if (i >= TOUT_LOOP) 
		{
			printf(".%s: tx buffer not ready\n", dev->name);
                        fail = 1;
			goto Done;
		}
	}

	if (le32_to_cpu(tx_ring[priv->mac_num][tx_new[priv->mac_num]].status) & CRC_ERR) {
		tx_ring[priv->mac_num][tx_new[priv->mac_num]].status = 0x0;
                fail = 1;
		goto Done;
	}
        
        if (fail != 1) {
            status = oldlength;
        }

 Done:
    tx_new[priv->mac_num] = (tx_new[priv->mac_num]+1) % NUM_TX_DESC;

	return status;
}

static int faradaynic_recv(struct eth_device* dev)
{
    s32		status;
    int		length    = 0;
    faradaynic_priv *priv = dev->priv;

	for ( ; ; ) 
	{
		status = (s32)le32_to_cpu(rx_ring[priv->mac_num][rx_new[priv->mac_num]].status);

		if ((status & RXPKT_STATUS) == 0) {
			break;
		}

		if (status & LRS) {
			/* Valid frame status.
			 */
			if (status & (RX_ERR | CRC_ERR | FTL | RUNT | RX_ODD_NB)) {

				/* There was an error.
				 */
				printf("RX error status = 0x%08X\n", status);
			} else
			{
				/* A valid frame received.
				 */
				length = (le32_to_cpu(rx_ring[priv->mac_num][rx_new[priv->mac_num]].status) & 0x3FFF);
				/* Pass the packet up to the protocol
				 * layers.
				 */

                        if (priv->mac_intf & MAC_INTF_NCSI)
                        {
                            if (S_ncsi_init == 1)
                            {
                                //printf("receive the normal packet\n");
                                NetReceive((uchar *) NetRxPackets[rx_new[priv->mac_num]], length - 4);
                            }
                            else
                            {
                                //printf("receive the NCSI response\n");
                                ncsi_receive((uchar *) NetRxPackets[rx_new[priv->mac_num]], length - 4);
                            }
                        }
                        else
                        {
                            NetReceive(NetRxPackets[rx_new[priv->mac_num]], length - 4);
                        }
			}

			/* Change buffer ownership for this frame, back
			 * to the adapter.
			 */
			rx_ring[priv->mac_num][rx_new[priv->mac_num]].status &= cpu_to_le32(0x7FFFFFFF);
//			rx_ring[rx_new].status = cpu_to_le32(RXPKT_RDY);
		}

		/* Update entry information.
		 */
		rx_new[priv->mac_num] = (rx_new[priv->mac_num] + 1) % rxRingSize[priv->mac_num];
	}

	return length;
}

static void faradaynic_halt(struct eth_device* dev)
{
    struct eth_device *ethdev;
    UINT8 i;

    /* Don't care the input parameter, halt the ALL MAC for avoid kernel panic issue */
    
    /* In current implementation, we will initial share nic and dedicated nic 
       after uboot initialized. But, we only disable the "Current used NIC"
       during u-boot to be boot into the kernel. So, it is possible to receive
       the packet from the other standby NIC, and then have the chance to occur
       the kernel panic due to invalid DMA address of NIC. */
    for (i = 0; i < 2; i++)
    {
        ethdev = &faradaynic_device[i];
        STOP_MAC(ethdev);
    }
}

static void set_mac_address (struct eth_device* dev, bd_t* bis)
{
    ps_info_type *ps;
    faradaynic_priv *priv = dev->priv;

    /* get the start address of persistent storage */
    ps = (ps_info_type *) CFG_PS_ADDR;

    if (MAC1 == priv->mac_num)
    {
        memcpy(dev->enetaddr, ps->mac1, 6);
            
        /* get mac address from ps */
        OUTL(dev, ((ps->mac1[2] << 24) | (ps->mac1[3] << 16) | (ps->mac1[4] << 8) | ps->mac1[5]), MAC_LADR_REG);
        OUTL(dev, ((ps->mac1[0] << 8) | ps->mac1[1]), MAC_MADR_REG);

        //printf("set_mac1_address: 0x%lx:0x%lx:0x%lx:0x%lx:0x%lx:0x%lx\n", ps->mac1[0], ps->mac1[1], ps->mac1[2], ps->mac1[3], ps->mac1[4], ps->mac1[5]);
    }
    else
    {
        memcpy(dev->enetaddr, ps->mac0, 6);
            
        /* get mac address from ps */
        OUTL(dev, ((ps->mac0[2] << 24) | (ps->mac0[3] << 16) | (ps->mac0[4] << 8) | ps->mac0[5]), MAC_LADR_REG);
        OUTL(dev, ((ps->mac0[0] << 8) | ps->mac0[1]), MAC_MADR_REG);
        //printf("set_mac0_address: 0x%lx:0x%lx:0x%lx:0x%lx:0x%lx:0x%lx\n", ps->mac0[0], ps->mac0[1], ps->mac0[2], ps->mac0[3], ps->mac0[4], ps->mac0[5]);
    }
    
}


static u16 phy_read_register (struct eth_device* dev, u8 PHY_Register, u8 PHY_Address)
{
       u32 Data, Status = 0, Loop_Count = 0, PHY_Ready = 1;
       u16 Return_Data;

//20us * 100 = 2ms > (1 / 2.5Mhz) * 0x34
       OUTL(dev, (PHY_Register << 21) + (PHY_Address << 16) + MIIRD + MDC_CYCTHR, PHYCR_REG);
       do {
            udelay(20);
           Status = (INL (dev, PHYCR_REG) & MIIRD);
            Loop_Count++;
            if (Loop_Count >= 100) {
                PHY_Ready = 0;
                break;
            }
       } while (Status == MIIRD);
       
       if (PHY_Ready == 0) {
           printf ("PHY NOT REDAY ");
           return 0;
       }
       Data = INL (dev, PHYDATA_REG);
       Return_Data = (Data >> 16);
      
       return Return_Data;
}


static void phy_write_register (struct eth_device* dev, u8 PHY_Register, u8 PHY_Address, u16 PHY_Data)
{
       u32 Status = 0, Loop_Count = 0, PHY_Ready = 1;

       OUTL(dev, PHY_Data, PHYDATA_REG);
       OUTL(dev, (PHY_Register << 21) + (PHY_Address << 16) + MIIWR + MDC_CYCTHR, PHYCR_REG);
       do {
            udelay(20);
           Status = (INL (dev, PHYCR_REG) & MIIWR);
            Loop_Count++;
            if (Loop_Count >= 100) {
                PHY_Ready = 0;
                break;
            }
       } while (Status == MIIWR);
       if (PHY_Ready == 0) {
           printf ("PHY NOT REDAY ");
       }
}

static void set_mac_control_register (struct eth_device* dev)
{
    unsigned long	MAC_CR_Register = 0, Loop_Count = 0, PHY_Ready = 1, SCU_Register;
    u16    PHY_Status, PHY_Speed, PHY_Duplex, Resolved_Status = 0, Chip_ID, Advertise, Link_Partner;
    
    faradaynic_priv *priv = dev->priv;

    MAC_CR_Register = SPEED_100M_MODE_bit | RX_BROADPKT_bit | FULLDUP_bit | RXMAC_EN_bit | RXDMA_EN_bit | TXMAC_EN_bit | TXDMA_EN_bit | CRC_APD_bit;

    /* unlock SCU registers */
    *(volatile ulong *)(SCU_BASE) = 0x1688A8A8;

    /* Set SCU hardware trap to RMII  (also for kernel use) */
    SCU_Register = le32_to_cpu(*(volatile u_long *)(SCU_BASE + SCU_HARDWARE_TRAPPING));

    if (priv->mac_intf & (MAC_INTF_RMII_PHY | MAC_INTF_NCSI))
    {
        //printf("set SCU to support two RMII interface\n");
        
        /* set SCU to support two RMII interface */
        *(volatile u_long *)(SCU_BASE + SCU_HARDWARE_TRAPPING) = cpu_to_le32((SCU_Register & ~(MAC_INTERFACE)) | RMII_AND_RMII);
    }
    else if (priv->mac_intf & (MAC_INTF_MII_PHY))
    {
        //printf("set SCU to support MII interface\n");

        /* set SCU to support one MII interface */
        *(volatile u_long *)(SCU_BASE + SCU_HARDWARE_TRAPPING) = cpu_to_le32((SCU_Register & ~(MAC_INTERFACE)) | MII);
    }
    else if (priv->mac_intf & (MAC_INTF_GMII_PHY))
    {
        //printf("set SCU to support GMII interface\n");

        /* set SCU to support one MII interface */
        *(volatile u_long *)(SCU_BASE + SCU_HARDWARE_TRAPPING) = cpu_to_le32((SCU_Register & ~(MAC_INTERFACE)) | GMII);
    }
    else
    {
        //printf("DEFAULT: set SCU to support two RMII interface\n");
        
        /* default to set SCU to support two RMII interface */
        *(volatile u_long *)(SCU_BASE + SCU_HARDWARE_TRAPPING) = cpu_to_le32((SCU_Register & ~(MAC_INTERFACE)) | RMII_AND_RMII);
    }
    
    /* lock SCU registers */
    *(volatile ulong *)(SCU_BASE) = 0;
    
    //printf("SCU_Register=0x%lx, MAC_CR_Register=0x%lx\n", SCU_Register, MAC_CR_Register);
    
    if (priv->mac_intf & (MAC_INTF_RMII_PHY | MAC_INTF_MII_PHY | MAC_INTF_GMII_PHY))
    {
        //printf("INIT PHY=0x%lx\n", priv->mac_intf);
        
        /* The MAC interface is PHY, begin to init PHY */ 
        Chip_ID = phy_read_register (dev, 0x02, 0);
        
        if (Chip_ID == 0x40)
        {
            Advertise = phy_read_register (dev, 0x04, 0);
            Link_Partner = phy_read_register (dev, 0x05, 0);
            Advertise = (Advertise & PHY_SPEED_DUPLEX_MASK);
            Link_Partner = (Link_Partner & PHY_SPEED_DUPLEX_MASK);
            if ((Advertise & Link_Partner) & PHY_100M_DUPLEX)
            {
                MAC_CR_Register |= SPEED_100M_MODE_bit;
                MAC_CR_Register |= FULLDUP_bit;
            }
            else if ((Advertise & Link_Partner) & PHY_100M_HALF)
            {
                MAC_CR_Register |= SPEED_100M_MODE_bit;
                MAC_CR_Register &= ~FULLDUP_bit;
            }
            else if ((Advertise & Link_Partner) & PHY_10M_DUPLEX)
            {
                MAC_CR_Register &= ~SPEED_100M_MODE_bit;
                MAC_CR_Register |= FULLDUP_bit;
            }
            else if ((Advertise & Link_Partner) & PHY_10M_HALF)
            {
                MAC_CR_Register &= ~SPEED_100M_MODE_bit;
                MAC_CR_Register &= ~FULLDUP_bit;
            }
        }
        else if (Chip_ID == 0x141) 
        {
            //Max waiting time = (20 + 2)ms * 250(PHY_LOOP) = 5.5s
            do 
            {
		    	udelay (20000);
                Resolved_Status = (phy_read_register (dev, 0x11, 0) & RESOLVED_BIT);
                Loop_Count++;
                if (Loop_Count >= PHY_LOOP)
                {
                    PHY_Ready = 0;
                    printf ("PHY NOT READY ");
                    break;
                }
            } while (Resolved_Status != RESOLVED_BIT);

            if (PHY_Ready == 1)
            {
                PHY_Status = phy_read_register (dev, 0x11, 0);
                PHY_Speed = (PHY_Status & PHY_SPEED_MASK) >> 14;
                PHY_Duplex = (PHY_Status & PHY_DUPLEX_MASK) >> 13;

                if (PHY_Speed == SPEED_1000M)
                {
                    MAC_CR_Register |= GMAC_MODE_bit;
                    //SCU to GMII
                    //                SCU_Register = le32_to_cpu(*(volatile u_long *)(SCU_BASE + SCU_HARDWARE_TRAPPING));
                    //	        *(volatile u_long *)(SCU_BASE + SCU_HARDWARE_TRAPPING) = cpu_to_le32((SCU_Register & ~(MAC_INTERFACE)) | GMII);
                    //                OUTL(dev, MAC_CR_Register, MACCR_REG);
                }
                else
                {
                    MAC_CR_Register &= ~GMAC_MODE_bit;
                    //SCU to MII
                    //                SCU_Register = le32_to_cpu(*(volatile u_long *)(SCU_BASE + SCU_HARDWARE_TRAPPING));
                    //	        *(volatile u_long *)(SCU_BASE + SCU_HARDWARE_TRAPPING) = cpu_to_le32((SCU_Register & ~(MAC_INTERFACE)) | MII);
                    if (PHY_Speed == SPEED_10M)
                    {
                        MAC_CR_Register &= ~SPEED_100M_MODE_bit;
                    }
                }

                if (PHY_Duplex == DUPLEX_HALF)
                {
                    MAC_CR_Register &= ~FULLDUP_bit;
                }
            }
        }
    }
    OUTL(dev, MAC_CR_Register, MACCR_REG);
}

#endif	/* CFG_CMD_NET && CONFIG_NET_MULTI && CONFIG_FARADAYMAC */
