# Start the ast2050 usb drivers
# installs udc driver (peripheral controller driver)
# installs composite driver 
# installs kbdmouse driver
# installs mass_storage driver
# Note: for a configuration that is not full features (ipmi only)
# the kbdmouse driver is not loaded
# Startup Dependency: 

RETVAL=0

MODULEDIR="/lib/modules"
UDC="g_ast2050_udc"
COMP="g_composite"
KBD="g_kbdmouse"
MS="g_mass_storage"
DEVDIR="/dev/avct"
FULLFEATURES="true"

#
# full_features () - check for full features configuration
# Note - there is no good way to do this.  A non full features
# configuration (ipmi only) will not include JAVA clients
#
# updates global variable FULLFEATUES
#
full_features() {
	if [ -f "/usr/local/www/software/avctKVM.jar" ] ; then
		return 1;
	fi;

	echo "NOTICE: Not full features configuration - kbdmouse not operational"
	FULLFEATURES="false"
	return 0;
}

#
# driver_loaded () - checks for driver being loaded
# $1 = driver name
#
# returns 0 if loaded, !0 if not loaded
#
driver_loaded() {
	lsmod | grep $1 > /dev/null
	return $?
} 

#
# driver_major () - returns major number assigned to driver
# $1 = driver name
#
# returns major number, 0 if not found in /proc/devices
#
driver_major() {
	major=`cat /proc/devices | grep $1 | cut -f1 -d" "`
	if [ major = "" ] ; then
		major=0
	fi;
	return $major
} 

#
# driver_mknod () - do character device mknod
# $1 = major number
# $2 = minor number
# $3 = device pathname
#
# returns 0 if no errors
#
driver_mknod() {
	mknod $3 c $1 $2 > /dev/null
	if [ $? -ne 0 ] ; then
		echo "failed mknod of $3"
		return 1
	fi;
	return 0
} 

start() {

	full_features

	# peripheral controller driver
	insmod $MODULEDIR/$UDC.ko
	driver_loaded $UDC
	if [ $? -ne 0 ] ; then
		echo "failed load of $UDC"
		return 1
	fi;

	# composite driver
	if [ $FULLFEATURES = "true" ] ; then
		insmod $MODULEDIR/$COMP.ko functions=2
	else
		insmod $MODULEDIR/$COMP.ko functions=1
	fi;

	driver_loaded $COMP
	if [ $? -ne 0 ] ; then
		echo "failed load of $COMP"
		return 1
	fi;

	# kbdmouse driver
	if [ $FULLFEATURES = "true" ] ; then
		insmod $MODULEDIR/$KBD.ko 
		driver_loaded $KBD
		if [ $? -ne 0 ] ; then
			echo "failed load of $KBD"
			return 1
		fi;

		driver_major $KBD
		kbd_major=$?
		if [ $kbd_major -eq 0 ]; then
			return 1
		fi;
	fi;

	# mass_storage driver
	insmod $MODULEDIR/$MS.ko interfaces=2 vendor="MP EMS"
	driver_loaded $MS
	if [ $? -ne 0 ] ; then
		echo "failed load of $MS"
		return 1
	fi;

	driver_major $MS
	ms_major=$?
	if [ $ms_major -eq 0 ]; then
		return 1
	fi;

	if [ ! -e $DEVDIR ]; then
        	mkdir $DEVDIR
	fi;

	if [ $FULLFEATURES = "true" ] ; then
		driver_mknod $kbd_major 0 "$DEVDIR/usb_keyboard"
		driver_mknod $kbd_major 1 "$DEVDIR/usb_mouse"
		driver_mknod $kbd_major 2 "$DEVDIR/usb_mouse_rel"
	fi;

	driver_mknod $ms_major 0 "$DEVDIR/usb_iface1"
	driver_mknod $ms_major 1 "$DEVDIR/usb_iface2"
#	driver_mknod $ms_major 2 "$DEVDIR/usb_iface3"

	return 0
}

stop() {
	rmmod $MS
	rmmod $KBD
	rmmod $COMP
	rmmod $UDC
	rm -f "$DEVDIR/usb_keyboard"
	rm -f "$DEVDIR/usb_mouse"
	rm -f "$DEVDIR/usb_mouse_rel"
	rm -f "$DEVDIR/usb_iface1"
#	rm -f "$DEVDIR/usb_iface2"
#	rm -f "$DEVDIR/usb_iface3"
	return 0
}

status() {
	echo -n "$UDC driver "
	driver_loaded $UDC
	if [ $? -eq 0 ];
	then
		echo "loaded "
	else
		echo "NOT loaded "
	fi;

	echo -n "$COMP driver "
	driver_loaded $COMP
	if [ $? -eq 0 ];
	then
		echo "loaded "
	else
		echo "NOT loaded "
	fi;

	echo -n "$KBD driver "
	driver_loaded $KBD
	if [ $? -eq 0 ];
	then
		echo -n "loaded "
	else
		echo -n "NOT loaded "
	fi;

	echo -n "major number: "
	driver_major $KBD
	major=$?
	if [ $major -eq 0 ]; then
		echo "not assigned"
	else
		echo "$major"
	fi;

	echo -n "$MS driver "
	driver_loaded $MS
	if [ $? -eq 0 ];
	then
		echo -n "loaded "
	else
		echo -n "NOT loaded "
	fi;

	echo -n "major number: "
	driver_major $MS
	major=$?
	if [ $major -eq 0 ]; then
		echo "not assigned"
	else
		echo "$major"
	fi;


	return 0
}

case "$1" in
start)
	start
	RETVAL=$?
	;;
stop)
	stop
	RETVAL=$?
	;;
status)
	status
	RETVAL=$?
	;;
*)
	echo $"Usage: $0 {start|stop|status}"
	RETVAL=1
esac

exit $RETVAL
