# Start the video engine driver component
# Note: for a configuration that is not full features (ipmi only)
# the video driver is not loaded
# Startup Dependency: 

RETVAL=0

MODULEDIR="/lib/modules"
VIDEO="aess_video"
DEVDIR="/dev/avct"
FULLFEATURES="true"

#
# full_features () - check for full features configuration
# Note - there is no good way to do this.  A non full features
# configuration (ipmi only) will not include JAVA clients
#
# updates global variable FULLFEATUES
#
full_features() {
	if [ -f "/usr/local/www/software/avctKVM.jar" ] ; then
		return 1;
	fi;

	echo "NOTICE: Not full features configuration - video driver not operational"
	FULLFEATURES="false"
	return 0;
}

#
# driver_loaded () - checks for driver being loaded
# $1 = driver name
#
# returns 0 if loaded, !0 if not loaded
#
driver_loaded() {
	lsmod | grep $1 > /dev/null
	return $?
} 

#
# driver_major () - returns major number assigned to driver
# $1 = driver name
#
# returns major number, 0 if not found in /proc/devices
#
driver_major() {
	major=`cat /proc/devices | grep $1 | cut -f1 -d" "`
	if [ major = "" ] ; then
		major=0
	fi;
	return $major
} 

#
# driver_mknod () - do character device mknod
# $1 = major number
# $2 = minor number
# $3 = device pathname
#
# returns 0 if no errors
#
driver_mknod() {
	mknod $3 c $1 $2 > /dev/null
	if [ $? -ne 0 ] ; then
		echo "failed mknod of $3"
		return 1
	fi;
	return 0
} 

start() {
	
	full_features

	if [ $FULLFEATURES = "false" ] ; then
		return 0
	fi;

	insmod $MODULEDIR/$VIDEO.ko
	driver_loaded $VIDEO
	if [ $? -ne 0 ] ; then
		echo "failed load of $VIDEO"
		return 1
	fi;

	driver_major $VIDEO
	major=$?
	if [ $major -eq 0 ]; then
		echo "failed to find major number for $VIDEO"
		return 1
	fi;

	if [ ! -e $DEVDIR ]; then
        	mkdir $DEVDIR
	fi;

	dev="$DEVDIR/video"
	driver_mknod $major 0 $dev
	if [ $? -ne 0 ] ; then
		return 1
	fi;
	return 0
}

stop() {
	rmmod $VIDEO
	dev="$DEVDIR/video"
	rm -f $dev
	return 0
}

status() {
	echo -n "$VIDEO driver "
	driver_loaded $VIDEO
	if [ $? -eq 0 ];
	then
		echo -n "loaded "
	else
		echo -n "NOT loaded "
	fi;

	echo -n "major number: "
	driver_major $VIDEO
	major=$?
	if [ $major -eq 0 ]; then
		echo "not assigned"
	else
		echo "$major"
	fi;
	return 0
}

case "$1" in
start)
	start
	RETVAL=$?
	;;
stop)
	stop
	RETVAL=$?
	;;
status)
	status
	RETVAL=$?
	;;
*)
	echo $"Usage: $0 {start|stop|status}"
	RETVAL=1
esac

exit $RETVAL
